import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;

import com.mindfusion.diagramming.Action;
import com.mindfusion.diagramming.BehaviorBase;
import com.mindfusion.diagramming.CursorHint;
import com.mindfusion.diagramming.DiagramNode;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.InteractionState;

public class FishboneBehavior extends BehaviorBase
{
	public FishboneBehavior(DiagramView view, DiagramNode backbone)
	{
		super(view);
		this.backbone = backbone;
	}
	
	@Override
	protected InteractionState startDraw(Point2D point, MouseEvent e)
	{
		if (getDiagram().getNodeAt(point, 2) instanceof CauseNode)
		{
			CauseNode cause = (CauseNode)getDiagram().getNodeAt(point, 2);
			
			Point2D.Double handle = (cause.getBounds().getY() < backbone.getBounds().getY()) ? cause.getP2() : cause.getP1();
			if (Math.abs(handle.getX() - point.getX()) <= 2 && Math.abs(handle.getY() - point.getY()) <= 2)
				return new InteractionState(cause, 8, Action.Modify);

			if (canDrawAspectAt(point))
			{
				AspectNode node = new AspectNode(cause);
				node.getPen().setWidth(1);
				return new InteractionState(node, -1, Action.Create);
			}
		}
		
		if (backbone.getBounds().contains(point))
		{
			CauseNode node = new CauseNode(backbone);
			node.getPen().setWidth(1);
			return new InteractionState(node, -1, Action.Create);
		}

		return null;
	}
	
	private boolean canDrawAspectAt(Point2D point)
	{
		if (getDiagram().getNodeAt(point) instanceof CauseNode)
		{
			CauseNode cause = (CauseNode)getDiagram().getNodeAt(point);
			if (cause.getBounds().getY() < backbone.getBounds().getY())
			{
				// above the backbone
				Point2D.Double c = new Point2D.Double(cause.getBounds().getX(), cause.getBounds().getY());
				double dx = point.getX() - c.x;
				double dy = point.getY() - c.y;

				if (Math.abs(dx - dy) < 0.5)
					return true;
			}
			else
			{
				// below the backbone
				Point2D.Double c = new Point2D.Double(cause.getBounds().getMaxX(), cause.getBounds().getY());
				//c.x = cause.getBounds().x + cause.getBounds().width;
				double dx = point.getX() - c.x;
				double dy = - (point.getY() - c.y);

				if (Math.abs(dx - dy) < 0.5)
					return true;
			}
		}

		return false;
	}

	private DiagramNode backbone;
	
	protected CursorHint setMouseCursor(Point2D point, Boolean startInteraction)
	{
		if (getDiagram().getNodeAt(point, 2) instanceof CauseNode)
		{
			CauseNode cause = (CauseNode)getDiagram().getNodeAt(point, 2);
			Point2D handle = (cause.getBounds().getY() < backbone.getBounds().getY()) ? cause.getP2() : cause.getP1();
			
			if (Math.abs(handle.getX() - point.getX()) <= 2 && Math.abs(handle.getY() - point.getY()) <= 2)
			{
				return CursorHint.Move;
			}
		}
		
		if (backbone.getBounds().contains(point) || canDrawAspectAt(point))
			return CursorHint.DrawLink;
		
		return CursorHint.Pointer;
	}
}
