
package com.mindfusion.diagramming.samples.demo;

import java.io.*;
import java.awt.*;
import java.awt.datatransfer.*;
import javax.swing.*;

import com.mindfusion.diagramming.DiagramView;


public class FlowChartTransferHandler extends TransferHandler
{
	private static boolean _dnd = false;
	static Image image;
	static String text;

	private class PictureTransferable implements Transferable
	{
		PictureTransferable(Image image)
		{
			_image = image;
		}

		public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException
		{
			if (!isDataFlavorSupported(flavor))
				throw new UnsupportedFlavorException(flavor);

			return _image;
		}

		public DataFlavor[] getTransferDataFlavors()
		{
			return new DataFlavor[]
			{
				DataFlavor.imageFlavor,
				DataFlavor.stringFlavor
			};
		}

		public boolean isDataFlavorSupported(DataFlavor flavor)
		{
			return DataFlavor.imageFlavor.equals(flavor) ||
				DataFlavor.stringFlavor.equals(flavor);
		}


		private Image _image;
	}



	protected Transferable createTransferable(JComponent c)
	{
		if (c instanceof ImageBox)
			return new PictureTransferable(((ImageBox)c).getImage());
		else
			return new StringSelection(((JList)c).getSelectedValue().toString());
	}


	public boolean importData(JComponent c, Transferable t)
	{
		if (canImport(c, t.getTransferDataFlavors()))
		{
			// Can only drop on the flow chart control
			if (!(c instanceof DiagramView))
				return false;

			DiagramView flowChart = (DiagramView)c;
			image = null;
			text = null;

			if (t.isDataFlavorSupported(DataFlavor.imageFlavor))
			{
				try
				{
					image = (Image)t.getTransferData(DataFlavor.imageFlavor);
					_dnd = true;
					return true;
				}
				catch (UnsupportedFlavorException ufe)
				{
					System.out.println("importData: unsupported data flavor");
				}
				catch (IOException ioe)
				{
					System.out.println("importData: I/O exception");
				}
			}
			else if (t.isDataFlavorSupported(DataFlavor.stringFlavor))
			{
				try
				{
					text = (String)t.getTransferData(DataFlavor.stringFlavor);
					_dnd = true;
					return true;
				}
				catch (UnsupportedFlavorException ufe)
				{
					System.out.println("importData: unsupported data flavor");
				}
				catch (IOException ioe)
				{
					System.out.println("importData: I/O exception");
				}
			}
		}

		return false;
	}

	public int getSourceActions(JComponent c)
	{
		return COPY;
	}

	protected void exportDone(JComponent c, Transferable data, int action)
	{
	}

	public boolean canImport(JComponent c, DataFlavor[] flavors)
	{
		if (!(c instanceof DiagramView))
			return false;

		for (int i = 0; i < flavors.length; i++)
		{
			if (DataFlavor.imageFlavor.equals(flavors[i]))
				return true;

			if (DataFlavor.stringFlavor.equals(flavors[i]))
				return true;
		}

		return false;
	}

	public static boolean isDropped()
	{
		return _dnd;
	}

	public static void setDropped()
	{
		_dnd = false;
	}

	public static Image getBoxImage()
	{
		return image;
	}

	public static String getText()
	{
		return text;
	}
}