package com.mindfusion.diagramming.samples.demo;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.text.html.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
 
import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;

public class MainFrame extends JFrame
{
	/**
	 * Initializes a new instance of the MainFrame class.
	 */
	public MainFrame()
	{
		// Initialize frame
		initializeComponent();

		// Switch the control pane
		showControlPane("Empty");

		_fileDialog = new JFileChooser();

		_random = new Random();

		_leftInBottomOut = new AnchorPattern(
			new AnchorPoint[]
			{
				new AnchorPoint((short)0, (short)50, true, false, MarkStyle.None),
				new AnchorPoint((short)50, (short)100, false, true, MarkStyle.None)
			}, "LeftInBottomOut");

		_decisionInOut = new AnchorPattern(
			new AnchorPoint[]
			{
				new AnchorPoint((short)0, (short)50, true, true, MarkStyle.Circle),
				new AnchorPoint((short)50, (short)100, true, true, MarkStyle.Circle),
				new AnchorPoint((short)100, (short)50, true, true, MarkStyle.Circle),
				new AnchorPoint((short)50, (short)0, true, true, MarkStyle.Circle),
			}, "DecisionInOut");

		// Enable undo
		_flowChart.getUndoManager().setUndoEnabled(true);

		// Load some images
		_frame = getImageRelative("Files/frame.gif");
		_male = getImageRelative("Files/male.png");
		_female = getImageRelative("Files/female.png");

		// Load semitransparent images used in custom drawing
		Image temp = null;
		int w = 0;
		int h = 0;
		Graphics2D g = null;

		temp = getImageRelative("Files/Pattern.png");
		w = temp.getWidth(null);
		h = temp.getHeight(null);
		_tImage = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);

		g = _tImage.createGraphics();
		g.drawImage(temp, 0, 0, null);
		g.dispose();

		temp = getImageRelative("Files/PatternS.png");
		w = temp.getWidth(null);
		h = temp.getHeight(null);
		_sImage = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);

		g = _sImage.createGraphics();
		g.drawImage(temp, 0, 0, null);
		g.dispose();

		for (int i = 0; i < _tImage.getWidth(null); i++)
		{
			for (int j = 0; j < _tImage.getHeight(null); j++)
			{
				int rgb = _tImage.getRGB(i, j);
				rgb = ((0x00ffffff & rgb) | 0x9f000000);
				_tImage.setRGB(i, j, rgb);
			}
		}

		for (int i = 0; i < _sImage.getWidth(null); i++)
		{
			for (int j = 0; j < _sImage.getHeight(null); j++)
			{
				int rgb = _sImage.getRGB(i, j);
				rgb = ((0x00ffffff & rgb) | 0x3f000000);
				_sImage.setRGB(i, j, rgb);
			}
		}

		_shadow = new BufferedImage(100, 100, BufferedImage.TYPE_INT_ARGB);
		for (int i = 0; i < _shadow.getWidth(null); i++)
		{
			for (int j = 0; j < _shadow.getHeight(null); j++)
			{
				double dist =
					Math.sqrt(Math.pow(i - 50, 2) + Math.pow(j - 50, 2));
				dist = dist * 255 / 50;
				if (dist > 255)
				dist = 255;

				_shadow.setRGB(i, j, new Color(0, 0, 0, (int)(255 - dist)).getRGB());
			}
		}

		_netImages = new Image[7];
		_netImages[0] = getImageRelative("Files//pc.png");
		_netImages[1] = getImageRelative("Files//net.png");
		_netImages[2] = getImageRelative("Files//users.png");
		_netImages[3] = getImageRelative("Files//globe.png");
		_netImages[4] = getImageRelative("Files//lan.png");
		_netImages[5] = getImageRelative("Files//home.png");
		_netImages[6] = getImageRelative("Files//print.png");
		
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowOpened(WindowEvent e)
			{
				super.windowOpened(e);
				_emptyControlPane.setSize(300, 200);
				MainFrame.this.doLayout();
				
				// Select the first diagram in the tree
				TreeModel model = _steps.getModel();
				DefaultMutableTreeNode root = (DefaultMutableTreeNode)model.getRoot();
				DefaultMutableTreeNode category = (DefaultMutableTreeNode)root.getChildAt(0);
				DefaultMutableTreeNode sample = (DefaultMutableTreeNode)category.getChildAt(0);
				TreePath path = new TreePath(new Object[] { root, category, sample });
				_steps.getSelectionModel().setSelectionPath(path);
			}
		});
	}

	/**
	 * Initializes window frame and child controls.
	 */
	private void initializeComponent()
	{
		//
		// _menuFileClear
		//
		_menuFileClear = new JMenuItem();
		_menuFileClear.setText("Clear");
		_menuFileClear.setAccelerator(KeyStroke.getKeyStroke(
			KeyEvent.VK_N, InputEvent.CTRL_MASK));
		_menuFileClear.setMnemonic(KeyEvent.VK_C);
		_menuFileClear.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onFileClear(e);
				}
			}
		);

		//
		// _menuFileOpen
		//
		_menuFileOpen = new JMenuItem();
		_menuFileOpen.setText("Open");
		_menuFileOpen.setAccelerator(KeyStroke.getKeyStroke(
			Character.valueOf('O'), InputEvent.CTRL_MASK));
		_menuFileOpen.setMnemonic(KeyEvent.VK_O);
		_menuFileOpen.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onFileOpen(e);
				}
			}
		);

		//
		// _menuFileSave
		//
		_menuFileSave = new JMenuItem();
		_menuFileSave.setText("Save");
		_menuFileSave.setAccelerator(KeyStroke.getKeyStroke(
			Character.valueOf('S'), InputEvent.CTRL_MASK));
		_menuFileSave.setMnemonic(KeyEvent.VK_S);
		_menuFileSave.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onFileSave(e);
				}
			}
		);

		// 
		// _menuFileImport
		//
		_menuFileImport = new JMenuItem();
		_menuFileImport.setText("Load XML");
		_menuFileImport.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onFileImport();
				}
			}
		);
		
		//
		// _menuFileExport
		//
		_menuFileExport = new JMenuItem();
		_menuFileExport.setText("Save XML");
		_menuFileExport.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onFileExport();
				}
			}
		);
		
		//
		// _menuFilePrintPreview
		//
		_menuFilePrintPreview = new JMenuItem();
		_menuFilePrintPreview.setText("Print Preview");
		_menuFilePrintPreview.setMnemonic(KeyEvent.VK_V);
		_menuFilePrintPreview.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onFilePrintPreview(e);
				}
			}
		);

		//
		// _menuFilePrint
		//
		_menuFilePrint = new JMenuItem();
		_menuFilePrint.setText("Print");
		_menuFilePrint.setAccelerator(KeyStroke.getKeyStroke(
			Character.valueOf('P'), InputEvent.CTRL_MASK));
		_menuFilePrint.setMnemonic(KeyEvent.VK_P);
		_menuFilePrint.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onFilePrint(e);
				}
			}
		);

		//
		// _menuFileExit
		//
		_menuFileExit = new JMenuItem();
		_menuFileExit.setText("Exit");
		_menuFileExit.setMnemonic(KeyEvent.VK_X);
		_menuFileExit.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onFileExit(e);
				}
			}
		);

		//
		// _menuEditUndo
		//
		_menuEditUndo = new JMenuItem();
		_menuEditUndo.setText("Undo");
		_menuEditUndo.setAccelerator(KeyStroke.getKeyStroke(
			Character.valueOf('Z'), InputEvent.CTRL_MASK));
		_menuEditUndo.setMnemonic(KeyEvent.VK_U);
		_menuEditUndo.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onEditUndo(e);
				}
			}
		);

		//
		// _menuEditRedo
		//
		_menuEditRedo = new JMenuItem();
		_menuEditRedo.setText("Redo");
		_menuEditRedo.setAccelerator(KeyStroke.getKeyStroke(
			Character.valueOf('Y'), InputEvent.CTRL_MASK));
		_menuEditRedo.setMnemonic(KeyEvent.VK_R);
		_menuEditRedo.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onEditRedo(e);
				}
			}
		);

		//
		// _menuEditCut
		//
		_menuEditCut = new JMenuItem();
		_menuEditCut.setText("Cut");
		_menuEditCut.setAccelerator(KeyStroke.getKeyStroke(
			Character.valueOf('X'), InputEvent.CTRL_MASK));
		_menuEditCut.setMnemonic(KeyEvent.VK_T);
		_menuEditCut.setEnabled(false);
		_menuEditCut.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onEditCut(e);
				}
			}
		);

		//
		// _menuEditCopy
		//
		_menuEditCopy = new JMenuItem();
		_menuEditCopy.setText("Copy");
		_menuEditCopy.setAccelerator(KeyStroke.getKeyStroke(
			Character.valueOf('C'), InputEvent.CTRL_MASK));
		_menuEditCopy.setMnemonic(KeyEvent.VK_C);
		_menuEditCopy.setEnabled(false);
		_menuEditCopy.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onEditCopy(e);
				}
			}
		);

		//
		// _menuEditPaste
		//
		_menuEditPaste = new JMenuItem();
		_menuEditPaste.setText("Paste");
		_menuEditPaste.setAccelerator(KeyStroke.getKeyStroke(
			KeyEvent.VK_P, InputEvent.CTRL_MASK));
		_menuEditPaste.setMnemonic(KeyEvent.VK_P);
		_menuEditPaste.setEnabled(false);
		_menuEditPaste.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onEditPaste(e);
				}
			}
		);

		//
		// _menuObjectDelete
		//
		_menuObjectDelete = new JMenuItem();
		_menuObjectDelete.setText("Delete");
		_menuObjectDelete.setMnemonic(KeyEvent.VK_D);
		_menuObjectDelete.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onObjectDelete(e);
				}
			}
		);

		//
		// _menuObjectGroup
		//
		_menuObjectGroup = new JMenuItem();
		_menuObjectGroup.setText("Group");
		_menuObjectGroup.setAccelerator(KeyStroke.getKeyStroke(
			Character.valueOf('G'), InputEvent.CTRL_MASK));
		_menuObjectGroup.setMnemonic(KeyEvent.VK_G);
		_menuObjectGroup.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onObjectGroup(e);
				}
			}
		);

		//
		// _menuObjectUngroup
		//
		_menuObjectUngroup = new JMenuItem();
		_menuObjectUngroup.setText("Ungroup");
		_menuObjectUngroup.setAccelerator(KeyStroke.getKeyStroke(
			Character.valueOf('U'), InputEvent.CTRL_MASK));
		_menuObjectUngroup.setMnemonic(KeyEvent.VK_U);
		_menuObjectUngroup.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onObjectUngroup(e);
				}
			}
		);

		//
		// _menuObjectUnlockAll
		//
		_menuObjectUnlockAll = new JMenuItem();
		_menuObjectUnlockAll.setText("Unlock All");
		_menuObjectUnlockAll.setMnemonic(KeyEvent.VK_A);
		_menuObjectUnlockAll.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onObjectUnlockAll(e);
				}
			}
		);

		//
		// _menuArrowRouteAll
		//
		_menuArrowRouteAll = new JMenuItem();
		_menuArrowRouteAll.setText("Route All");
		_menuArrowRouteAll.setMnemonic(KeyEvent.VK_R);
		_menuArrowRouteAll.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onArrowRouteAll(e);
				}
			}
		);

		//
		// _menuHelpAbout
		//
		_menuHelpAbout = new JMenuItem();
		_menuHelpAbout.setText("About");
		_menuHelpAbout.setMnemonic(KeyEvent.VK_A);
		_menuHelpAbout.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onHelpAbout(e);
				}
			}
		);

		//
		// _menuFile
		//
		_menuFile = new JMenu();
		_menuFile.setText("File");
		_menuFile.setMnemonic(KeyEvent.VK_F);
		_menuFile.add(_menuFileClear);
		_menuFile.add(_menuFileOpen);
		_menuFile.add(_menuFileSave);
		_menuFile.addSeparator();
		_menuFile.add(_menuFileImport);
		_menuFile.add(_menuFileExport);
		_menuFile.addSeparator();
		_menuFile.add(_menuFilePrintPreview);
		_menuFile.add(_menuFilePrint);
		_menuFile.addSeparator();
		_menuFile.add(_menuFileExit);

		//
		// _menuEdit
		//
		_menuEdit = new JMenu();
		_menuEdit.setText("Edit");
		_menuEdit.setMnemonic(KeyEvent.VK_E);
		_menuEdit.add(_menuEditUndo);
		_menuEdit.add(_menuEditRedo);
		_menuEdit.addSeparator();
		_menuEdit.add(_menuEditCut);
		_menuEdit.add(_menuEditCopy);
		_menuEdit.add(_menuEditPaste);

		//
		// _menuObject
		//
		_menuObject = new JMenu();
		_menuObject.setText("Object");
		_menuObject.setMnemonic(KeyEvent.VK_O);
		_menuObject.add(_menuObjectDelete);
		_menuObject.addSeparator();
		_menuObject.add(_menuObjectGroup);
		_menuObject.add(_menuObjectUngroup);
		_menuObject.addSeparator();
		_menuObject.add(_menuObjectUnlockAll);

		//
		// _menuArrow
		//
		_menuArrow = new JMenu();
		_menuArrow.setText("Arrow");
		_menuArrow.setMnemonic(KeyEvent.VK_A);
		_menuArrow.add(_menuArrowRouteAll);

		//
		// _menuHelp
		//
		_menuHelp = new JMenu();
		_menuHelp.setText("Help");
		_menuHelp.setMnemonic(KeyEvent.VK_H);
		_menuHelp.add(_menuHelpAbout);

		//
		// _menuBar
		//
		_menuBar = new JMenuBar();
		_menuBar.add(_menuFile);
		_menuBar.add(_menuEdit);
		_menuBar.add(_menuObject);
		_menuBar.add(_menuArrow);
		_menuBar.add(_menuHelp);

		//
		// _transferHandler
		//
		_transferHandler = new FlowChartTransferHandler();

		//
		// _flowChart
		//
		_flowChart = new Diagram();
		_flowChart.setShapeHandlesStyle(HandlesStyle.SquareHandles);
		_flowChart.setDefaultShape(com.mindfusion.diagramming.Shape.fromId("Rectangle"));
		_flowChart.addDiagramListener
		(
			new DiagramAdapter()
			{
				public void nodeCreated(NodeEvent e)
				{
					MainFrame.this.onItemCreated(e);
				}

				public void nodeClicked(NodeEvent e)
				{
					MainFrame.this.onItemClicked(e);
				}

				public void nodeDoubleClicked(NodeEvent e)
				{
					MainFrame.this.onItemDblClicked(e);
				}
				
				public void linkDoubleClicked(LinkEvent e)
				{
					MainFrame.this.onLinkDblClicked(e);
				}

				public void drawNode(DrawNodeEvent e)
				{
					MainFrame.this.onDrawItem(e);
				}

				public void clicked(DiagramEvent e)
				{
					MainFrame.this.onDocClicked(e);
				}
			}
		);

		//
		// diagView
		//
		_diagView = new DiagramView(_flowChart);
		_diagView.setTransferHandler(_transferHandler);
		_diagView.addMouseMotionListener(new MouseMotionAdapter(){
			public void mouseMoved(MouseEvent e){
				MainFrame.this.ItemDropped(e);
			}
		});
		
		//
		// _steps
		//
		_steps = new JTree();
		_steps.setModel(createSamplesTree());
		_steps.getSelectionModel().setSelectionMode(
			TreeSelectionModel.SINGLE_TREE_SELECTION);
		_steps.addTreeSelectionListener
		(
			new TreeSelectionListener()
			{
				public void valueChanged(TreeSelectionEvent e)
				{
					MainFrame.this.onStepChanged(e);
				}
			}
		);

		//
		// _stepsPane
		//
		_stepsPane = new JScrollPane(_steps);

		//
		// _info
		//
		_info = new JEditorPane();
		_info.setEditable(false);
		_info.setEditorKit(new HTMLEditorKit());
		_infoPane = new JScrollPane(_info);

		//
		// _sourceCode
		//
		_sourceCode = new JEditorPane();
		_sourceCode.setEditable(false);
		_sourceCode.setEditorKit(new HTMLEditorKit());
		try
		{
			_sourceCode.setPage(getResourceUrl("Files/InfoEmpty.htm"));
		}
		catch (java.io.IOException ex)
		{
			ex.printStackTrace();
		}
		_sourceCodePane = new JScrollPane(_sourceCode);

		//
		// _emptyControlPane
		//
		_emptyControlPane = new JPanel();

		//
		// _btnTakeATour
		//
		_btnTakeATour = new JButton();
		_btnTakeATour.setText("Take a tour");
		_btnTakeATour.setLocation(8, 8);
		_btnTakeATour.setSize(120, 24);
		_btnTakeATour.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnTakeATour(e);
				}
			}
		);

		//
		// _dialogueControlPane
		//
		_dialogueControlPane = new JPanel();
		_dialogueControlPane.setLayout(null);
		_dialogueControlPane.add(_btnTakeATour);

		//
		// _btnCreateEntity
		//
		_btnCreateEntity = new JButton();
		_btnCreateEntity.setText("Create entity");
		_btnCreateEntity.setLocation(8, 8);
		_btnCreateEntity.setSize(120, 24);
		_btnCreateEntity.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnCreateEntity(e);
				}
			}
		);

		//
		// _ermControlPane
		//
		_ermControlPane = new JPanel();
		_ermControlPane.setLayout(null);
		_ermControlPane.add(_btnCreateEntity);

		//
		// _btnAddTable
		//
		_btnAddTable = new JButton();
		_btnAddTable.setText("Add table");
		_btnAddTable.setLocation(8, 8);
		_btnAddTable.setSize(120, 24);
		_btnAddTable.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnAddTable(e);
				}
			}
		);

		//
		// _btnAddField
		//
		_btnAddField = new JButton();
		_btnAddField.setText("Add field");
		_btnAddField.setLocation(8, 40);
		_btnAddField.setSize(120, 24);
		_btnAddField.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnAddField(e);
				}
			}
		);

		//
		// _dbControlPane
		//
		_dbControlPane = new JPanel();
		_dbControlPane.setLayout(null);
		_dbControlPane.add(_btnAddTable);
		_dbControlPane.add(_btnAddField);

		//
		// _btnAnimate
		//
		_btnAnimate = new JButton();
		_btnAnimate.setText("Animate");
		_btnAnimate.setLocation(8, 8);
		_btnAnimate.setSize(120, 24);
		_btnAnimate.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnAnimate(e);
				}
			}
		);

		//
		// _vesselsControlPane
		//
		_vesselsControlPane = new JPanel();
		_vesselsControlPane.setLayout(null);
		_vesselsControlPane.add(_btnAnimate);

		//
		// _lblType
		//
		_lblType = new JLabel();
		_lblType.setText("Type");
		_lblType.setLocation(8, 8);
		_lblType.setSize(76, 24);

		//
		// _cbType
		//
		_cbType = new JComboBox();
		_cbType.setModel(new DefaultComboBoxModel(
			new Object[]
			{
				"Centered",
				"Cascading",
				"Radial"
			}));
		_cbType.setLocation(8, 34);
		_cbType.setSize(130, 20);

		//
		// _lblDirection
		//
		_lblDirection = new JLabel();
		_lblDirection.setText("Direction");
		_lblDirection.setLocation(146, 8);
		_lblDirection.setSize(108, 24);

		//
		// _cbDirection
		//
		_cbDirection = new JComboBox();
		_cbDirection.setModel(new DefaultComboBoxModel(
			new Object[]
			{
				"Top-to-bottom",
				"Bottom-to-top",
				"Left-to-right",
				"Right-to-left"
			}));
		_cbDirection.setLocation(144, 34);
		_cbDirection.setSize(130, 20);

		//
		// _lblArrows
		//
		_lblArrows = new JLabel();
		_lblArrows.setText("Links");
		_lblArrows.setLocation(280, 8);
		_lblArrows.setSize(106, 24);

		//
		// _cbArrows
		//
		_cbArrows = new JComboBox();
		_cbArrows.setModel(new DefaultComboBoxModel(
			new Object[]
			{
				"Straight",
				"Perpendicular-2",
				"Perpendicular-3",
				"Rounded"
			}));
		_cbArrows.setLocation(280, 34);
		_cbArrows.setSize(130, 20);

		//
		// _lblLevelSpacing
		//
		_lblLevelSpacing = new JLabel();
		_lblLevelSpacing.setText("Level spacing");
		_lblLevelSpacing.setLocation(418, 8);
		_lblLevelSpacing.setSize(94, 24);

		//
		// _txtLevelSpacing
		//
		_txtLevelSpacing = new JTextField();
		_txtLevelSpacing.setText("25");
		_txtLevelSpacing.setLocation(522, 10);
		_txtLevelSpacing.setSize(50, 20);

		//
		// _lblItemSpacing
		//
		_lblItemSpacing = new JLabel();
		_lblItemSpacing.setText("Item spacing");
		_lblItemSpacing.setLocation(418, 42);
		_lblItemSpacing.setSize(94, 26);

		//
		// _txtItemSpacing
		//
		_txtItemSpacing = new JTextField();
		_txtItemSpacing.setText("15");
		_txtItemSpacing.setLocation(522, 44);
		_txtItemSpacing.setSize(50, 20);

		//
		// _btnTreeLayout
		//
		_btnTreeLayout = new JButton();
		_btnTreeLayout.setText("Layout");
		_btnTreeLayout.setLocation(580, 8);
		_btnTreeLayout.setSize(120, 24);
		_btnTreeLayout.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnTreeLayout(e);
				}
			}
		);

		//
		// _treeLayoutControlPane
		//
		_treeLayoutControlPane = new JPanel();
		_treeLayoutControlPane.setLayout(null);
		_treeLayoutControlPane.add(_lblType);
		_treeLayoutControlPane.add(_cbType);
		_treeLayoutControlPane.add(_lblDirection);
		_treeLayoutControlPane.add(_cbDirection);
		_treeLayoutControlPane.add(_lblArrows);
		_treeLayoutControlPane.add(_cbArrows);
		_treeLayoutControlPane.add(_lblLevelSpacing);
		_treeLayoutControlPane.add(_txtLevelSpacing);
		_treeLayoutControlPane.add(_lblItemSpacing);
		_treeLayoutControlPane.add(_txtItemSpacing);
		_treeLayoutControlPane.add(_btnTreeLayout);

		//
		// _btnRadialLayout
		//
		_btnRadialLayout = new JButton();
		_btnRadialLayout.setText("Layout");
		_btnRadialLayout.setLocation(8, 8);
		_btnRadialLayout.setSize(120, 24);
		_btnRadialLayout.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnRadialLayout(e);
				}
			}
		);

		//
		// _radialLayoutControlPane
		//
		_radialLayoutControlPane = new JPanel();
		_radialLayoutControlPane.setLayout(null);
		_radialLayoutControlPane.add(_btnRadialLayout);

		//
		// _lblIterations
		//
		_lblIterations = new JLabel();
		_lblIterations.setText("Iterations");
		_lblIterations.setLocation(8, 16);
		_lblIterations.setSize(100, 16);

		//
		// _txtIterations
		//
		_txtIterations = new JTextField();
		_txtIterations.setText("600");
		_txtIterations.setLocation(108, 14);
		_txtIterations.setSize(80, 20);

		//
		// _lblNodeDistance
		//
		_lblNodeDistance = new JLabel();
		_lblNodeDistance.setText("Node distance");
		_lblNodeDistance.setLocation(8, 48);
		_lblNodeDistance.setSize(100, 16);

		//
		// _txtNodeDistance
		//
		_txtNodeDistance = new JTextField();
		_txtNodeDistance.setText("20");
		_txtNodeDistance.setLocation(108, 46);
		_txtNodeDistance.setSize(80, 20);

		//
		// _btnSpringLayout
		//
		_btnSpringLayout = new JButton();
		_btnSpringLayout.setText("Layout");
		_btnSpringLayout.setLocation(196, 8);
		_btnSpringLayout.setSize(120, 24);
		_btnSpringLayout.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnSpringLayout(e);
				}
			}
		);

		//
		// _springLayoutControlPane
		//
		_springLayoutControlPane = new JPanel();
		_springLayoutControlPane.setLayout(null);
		_springLayoutControlPane.add(_lblIterations);
		_springLayoutControlPane.add(_txtIterations);
		_springLayoutControlPane.add(_lblNodeDistance);
		_springLayoutControlPane.add(_txtNodeDistance);
		_springLayoutControlPane.add(_btnSpringLayout);

		_rbVerticalFlowChartLayout = new JRadioButton();
		_rbVerticalFlowChartLayout.setText("Vertical");
		_rbVerticalFlowChartLayout.setBounds(5,5,120, 25);
		_rbVerticalFlowChartLayout.setSelected(true);
		
		_rbHorizontalFlowChartLayout = new JRadioButton();
		_rbHorizontalFlowChartLayout.setText("Horizontal");
		_rbHorizontalFlowChartLayout.setBounds(5, 35, 120, 25);
		_rbHorizontalFlowChartLayout.setSelected(false);

		_bgOrientation = new ButtonGroup();
		_bgOrientation.add(_rbVerticalFlowChartLayout);
		_bgOrientation.add(_rbHorizontalFlowChartLayout);
		
		_chkIgnoreRepeatingLinks = new JCheckBox();
		_chkIgnoreRepeatingLinks.setText("Ignore reppeating links");
		_chkIgnoreRepeatingLinks.setBounds(5, 65, 170, 25);
		_chkIgnoreRepeatingLinks.setSelected(false);
		
		_prevStep = new JButton("<");
		_prevStep.setBounds(185, 5, 25, 25);
		_prevStep.setMargin(new Insets(0,0,0,0));
		_prevStep.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0)
			{ 
				loadFlowchartLayoutFile(
						MainFrame.this._cmBxLoadStep.getSelectedIndex() - 1);
			}
		});
		
		_cmBxLoadStep = new JComboBox();
		_cmBxLoadStep.setBounds(215, 5, 160, 25);
		String dirPath = "com\\mindfusion\\diagramming\\samples\\demo\\Files\\Resources\\Flowcharts\\";
		File dir = new File(dirPath);
		for (File f : dir.listFiles())
			_cmBxLoadStep.addItem(f.getName());
		_cmBxLoadStep.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				changeIndexFlowchartLayout(MainFrame.this._cmBxLoadStep.getSelectedIndex());
			}
		});

		_nextStep = new JButton(">");
		_nextStep.setBounds(380, 5, 25, 25);
		_nextStep.setMargin(new Insets(0,0,0,0));
		_nextStep.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0)
			{ 
				loadFlowchartLayoutFile(
						MainFrame.this._cmBxLoadStep.getSelectedIndex() + 1);
			}
		});
		
		_btnFlowchartLayout = new JButton("Layout");
		_btnFlowchartLayout.setBounds(185, 35, 220, 33);
		_btnFlowchartLayout.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				onBtnFlowchartLayout();
			}
		});
		
		_flowchartLayoutControlPane = new JPanel();
		_flowchartLayoutControlPane.setLayout(null);
		_flowchartLayoutControlPane.add(_rbVerticalFlowChartLayout);
		_flowchartLayoutControlPane.add(_rbHorizontalFlowChartLayout);
		_flowchartLayoutControlPane.add(_chkIgnoreRepeatingLinks);
		_flowchartLayoutControlPane.add(_prevStep);
		_flowchartLayoutControlPane.add(_cmBxLoadStep);
		_flowchartLayoutControlPane.add(_nextStep);
		_flowchartLayoutControlPane.add(_btnFlowchartLayout);

		//
		// _rbVertical
		//
		_rbVertical = new JRadioButton();
		_rbVertical.setText("Vertical");
		_rbVertical.setSelected(true);
		_rbVertical.setLocation(8, 16);
		_rbVertical.setSize(100, 16);

		//
		// _rbHorizontal
		//
		_rbHorizontal = new JRadioButton();
		_rbHorizontal.setText("Horizontal");
		_rbHorizontal.setLocation(8, 48);
		_rbHorizontal.setSize(100, 16);

		//
		// _bgOrientation
		//
		_bgOrientation = new ButtonGroup();
		_bgOrientation.add(_rbVertical);
		_bgOrientation.add(_rbHorizontal);

		//
		// _lblLevelSpacing2
		//
		_lblLevelSpacing2 = new JLabel();
		_lblLevelSpacing2.setText("Level spacing");
		_lblLevelSpacing2.setLocation(132, 16);
		_lblLevelSpacing2.setSize(80, 16);

		//
		// _txtLevelSpacing2
		//
		_txtLevelSpacing2 = new JTextField();
		_txtLevelSpacing2.setText("20");
		_txtLevelSpacing2.setLocation(212, 14);
		_txtLevelSpacing2.setSize(50, 20);

		//
		// _lblNodeSpacing
		//
		_lblNodeSpacing = new JLabel();
		_lblNodeSpacing.setText("Node spacing");
		_lblNodeSpacing.setLocation(132, 48);
		_lblNodeSpacing.setSize(80, 16);

		//
		// _txtNodeSpacing
		//
		_txtNodeSpacing = new JTextField();
		_txtNodeSpacing.setText("30");
		_txtNodeSpacing.setLocation(212, 46);
		_txtNodeSpacing.setSize(50, 20);

		//
		// _chkCompact
		//
		_chkCompact = new JCheckBox();
		_chkCompact.setText("Compact");
		_chkCompact.setLocation(268, 8);
		_chkCompact.setSize(92, 20);

		//
		// _btnLayeredLayout
		//
		_btnLayeredLayout = new JButton();
		_btnLayeredLayout.setText("Layout");
		_btnLayeredLayout.setLocation(368, 8);
		_btnLayeredLayout.setSize(120, 24);
		_btnLayeredLayout.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnLayeredLayout(e);
				}
			}
		);

		//
		// _layeredLayoutControlPane
		//
		_layeredLayoutControlPane = new JPanel();
		_layeredLayoutControlPane.setLayout(null);
		_layeredLayoutControlPane.add(_rbVertical);
		_layeredLayoutControlPane.add(_rbHorizontal);
		_layeredLayoutControlPane.add(_lblLevelSpacing2);
		_layeredLayoutControlPane.add(_txtLevelSpacing2);
		_layeredLayoutControlPane.add(_lblNodeSpacing);
		_layeredLayoutControlPane.add(_txtNodeSpacing);
		_layeredLayoutControlPane.add(_chkCompact);
		_layeredLayoutControlPane.add(_btnLayeredLayout);

		//
		// _lblIterations2
		//
		_lblIterations2 = new JLabel();
		_lblIterations2.setText("Iterations");
		_lblIterations2.setLocation(8, 18);
		_lblIterations2.setSize(80, 16);

		//
		// _txtIterations2
		//
		_txtIterations2 = new JTextField();
		_txtIterations2.setText("1500");
		_txtIterations2.setLocation(88, 16);
		_txtIterations2.setSize(50, 20);

		//
		// _lblGridSize
		//
		_lblGridSize = new JLabel();
		_lblGridSize.setText("Grid size");
		_lblGridSize.setLocation(8, 50);
		_lblGridSize.setSize(80, 16);

		//
		// _txtGridSize
		//
		_txtGridSize = new JTextField();
		_txtGridSize.setText("80");
		_txtGridSize.setLocation(88, 48);
		_txtGridSize.setSize(50, 20);

		//
		// _btnGridLayout
		//
		_btnGridLayout = new JButton();
		_btnGridLayout.setText("Layout");
		_btnGridLayout.setLocation(144, 8);
		_btnGridLayout.setSize(120, 24);
		_btnGridLayout.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnGridLayout(e);
				}
			}
		);

		//
		// _gridLayoutControlPane
		//
		_gridLayoutControlPane = new JPanel();
		_gridLayoutControlPane.setLayout(null);
		_gridLayoutControlPane.add(_lblIterations2);
		_gridLayoutControlPane.add(_txtIterations2);
		_gridLayoutControlPane.add(_lblGridSize);
		_gridLayoutControlPane.add(_txtGridSize);
		_gridLayoutControlPane.add(_btnGridLayout);
		
		//
		// _lbMGP
		//
		_lbMGP = new JLabel();
		_lbMGP.setText("MultipleGraphsPlacement");
		_lbMGP.setBounds(8, 18, 155, 25);
		 
		//
		// _cmBxMGP
		//
		_cmBxMGP = new JComboBox();
		_cmBxMGP.addItem("Horizontal");
		_cmBxMGP.addItem("MinimalArea");
		_cmBxMGP.addItem("Vertical");
		_cmBxMGP.setBounds(165, 18, 120, 25);
		
		//
		// _lbRefine
		//
		_lbRefine = new JLabel();
		_lbRefine.setText("Refine");
		_lbRefine.setBounds(8, 50, 50, 25);
		
		//
		// _cmBxRefine
		//
		_cmBxRefine = new JComboBox();
		_cmBxRefine.addItem("True");
		_cmBxRefine.addItem("False");
		_cmBxRefine.setBounds(165, 50, 120, 25);
		
		//
		// _lbPadding
		//
		_lbPadding = new JLabel();
		_lbPadding.setText("Padding");
		_lbPadding.setBounds(295, 18, 70, 25);
		
		//
		// _txFlPadding
		//
		_txFlPadding = new JTextField();
		_txFlPadding.setBounds(400, 18, 60, 22);
		_txFlPadding.setText("30");
		
		//
		// _lbMargins
		//
		_lbMargins = new JLabel();
		_lbMargins.setText("Margins");
		_lbMargins.setBounds(295, 50, 50, 25);
		
		//
		// _lbMarginX
		//
		_lbMarginX = new JLabel();
		_lbMarginX.setText("X:");
		_lbMarginX.setBounds(355, 50, 15, 22);
		
		//
		// _lbMarginY
		//
		_lbMarginY = new JLabel();
		_lbMarginY.setText("Y:");
		_lbMarginY.setBounds(410, 50, 15, 22);
		
		//
		// _txFlMarginX
		//
		_txFlMarginX = new JTextField();
		_txFlMarginX.setText("15");
		_txFlMarginX.setBounds(370, 50, 35, 22);
		
		//
		// _txFlMarginY
		//
		_txFlMarginY = new JTextField();
		_txFlMarginY.setBounds(425, 50, 35, 22);
		_txFlMarginY.setText("15");
		
		//
		// _btnOrthogonalLayout
		//
		_btnOrthogonalLayout = new JButton();
		_btnOrthogonalLayout.setBounds(480, 18, 160, 22);
		_btnOrthogonalLayout.setText("Orthogonal Layout");
		_btnOrthogonalLayout.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				MainFrame.this.onBtnOrthogonalLayout(e);
			}
		});

		_orthogonalLayoutControlPane = new JPanel();
		_orthogonalLayoutControlPane.setLayout(null);
		_orthogonalLayoutControlPane.add(_lbMGP);
		_orthogonalLayoutControlPane.add(_cmBxMGP);
		_orthogonalLayoutControlPane.add(_lbRefine);
		_orthogonalLayoutControlPane.add(_cmBxRefine);
		_orthogonalLayoutControlPane.add(_lbPadding);
		_orthogonalLayoutControlPane.add(_txFlPadding);
		_orthogonalLayoutControlPane.add(_lbMargins);
		_orthogonalLayoutControlPane.add(_lbMarginX);
		_orthogonalLayoutControlPane.add(_lbMarginY);
		_orthogonalLayoutControlPane.add(_txFlMarginX);
		_orthogonalLayoutControlPane.add(_txFlMarginY);
		_orthogonalLayoutControlPane.add(_btnOrthogonalLayout);

		//
		// _lbMGP2
		//
		_lbMGP2 = new JLabel();
		_lbMGP2.setText("MultipleGraphsPlacement");
		_lbMGP2.setBounds(8, 18, 155, 25);
		 
		//
		// _cmBxMGP2
		//
		_cmBxMGP2 = new JComboBox();
		_cmBxMGP2.addItem("Horizontal");
		_cmBxMGP2.addItem("MinimalArea");
		_cmBxMGP2.addItem("Vertical");
		_cmBxMGP2.setBounds(165, 18, 120, 25);
		
		//
		// _lbRadius
		//
		_lbRadius = new JLabel();
		_lbRadius.setText("Radius");
		_lbRadius.setBounds(8, 50, 50, 25);
		
		//
		// _txFlRadius
		//
		_txFlRadius = new JTextField();
		_txFlRadius.setBounds(165, 50, 120, 25);
		_txFlRadius.setText("60");
		
		//
		// _lbSiftRounds
		//
		_lbSiftRounds = new JLabel();
		_lbSiftRounds.setText("SiftRounds");
		_lbSiftRounds.setBounds(295, 18, 70, 25);
		
		//
		// _txSiftRounds
		//
		_txFlSiftRounds = new JTextField();
		_txFlSiftRounds.setBounds(400, 18, 60, 22);
		_txFlSiftRounds.setText("1");
		
		//
		// _lbMargins2
		//
		_lbMargins2 = new JLabel();
		_lbMargins2.setText("Margins");
		_lbMargins2.setBounds(295, 50, 50, 25);
		
		//
		// _lbMarginX2
		//
		_lbMarginX2 = new JLabel();
		_lbMarginX2.setText("X:");
		_lbMarginX2.setBounds(355, 50, 15, 22);
		
		//
		// _lbMarginY2
		//
		_lbMarginY2 = new JLabel();
		_lbMarginY2.setText("Y:");
		_lbMarginY2.setBounds(410, 50, 15, 22);
		
		//
		// _txFlMarginX2
		//
		_txFlMarginX2 = new JTextField();
		_txFlMarginX2.setText("15");
		_txFlMarginX2.setBounds(370, 50, 35, 22);
		
		//
		// _txFlMarginY2
		//
		_txFlMarginY2 = new JTextField();
		_txFlMarginY2.setBounds(425, 50, 35, 22);
		_txFlMarginY2.setText("15");
		
		//
		// _btnCircularLayout
		//
		_btnCircularLayout = new JButton();
		_btnCircularLayout.setBounds(480, 18, 160, 22);
		_btnCircularLayout.setText("Circular Layout");
		_btnCircularLayout.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				MainFrame.this.onBtnCircularLayout(e);
			}
		});

		_circularLayoutControlPane = new JPanel();
		_circularLayoutControlPane.setLayout(null);
		_circularLayoutControlPane.add(_lbMGP2);
		_circularLayoutControlPane.add(_cmBxMGP2);
		_circularLayoutControlPane.add(_lbRadius);
		_circularLayoutControlPane.add(_txFlRadius);
		_circularLayoutControlPane.add(_lbSiftRounds);
		_circularLayoutControlPane.add(_txFlSiftRounds);
		_circularLayoutControlPane.add(_lbMargins2);
		_circularLayoutControlPane.add(_lbMarginX2);
		_circularLayoutControlPane.add(_lbMarginY2);
		_circularLayoutControlPane.add(_txFlMarginX2);
		_circularLayoutControlPane.add(_txFlMarginY2);
		_circularLayoutControlPane.add(_btnCircularLayout);

		
		//
		// _btnClusteredLayout
		//
		_btnClusteredLayout = new JButton();
		_btnClusteredLayout.setText("Layout");
		_btnClusteredLayout.setLocation(8, 8);
		_btnClusteredLayout.setSize(120, 24);
		_btnClusteredLayout.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnClusteredLayout(e);
				}
			}
		);

		//
		// _custeredLayoutControlPane
		//
		_custeredLayoutControlPane = new JPanel();
		_custeredLayoutControlPane.setLayout(null);
		_custeredLayoutControlPane.add(_btnClusteredLayout);

		//
		// _btnOutgoing
		//
		_btnOutgoing = new JButton();
		_btnOutgoing.setText("Outgoing");
		_btnOutgoing.setLocation(8, 8);
		_btnOutgoing.setSize(120, 24);
		_btnOutgoing.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnOutgoing(e);
				}
			}
		);

		//
		// _btnIncoming
		//
		_btnIncoming = new JButton();
		_btnIncoming.setText("Incoming");
		_btnIncoming.setLocation(8, 44);
		_btnIncoming.setSize(120, 24);
		_btnIncoming.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnIncoming(e);
				}
			}
		);

		//
		// _btnChildren
		//
		_btnChildren = new JButton();
		_btnChildren.setText("Children");
		_btnChildren.setLocation(138, 44);
		_btnChildren.setSize(120, 24);
		_btnChildren.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnChildren(e);
				}
			}
		);

		//
		// _btnParents
		//
		_btnParents = new JButton();
		_btnParents.setText("Parents");
		_btnParents.setLocation(138, 8);
		_btnParents.setSize(120, 24);
		_btnParents.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnParents(e);
				}
			}
		);

		//
		// _btnSuccessors
		//
		_btnSuccessors = new JButton();
		_btnSuccessors.setText("Successors");
		_btnSuccessors.setLocation(268, 8);
		_btnSuccessors.setSize(120, 24);
		_btnSuccessors.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnSuccessors(e);
				}
			}
		);

		//
		// _btnPredecessors
		//
		_btnPredecessors = new JButton();
		_btnPredecessors.setText("Predecessors");
		_btnPredecessors.setLocation(268, 44);
		_btnPredecessors.setSize(120, 24);
		_btnPredecessors.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnPredecessors(e);
				}
			}
		);

		//
		// _graphControlPane
		//
		_graphControlPane = new JPanel();
		_graphControlPane.setLayout(null);
		_graphControlPane.add(_btnOutgoing);
		_graphControlPane.add(_btnIncoming);
		_graphControlPane.add(_btnChildren);
		_graphControlPane.add(_btnParents);
		_graphControlPane.add(_btnSuccessors);
		_graphControlPane.add(_btnPredecessors);

		//
		// _btnFindPath
		//
		_btnFindPath = new JButton();
		_btnFindPath.setText("Find path");
		_btnFindPath.setLocation(8, 8);
		_btnFindPath.setSize(120, 24);
		_btnFindPath.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnFindPath(e);
				}
			}
		);

		//
		// _btnFindAllPaths
		//
		_btnFindAllPaths = new JButton();
		_btnFindAllPaths.setText("Find all paths");
		_btnFindAllPaths.setLocation(8, 44);
		_btnFindAllPaths.setSize(120, 24);
		_btnFindAllPaths.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnFindAllPaths(e);
				}
			}
		);

		//
		// _btnDetectCycle
		//
		_btnDetectCycle = new JButton();
		_btnDetectCycle.setText("Detect cycle");
		_btnDetectCycle.setLocation(138, 8);
		_btnDetectCycle.setSize(120, 24);
		_btnDetectCycle.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnDetectCycle(e);
				}
			}
		);

		//
		// _btnFindCycle
		//
		_btnFindCycle = new JButton();
		_btnFindCycle.setText("Find cycle");
		_btnFindCycle.setLocation(138, 44);
		_btnFindCycle.setSize(120, 24);
		_btnFindCycle.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnFindCycle(e);
				}
			}
		);

		//
		// _btnFindAllCycles
		//
		_btnFindAllCycles = new JButton();
		_btnFindAllCycles.setText("Find all cycles");
		_btnFindAllCycles.setLocation(268, 8);
		_btnFindAllCycles.setSize(120, 24);
		_btnFindAllCycles.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnFindAllCycles(e);
				}
			}
		);

		//
		// _graphPathControlPane
		//
		_graphPathControlPane = new JPanel();
		_graphPathControlPane.setLayout(null);
		_graphPathControlPane.add(_btnFindPath);
		_graphPathControlPane.add(_btnFindAllPaths);
		_graphPathControlPane.add(_btnDetectCycle);
		_graphPathControlPane.add(_btnFindCycle);
		_graphPathControlPane.add(_btnFindAllCycles);

		//
		// _lblObjects
		//
		_lblObjects = new JLabel();
		_lblObjects.setText("Objects");
		_lblObjects.setLocation(8, 10);
		_lblObjects.setSize(60, 20);

		//
		// _cbObjects
		//
		_cbObjects = new JComboBox();
		_cbObjects.setModel(new DefaultComboBoxModel(
			new Object[]
			{
				"Desktop computer",
				"Network computer",
				"Users group",
				"Globe",
				"Local network",
				"Building",
				"Printer"
			}));
		_cbObjects.setLocation(68, 8);
		_cbObjects.setSize(154, 20);

		//
		// _chkAttachText
		//
		_chkAttachText = new JCheckBox();
		_chkAttachText.setText("Attach text");
		_chkAttachText.setSelected(true);
		_chkAttachText.setLocation(8, 44);
		_chkAttachText.setSize(96, 18);

		//
		// _txtAttachText
		//
		_txtAttachText = new JTextField();
		_txtAttachText.setText("");
		_txtAttachText.setLocation(116, 44);
		_txtAttachText.setSize(106, 20);

		//
		// _netSampleControlPane
		//
		_netSampleControlPane = new JPanel();
		_netSampleControlPane.setLayout(null);
		_netSampleControlPane.add(_lblObjects);
		_netSampleControlPane.add(_cbObjects);
		_netSampleControlPane.add(_chkAttachText);
		_netSampleControlPane.add(_txtAttachText);

		//
		// _btnLoadXml
		//
		_btnLoadXml = new JButton();
		_btnLoadXml.setText("Load XML");
		_btnLoadXml.setLocation(8, 8);
		_btnLoadXml.setSize(120, 24);
		_btnLoadXml.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnLoadXml(e);
				}
			}
		);

		//
		// _btnShowSample
		//
		_btnShowSample = new JButton();
		_btnShowSample.setText("Show sample");
		_btnShowSample.setLocation(8, 44);
		_btnShowSample.setSize(120, 24);
		_btnShowSample.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnShowSample(e);
				}
			}
		);

		//
		// _visualXmlControlPane
		//
		_visualXmlControlPane = new JPanel();
		_visualXmlControlPane.setLayout(null);
		_visualXmlControlPane.add(_btnLoadXml);
		_visualXmlControlPane.add(_btnShowSample);

		//
		// _btnAddPerson
		//
		_btnAddPerson = new JButton();
		_btnAddPerson.setText("Add person");
		_btnAddPerson.setLocation(8, 8);
		_btnAddPerson.setSize(120, 24);
		_btnAddPerson.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnAddPerson(e);
				}
			}
		);

		//
		// _genealogyControlPane
		//
		_genealogyControlPane = new JPanel();
		_genealogyControlPane.setLayout(null);
		_genealogyControlPane.add(_btnAddPerson);

		//
		// _image1
		//
		_image1 = new ImageBox();
		_image1.setImage(getImageRelative("Files/car1.png"));
		_image1.setImageAlign(ImageAlign.Fit);
		_image1.setBorder(new LineBorder(SystemColor.control));
		_image1.setLocation(8, 8);
		_image1.setSize(48, 56);
		_image1.setTransferHandler(_transferHandler);
		_image1.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label1
		//
		_label1 = new JLabel();
		_label1.setText("Car1");
		_label1.setHorizontalAlignment(SwingConstants.CENTER);
		_label1.setLocation(8, 64);
		_label1.setSize(48, 16);

		//
		// _image2
		//
		_image2 = new ImageBox();
		_image2.setImage(getImageRelative("Files/car2.png"));
		_image2.setImageAlign(ImageAlign.Fit);
		_image2.setBorder(new LineBorder(SystemColor.control));
		_image2.setLocation(64, 8);
		_image2.setSize(48, 56);
		_image2.setTransferHandler(_transferHandler);
		_image2.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label2
		//
		_label2 = new JLabel();
		_label2.setText("Car2");
		_label2.setHorizontalAlignment(SwingConstants.CENTER);
		_label2.setLocation(64, 64);
		_label2.setSize(48, 16);

		//
		// _image3
		//
		_image3 = new ImageBox();
		_image3.setImage(getImageRelative("Files/car3.png"));
		_image3.setImageAlign(ImageAlign.Fit);
		_image3.setBorder(new LineBorder(SystemColor.control));
		_image3.setLocation(120, 8);
		_image3.setSize(48, 56);
		_image3.setTransferHandler(_transferHandler);
		_image3.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label3
		//
		_label3 = new JLabel();
		_label3.setText("Car3");
		_label3.setHorizontalAlignment(SwingConstants.CENTER);
		_label3.setLocation(120, 64);
		_label3.setSize(48, 16);

		//
		// _image4
		//
		_image4 = new ImageBox();
		_image4.setImage(getImageRelative("Files/carMoving1.png"));
		_image4.setImageAlign(ImageAlign.Fit);
		_image4.setBorder(new LineBorder(SystemColor.control));
		_image4.setLocation(176, 8);
		_image4.setSize(48, 56);
		_image4.setTransferHandler(_transferHandler);
		_image4.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label4
		//
		_label4 = new JLabel();
		_label4.setText("Car4");
		_label4.setHorizontalAlignment(SwingConstants.CENTER);
		_label4.setLocation(176, 64);
		_label4.setSize(48, 16);

		//
		// _image5
		//
		_image5 = new ImageBox();
		_image5.setImage(getImageRelative("Files/carMoving2.png"));
		_image5.setImageAlign(ImageAlign.Fit);
		_image5.setBorder(new LineBorder(SystemColor.control));
		_image5.setLocation(232, 8);
		_image5.setSize(48, 56);
		_image5.setTransferHandler(_transferHandler);
		_image5.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label5
		//
		_label5 = new JLabel();
		_label5.setText("Car5");
		_label5.setHorizontalAlignment(SwingConstants.CENTER);
		_label5.setLocation(232, 64);
		_label5.setSize(48, 16);

		//
		// _image6
		//
		_image6 = new ImageBox();
		_image6.setImage(getImageRelative("Files/carMoving3.png"));
		_image6.setImageAlign(ImageAlign.Fit);
		_image6.setBorder(new LineBorder(SystemColor.control));
		_image6.setLocation(288, 8);
		_image6.setSize(48, 56);
		_image6.setTransferHandler(_transferHandler);
		_image6.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label6
		//
		_label6 = new JLabel();
		_label6.setText("Car6");
		_label6.setHorizontalAlignment(SwingConstants.CENTER);
		_label6.setLocation(288, 64);
		_label6.setSize(48, 16);

		//
		// _image7
		//
		_image7 = new ImageBox();
		_image7.setImage(getImageRelative("Files/tree.png"));
		_image7.setImageAlign(ImageAlign.Fit);
		_image7.setBorder(new LineBorder(SystemColor.control));
		_image7.setLocation(344, 8);
		_image7.setSize(48, 56);
		_image7.setTransferHandler(_transferHandler);
		_image7.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label7
		//
		_label7 = new JLabel();
		_label7.setText("Tree");
		_label7.setHorizontalAlignment(SwingConstants.CENTER);
		_label7.setLocation(344, 64);
		_label7.setSize(48, 16);

		//
		// _image8
		//
		_image8 = new ImageBox();
		_image8.setImage(getImageRelative("Files/person2.png"));
		_image8.setImageAlign(ImageAlign.Fit);
		_image8.setBorder(new LineBorder(SystemColor.control));
		_image8.setLocation(400, 8);
		_image8.setSize(48, 56);
		_image8.setTransferHandler(_transferHandler);
		_image8.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label8
		//
		_label8 = new JLabel();
		_label8.setText("Person1");
		_label8.setHorizontalAlignment(SwingConstants.CENTER);
		_label8.setLocation(400, 64);
		_label8.setSize(48, 16);

		//
		// _image9
		//
		_image9 = new ImageBox();
		_image9.setImage(getImageRelative("Files/person3.png"));
		_image9.setImageAlign(ImageAlign.Fit);
		_image9.setBorder(new LineBorder(SystemColor.control));
		_image9.setLocation(456, 8);
		_image9.setSize(48, 56);
		_image9.setTransferHandler(_transferHandler);
		_image9.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label9
		//
		_label9 = new JLabel();
		_label9.setText("Person2");
		_label9.setHorizontalAlignment(SwingConstants.CENTER);
		_label9.setLocation(456, 64);
		_label9.setSize(48, 16);

		//
		// _image10
		//
		_image10 = new ImageBox();
		_image10.setImage(getImageRelative("Files/personWalking1.png"));
		_image10.setImageAlign(ImageAlign.Fit);
		_image10.setBorder(new LineBorder(SystemColor.control));
		_image10.setLocation(512, 8);
		_image10.setSize(48, 56);
		_image10.setTransferHandler(_transferHandler);
		_image10.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label10
		//
		_label10 = new JLabel();
		_label10.setText("Person3");
		_label10.setHorizontalAlignment(SwingConstants.CENTER);
		_label10.setLocation(512, 64);
		_label10.setSize(48, 16);

		//
		// _image11
		//
		_image11 = new ImageBox();
		_image11.setImage(getImageRelative("Files/personWalking2.png"));
		_image11.setImageAlign(ImageAlign.Fit);
		_image11.setBorder(new LineBorder(SystemColor.control));
		_image11.setLocation(568, 8);
		_image11.setSize(48, 56);
		_image11.setTransferHandler(_transferHandler);
		_image11.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label11
		//
		_label11 = new JLabel();
		_label11.setText("Person4");
		_label11.setHorizontalAlignment(SwingConstants.CENTER);
		_label11.setLocation(568, 64);
		_label11.setSize(48, 16);

		//
		// _image12
		//
		_image12 = new ImageBox();
		_image12.setImage(getImageRelative("Files/personWithBriefcase1.png"));
		_image12.setImageAlign(ImageAlign.Fit);
		_image12.setBorder(new LineBorder(SystemColor.control));
		_image12.setLocation(626, 8);
		_image12.setSize(48, 56);
		_image12.setTransferHandler(_transferHandler);
		_image12.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label12
		//
		_label12 = new JLabel();
		_label12.setText("Person5");
		_label12.setHorizontalAlignment(SwingConstants.CENTER);
		_label12.setLocation(626, 64);
		_label12.setSize(48, 16);

		//
		// _image13
		//
		_image13 = new ImageBox();
		_image13.setImage(getImageRelative("Files/personWithBriefcase2.png"));
		_image13.setImageAlign(ImageAlign.Fit);
		_image13.setBorder(new LineBorder(SystemColor.control));
		_image13.setLocation(684, 8);
		_image13.setSize(48, 56);
		_image13.setTransferHandler(_transferHandler);
		_image13.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label13
		//
		_label13 = new JLabel();
		_label13.setText("Person6");
		_label13.setHorizontalAlignment(SwingConstants.CENTER);
		_label13.setLocation(684, 64);
		_label13.setSize(48, 16);

		//
		// _image14
		//
		_image14 = new ImageBox();
		_image14.setImage(getImageRelative("Files/personWithBriefcase3.png"));
		_image14.setImageAlign(ImageAlign.Fit);
		_image14.setBorder(new LineBorder(SystemColor.control));
		_image14.setLocation(740, 8);
		_image14.setSize(48, 56);
		_image14.setTransferHandler(_transferHandler);
		_image14.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					MainFrame.this.onImagePressed(e);
				}
			}
		);

		//
		// _label14
		//
		_label14 = new JLabel();
		_label14.setText("Person7");
		_label14.setHorizontalAlignment(SwingConstants.CENTER);
		_label14.setLocation(740, 64);
		_label14.setSize(48, 16);

		//
		// _rotationControlPane
		//
		_rotationControlPane = new JPanel();
		_rotationControlPane.setLayout(null);
		_rotationControlPane.add(_image1);
		_rotationControlPane.add(_label1);
		_rotationControlPane.add(_image2);
		_rotationControlPane.add(_label2);
		_rotationControlPane.add(_image3);
		_rotationControlPane.add(_label3);
		_rotationControlPane.add(_image4);
		_rotationControlPane.add(_label4);
		_rotationControlPane.add(_image5);
		_rotationControlPane.add(_label5);
		_rotationControlPane.add(_image6);
		_rotationControlPane.add(_label6);
		_rotationControlPane.add(_image7);
		_rotationControlPane.add(_label7);
		_rotationControlPane.add(_image8);
		_rotationControlPane.add(_label8);
		_rotationControlPane.add(_image9);
		_rotationControlPane.add(_label9);
		_rotationControlPane.add(_image10);
		_rotationControlPane.add(_label10);
		_rotationControlPane.add(_image11);
		_rotationControlPane.add(_label11);
		_rotationControlPane.add(_image12);
		_rotationControlPane.add(_label12);
		_rotationControlPane.add(_image13);
		_rotationControlPane.add(_label13);
		_rotationControlPane.add(_image14);
		_rotationControlPane.add(_label14);

		//
		// _imgDrag
		//
		_imgDrag = new ImageBox();
		_imgDrag.setImage(getImageRelative("Files/drag.jpg"));
		_imgDrag.setImageAlign(ImageAlign.Center);
		_imgDrag.setBorder(new LineBorder(Color.BLACK));
		_imgDrag.setLocation(8, 8);
		_imgDrag.setSize(72, 72);
		_imgDrag.setTransferHandler(_transferHandler);

		//
		// List model
		//
		DefaultListModel lstDragModel = new DefaultListModel();
		lstDragModel.addElement("drag me!");
		lstDragModel.addElement("me too!");
		lstDragModel.addElement("don\'t leave me here!");

		//
		// _lstDrag
		//
		_lstDrag = new JList();
		_lstDrag.setModel(lstDragModel);
		_lstDrag.setDragEnabled(true);
		_lstDrag.setTransferHandler(_transferHandler);
		_lstDrag.setBorder(new LineBorder(Color.BLACK));
		_lstDrag.setLocation(92, 8);
		_lstDrag.setSize(137, 60);

		//
		// _dragDropControlPane
		//
		_dragDropControlPane = new JPanel();
		_dragDropControlPane.setLayout(null);
		_dragDropControlPane.add(_imgDrag);
		_dragDropControlPane.add(_lstDrag);

		//
		// _btnPreserveForm
		//
		_btnPreserveForm = new JButton();
		_btnPreserveForm.setText("Preserve form");
		_btnPreserveForm.setLocation(8, 8);
		_btnPreserveForm.setSize(120, 24);
		_btnPreserveForm.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnPreserveForm(e);
				}
			}
		);

		//
		// _formPreservationControlPane
		//
		_formPreservationControlPane = new JPanel();
		_formPreservationControlPane.setLayout(null);
		_formPreservationControlPane.add(_btnPreserveForm);

		//
		// _btnCreateSpanning
		//
		_btnCreateSpanning = new JButton();
		_btnCreateSpanning.setText("Create");
		_btnCreateSpanning.setLocation(8, 8);
		_btnCreateSpanning.setSize(120, 24);
		_btnCreateSpanning.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					MainFrame.this.onBtnCreateSpanning(e);
				}
			}
		);

		//
		// _spanningControlPane
		//
		_spanningControlPane = new JPanel();
		_spanningControlPane.setLayout(null);
		_spanningControlPane.add(_btnCreateSpanning);

		_btnCreateTreeNode = new JButton("Create TreeNode");
		_btnCreateTreeNode.setBounds(8, 8, 150, 25);
		_btnCreateTreeNode.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0)
			{
				MainFrame.this.onBtnCreateTreeNode();
			}
		});
		
		_spanningControlPane = new JPanel();
		_spanningControlPane.setLayout(null);
		_spanningControlPane.add(_btnCreateTreeNode);

		//
		// _formDesigner
		//
		_formDesigner = new JPanel();
		_formDesigner.setLayout(null);
		
		//
		// _lblFormDesigner
		//
		_lblFormDesigner = new JLabel("Control type:");
		_lblFormDesigner.setBounds(10, 10, 100, 20);
		
		//
		// _cmBxControlType
		//
		_cmBxControlType = new JComboBox(new String[]{"Button", "JTextArea", "JLabel", "JCheckBox", "JList", "JTable"});
		_cmBxControlType.setBounds(10, 40, 100, 20);
		_cmBxControlType.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				cmBxControlTypeChanged();
			}
		});
		
		//
		// _btnFormTest
		//
		_btnTestForm = new JButton("Test form");
		_btnTestForm.setBounds(120, 10, 100, 50);
		_btnTestForm.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				btnTestFormClicked();
			}
		});
		
		_formDesigner.add(_lblFormDesigner);
		_formDesigner.add(_cmBxControlType);
		_formDesigner.add(_btnTestForm);
		
		//
		// _controlPane
		//
		_controlPane = new JPanel();
		_controlPane.setLayout(new CardLayout());
		_controlPane.add(_emptyControlPane, "Empty");
		_controlPane.add(_dialogueControlPane, "Dialogue");
		_controlPane.add(_ermControlPane, "Erm");
		_controlPane.add(_dbControlPane, "Database");
		_controlPane.add(_vesselsControlPane, "Vessels");
		_controlPane.add(_treeLayoutControlPane, "TreeLayout");
		_controlPane.add(_radialLayoutControlPane, "RadialLayout");
		_controlPane.add(_springLayoutControlPane, "SpringLayout");
		_controlPane.add(_layeredLayoutControlPane, "LayeredLayout");
		_controlPane.add(_gridLayoutControlPane, "GridLayout");
		_controlPane.add(_custeredLayoutControlPane, "ClusteredLayout");
		_controlPane.add(_graphControlPane, "Graph");
		_controlPane.add(_graphPathControlPane, "GraphPath");
		_controlPane.add(_netSampleControlPane, "NetSample");
		_controlPane.add(_visualXmlControlPane, "VisualXml");
		_controlPane.add(_genealogyControlPane, "Genealogy");
		_controlPane.add(_rotationControlPane, "Rotation");
		_controlPane.add(_dragDropControlPane, "DragDrop");
		_controlPane.add(_formPreservationControlPane, "PreserveForm");
		_controlPane.add(_spanningControlPane, "SpanningCells");
		_controlPane.add(_orthogonalLayoutControlPane, "OrthogonalLayout");
		_controlPane.add(_circularLayoutControlPane, "CircularLayout");
		_controlPane.add(_spanningControlPane, "TreeViewNodes");
		_controlPane.add(_flowchartLayoutControlPane, "FlowchartLayout");
		_controlPane.add(_formDesigner, "FormDesigner");
		_controlPane.setPreferredSize(new Dimension(300, 200));

		//
		// _flowChartPane
		//
		_flowChartPane = new JScrollPane();
		_flowChartPane.setViewportView(_diagView);

		//
		// _mainTabPane
		//
		_mainTabPane = new JTabbedPane();
		_mainTabPane.add("Diagram", _flowChartPane);
		_mainTabPane.add("Source Code", _sourceCodePane);

		//
		// _sourceCodeSplitPane
		//
		_sourceCodeSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
			_mainTabPane, _controlPane);
		_sourceCodeSplitPane.setResizeWeight(0.75);
		_sourceCodeSplitPane.setDividerSize(4);

		//
		// _bottomSplitPane
		//
		_bottomSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
			_stepsPane, _sourceCodeSplitPane);
		_bottomSplitPane.setDividerLocation(200);
		_bottomSplitPane.setDividerSize(4);

		//
		// _mainSplitPane
		//
		_mainSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
			_infoPane, _bottomSplitPane);
		_mainSplitPane.setDividerLocation(90);
		_mainSplitPane.setDividerSize(4);

		// Initialize the main frame
		setJMenuBar(_menuBar);
		setTitle("MindFusion JDiagram Demo");
		getContentPane().add(_mainSplitPane, BorderLayout.CENTER);

		// Set default size and maximize
		setSize(new Dimension(400, 400));
		setExtendedState(MAXIMIZED_BOTH);

		// Overview
		overview = new Overview();
		overview.setFitAll(true);
		overview.setDimColor(new Color(100, 100, 100, 80));
		overview.setDiagramView(_diagView);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		// overview's frame
		overviewFrame = new JFrame();
		overviewFrame.setTitle("Overview");
		overviewFrame.setSize(230, 230);
		overviewFrame.setAlwaysOnTop(true);
		overviewFrame.setLayout(new java.awt.GridLayout(1, 1));
		overviewFrame.add(overview);

		Point location = new Point((int)screenSize.getWidth() - overviewFrame.getWidth() - 30,
			(int)screenSize.getHeight() - overviewFrame.getHeight() - 30);

		overviewFrame.setLocation(location);

		// Listen to the closing event
		addWindowListener
		(
			new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{
					MainFrame.this.onWindowClosing(e);
				}
			}
		);
	}

	protected void btnTestFormClicked() 
	{
		// Display the form as arranged by the user
		Group group = _flowChart.findGroup("Form");
		if (group == null)
			return;

		ShapeNode formBox = ((ShapeNode)group.getMainItem());
		Rectangle2D bounds = formBox.getBounds();
		Point origin = _diagView.docToDevice(new Point2D.Double(bounds.getX(), bounds.getY()));
		Point stretch = _diagView.docToDevice(new Point2D.Double(bounds.getMaxX(), bounds.getMaxY()));
		Rectangle formSize = new Rectangle(origin.x, origin.y,
			stretch.x - origin.x, stretch.y - origin.y);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		int formX = (int)(screenSize.width /2 - formSize.width / 2);
		int formY = (int)(screenSize.height /2 - formSize.height / 2);
		
		JDialog form = new JDialog(this);
		form.setBounds(formX, formY, formSize.width, formSize.height);
		form.setResizable(false);
		form.setTitle(formBox.getText());
		form.setModal(true);
		Container cp = form.getContentPane();
		cp.setLayout(null);
		
		
		// Traverse the controls and add them to the form
		Point formOrigin = _diagView.docToDevice(new Point2D.Double(
			formBox.getBounds().getX(), formBox.getBounds().getY()));
		for (DiagramNode obj : group.getAttachedNodes())
		{
			if (!(obj instanceof ControlNode))
				continue;

			ControlNode host = (ControlNode)obj;
			Component ctrl = host.getControl();
			if (ctrl != null)
			{
				//host.getControl().setTag = host;
				host.setTag(host.getControl());
				host.setControl(null);

				cp.add(ctrl);
				
				// Adjust control location
				Point hostOrigin = _diagView.docToDevice(new Point2D.Double(
					host.getBounds().getX(), host.getBounds().getY()));
				ctrl.setLocation(new Point(hostOrigin.x - formOrigin.x,
					hostOrigin.y - formOrigin.y));
			}
		}

		form.setVisible(true);

		// Return the controls to the flowchart
//		while (form.Controls.Count > 0)
//		{
//			Control ctrl = form.Controls[0];
//			ControlNode host = (ControlNode)ctrl.Tag;
//			if (host != null)
//				host.Control = ctrl;
//		}
		for (DiagramNode node : _flowChart.getNodes())
		{
			if (!(node instanceof ControlNode))
				continue;
			
			ControlNode controlNode = (ControlNode)node;
			if (!(controlNode.getTag() instanceof Component))
				continue;
			
			Component component = (Component)controlNode.getTag();
			form.remove(component);
			controlNode.setControl(component);
			controlNode.setTag(null);
		}
	}

	protected void cmBxControlTypeChanged()
	{
		Class controlClass = null;
		switch (_cmBxControlType.getSelectedIndex())
		{
		case 0:
			controlClass = JButton.class;
			break;
		case 1:
			controlClass = JTextArea.class;
			break;
		case 2:
			controlClass = JLabel.class;
			break;
		case 3:
			controlClass = JCheckBox.class;
			break;
		case 4:
			controlClass = JList.class;
			break;
		case 5:
			controlClass = JTable.class;
			break;
		}

		try {
			_flowChart.setDefaultControlType(controlClass);
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Toggles the control pane with the specified name.
	 */
	private void showControlPane(String pane)
	{
	    CardLayout layout = (CardLayout)_controlPane.getLayout();
	    layout.show(_controlPane, pane);
	}

	/**
	 * Create the sample steps tree.
	 */
	private DefaultTreeModel createSamplesTree()
	{
		DefaultMutableTreeNode subcategory = null;
		DefaultMutableTreeNode node = new DefaultMutableTreeNode("Samples");

		subcategory = new DefaultMutableTreeNode("Flowchart diagrams");
		subcategory.add(new DefaultMutableTreeNode(
			new Step(0, "Flowchart", "Files/Info0.htm", "Steps/FlowChart.jds", "Files/Code0.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(35, "Lane diagram", "Files/Info35.htm", "Steps/Lanes.jds", "Files/Code35.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(1, "Dialogue chart", "Files/Info1.htm", "Steps/Dialogue.jds", "Files/Code1.htm", "Dialogue")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(2, "Hierarchy chart", "Files/Info2.htm", "Steps/Hierarchy.jds", "Files/Code2.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(31, "Chemistry synthesis tree", "Files/Info31.htm", "Steps/Chemistry.jds", "Files/Code31.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(3, "Classification chart", "Files/Info3.htm", "Steps/Classification.jds", "Files/Code3.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(4, "Genealogy tree", "Files/Info4.htm", "Steps/Genealogy.jds", "Files/Code4.htm", "Genealogy")));
		node.add(subcategory);

		subcategory = new DefaultMutableTreeNode("Table-related diagrams");
		subcategory.add(new DefaultMutableTreeNode(
			new Step(22, "ERM diagram", "Files/Info22.htm", "Steps/Erm.jds", "Files/Code22.htm", "Erm")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(5, "Class-hierarchy diagram", "Files/Info5.htm", "Steps/Classes.jds", "Files/Code5.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(6, "Database relationship diagram", "Files/Info6.htm", "Steps/Database.jds", "Files/Code6.htm", "Database")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(36, "Spanning cells", "Files/Info36.htm", "Steps/Spanning.jds", "Files/Code36.htm", "SpanningCells")));
		node.add(subcategory);

		subcategory = new DefaultMutableTreeNode("Control hosting");
		subcategory.add(new DefaultMutableTreeNode(
				new Step(47, "Form designer", "Files/Info47.htm", "Steps/FormDesigner.xml", "Files/Code47.htm", "FormDesigner")));
		node.add(subcategory);
		
		subcategory = new DefaultMutableTreeNode("Special nodes");
		subcategory.add(new DefaultMutableTreeNode(
				new Step(44, "Tree view nodes", "Files/Info44.htm", "Steps/TreeViewNodes.jds", "Files/Code44.htm", "TreeViewNodes")));
		subcategory.add(new DefaultMutableTreeNode(
				new Step(45, "Containers", "Files/Info45.htm", "Steps/ContainerNodes.jds", "Files/Code45.htm", "ContainerNodes")));
		node.add(subcategory);

		subcategory = new DefaultMutableTreeNode("Links");
		subcategory.add(new DefaultMutableTreeNode(
			new Step(24, "Arrowheads", "Files/Info24.htm", "Steps/ArrowHeads.jds", "Files/Code24.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(23, "Routing", "Files/Info23.htm", "Steps/Routing.jds", "Files/Code23.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(26, "Text orientation", "Files/Info26.htm", "Steps/ArrowText.jds", "Files/Code26.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(25, "Form preservation", "Files/Info25.htm", "Steps/ArrowForm.jds", "Files/Code25.htm", "PreserveForm")));
		node.add(subcategory);

		subcategory = new DefaultMutableTreeNode("Automatic layouts");
		subcategory.add(new DefaultMutableTreeNode(
			new Step(17, "Tree layout", "Files/Info17.htm", "Steps/TreeLayout.jds", "Files/Code17.htm", "TreeLayout")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(21, "Big radial layout", "Files/Info21.htm", "", "Files/Code21.htm", "RadialLayout")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(19, "Spring layout", "Files/Info19.htm", "Steps/SpringLayout.jds", "Files/Code19.htm", "SpringLayout")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(30, "Layered layout", "Files/Info30.htm", "Steps/LayeredLayout.jds", "Files/Code30.htm", "LayeredLayout")));
		subcategory.add(new DefaultMutableTreeNode(
				new Step(46, "Flowchart layout", "Files/Info46.htm", "Files/Resources/Flowcharts/branch2", "Files/Code46.htm", "FlowchartLayout")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(32, "Grid layout", "Files/Info32.htm", "Steps/GridLayout.jds", "Files/Code32.htm", "GridLayout")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(33, "Clustered layout", "Files/Info33.htm", "Steps/ClusteredLayout.jds", "Files/Code33.htm", "ClusteredLayout")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(42, "Orthogonal layout", "Files/Info42.htm", "Steps/OrthogonalLayout.jds", "Files/Code42.htm", "OrthogonalLayout")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(43, "Circular layout", "Files/Info43.htm", "Steps/CircularLayout.jds", "Files/Code43.htm", "CircularLayout")));
		node.add(subcategory);

		subcategory = new DefaultMutableTreeNode("Graph management");
		subcategory.add(new DefaultMutableTreeNode(
			new Step(11, "Basic operations", "Files/Info11.htm", "Steps/GraphBasic.jds", "Files/Code11.htm", "Graph")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(10, "Pathfinding", "Files/Info10.htm", "Steps/Pathfinding.jds", "Files/Code10.htm", "GraphPath")));
		node.add(subcategory);

		subcategory = new DefaultMutableTreeNode("Miscellaneous features");
		subcategory.add(new DefaultMutableTreeNode(
			new Step(37, "Rotation", "Files/Info37.htm", "Steps/Rotation.jds", "Files/Code37.htm", "Rotation")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(20, "Grouping", "Files/Info20.htm", "Steps/Grouping.jds", "Files/Code20.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(13, "Drag and drop", "Files/Info13.htm", "", "Files/Code13.htm", "DragDrop")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(18, "Complex node shapes", "Files/Info18.htm", "", "Files/Code18.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(15, "Owner-drawn objects", "Files/Info15.htm", "Steps/OwnerDraw.jds", "Files/Code15.htm", "")));
		node.add(subcategory);

		subcategory = new DefaultMutableTreeNode("Network sample");
		subcategory.add(new DefaultMutableTreeNode(
			new Step(9, "Ready diagram", "Files/Info9.htm", "Steps/NetSample.jds", "Files/Code9.htm", "")));
		subcategory.add(new DefaultMutableTreeNode(
			new Step(8, "NetSample", "Files/Info8.htm", "", "Files/Code8.htm", "NetSample")));
		node.add(subcategory);

		subcategory = new DefaultMutableTreeNode(
			new Step(12, "Visual XML sample", "Files/Info12.htm", "", "Files/Code12.htm", "VisualXml"));
		node.add(subcategory);

		subcategory = new DefaultMutableTreeNode(
			new Step(16, "Overview", "Files/Info16.htm", "Steps/Overview.jds", "Files/Code16.htm", ""));
		node.add(subcategory);

		return new DefaultTreeModel(node);
 	}

	/**
	 * Handles the WindowClosing event on the main frame.
	 */
	private void onWindowClosing(WindowEvent e)
	{
		// Exit application
		System.exit(0);
	}

	/**
	 * Handles the File.Clear menu event.
	 */
	private void onFileClear(ActionEvent e)
	{
		_flowChart.clearAll();
	}

	/**
	 * Handles the File.Open menu event.
	 */
	private void onFileOpen(ActionEvent e)
	{
		_fileDialog.setDialogTitle("Open");
		if (_fileDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				_flowChart.loadFrom(_fileDialog.getSelectedFile().getAbsolutePath());
			}
			catch (FileNotFoundException ex)
			{
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			catch (IOException ex)
			{
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
		}
	}

	/**
	 * Handles the File.Save menu event.
	 */
	private void onFileSave(ActionEvent e)
	{
		for (DiagramNode node : _flowChart.getNodes())
			if (node instanceof TreeViewNode)
				((TreeViewNode)node).setBrush(new SolidBrush(Color.white));
		
		_fileDialog.setDialogTitle("Save");
		if (_fileDialog.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				_flowChart.saveTo(_fileDialog.getSelectedFile().getAbsolutePath());
			}
			catch (FileNotFoundException ex)
			{
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
			catch (IOException ex)
			{
				JOptionPane.showMessageDialog(this, ex.getMessage());
			}
		}
	}

	/**
	 * Handles the File.Import menu event.
	 */
	private void onFileImport()
	{
		_fileDialog.setDialogTitle("Import");
		if (_fileDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				_flowChart.loadFromXml(_fileDialog.getSelectedFile().getAbsolutePath());
			} catch (FileNotFoundException e)
			{
				e.printStackTrace();
			} catch (IOException e)
			{
				e.printStackTrace();
			} catch (XmlException e)
			{
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Handles the File.Export menu event.
	 */
	private void onFileExport()
	{
		_fileDialog.setDialogTitle("Export");
		if (_fileDialog.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				_flowChart.saveToXml(_fileDialog.getSelectedFile().getAbsolutePath());
			} catch (FileNotFoundException e)
			{
				e.printStackTrace();
			} catch (IOException e)
			{
				e.printStackTrace();
			} catch (XmlException e)
			{
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Handles the File.PrintPreview menu event.
	 */
	private void onFilePrintPreview(ActionEvent e)
	{
		PrintPreview printPreview = new PrintPreview(_diagView);
		printPreview.setVisible(true);
		printPreview.setAlwaysOnTop(true);
	}
	

	/**
	 * Handles the File.Print menu event.
	 */
	private void onFilePrint(ActionEvent e)
	{
		_diagView.print();
	}

	/**
	 * Handles the File.Exit menu event.
	 */
	private void onFileExit(ActionEvent e)
	{
		// Exit application
		System.exit(0);
	}

	/**
	 * Handles the Edit.Undo menu event.
	 */
	private void onEditUndo(ActionEvent e)
	{
		_flowChart.getUndoManager().undo();
	}

	/**
	 * Handles the Edit.Redo menu event.
	 */
	private void onEditRedo(ActionEvent e)
	{
		_flowChart.getUndoManager().redo();
	}

	/**
	 * Handles the Edit.Cut menu event.
	 */
	private void onEditCut(ActionEvent e)
	{
	}

	/**
	 * Handles the Edit.Copy menu event.
	 */
	private void onEditCopy(ActionEvent e)
	{
	}

	/**
	 * Handles the Edit.Paste menu event.
	 */
	private void onEditPaste(ActionEvent e)
	{
	}

	/**
	 * Handles the Object.Delete menu event.
	 */
	private void onObjectDelete(ActionEvent e)
	{
		if (_flowChart.getSelection() != null)
		{
			//while (_flowChart.getSelection().getBoxes().size() > 0)
			//	_flowChart.deleteItem(_flowChart.getSelection().getBoxes().get(0));

			if (!(_flowChart.getSelection().getNodes().isEmpty()))
				_flowChart.getNodes().removeAll(_flowChart.getSelection().getNodes());
			
			//while (_flowChart.getSelection().getArrows().size() > 0)
			//	_flowChart.deleteItem(_flowChart.getSelection().getArrows().get(0));
			if (!(_flowChart.getSelection().getLinks().isEmpty()))
				_flowChart.getLinks().removeAll(_flowChart.getSelection().getLinks());
		}
	}

	/**
	 * Handles the Object.Group menu event.
	 */
	private void onObjectGroup(ActionEvent e)
	{
		DiagramItem item = _flowChart.getActiveItem();

		if (item == null)
			return;

		Group group = _flowChart.getFactory().createGroup(item);
		if (group == null)
			return;

		if (item instanceof DiagramLink)
		{
			DiagramLink a = (DiagramLink)item;
			int segments = a.getSegmentCount() + 1, i = 0;

			for (DiagramNode node : _flowChart.getSelection().getNodes())
			{
				if (node != item)
					group.attachToLinkPoint(node, i++ % segments);
			}
		}
		else
		{
			for (DiagramNode node : _flowChart.getSelection().getNodes())
			{
					group.attachToCorner(node, 0);
			}
		}
	}

	/**
	 * Handles the Object.Ungroup menu event.
	 */
	private void onObjectUngroup(ActionEvent e)
	{
		if (_flowChart.getActiveItem().getSubordinateGroup() == null)
			return;

		_flowChart.destroyGroup(_flowChart.getActiveItem().getSubordinateGroup());
	}

	/**
	 * Handles the Object.UnlockAll menu event.
	 */
	private void onObjectUnlockAll(ActionEvent e)
	{
		for (DiagramItem item : _flowChart.getItems())
			item.setLocked(false);
	}

	/**
	 * Handles the Arrow.RouteAll menu event.
	 */
	private void onArrowRouteAll(ActionEvent e)
	{
		_flowChart.routeAllLinks();
	}

	/**
	 * Handles the Help.About menu event.
	 */
	private void onHelpAbout(ActionEvent e)
	{
		// Display the about dialog
		AboutDialog dlg = new AboutDialog(this);
		dlg.setVisible(true);
	}

	/**
	 * Handles the 'Take a tour' button click.
	 */
	private void onBtnTakeATour(ActionEvent e)
	{
		ShapeNode selected = null;
		boolean hasSelectedBox = false;
		//if (_flowChart.getSelection().getBoxes().size() > 0)
		if (!_flowChart.getSelection().getNodes().isEmpty())
		{
			for (DiagramNode node : _flowChart.getSelection().getNodes())
			{
				if (node instanceof ShapeNode)	
				{
					selected = (ShapeNode)node;
					break;
				}
			}
		}

		if (selected == null)
		{
			JOptionPane.showMessageDialog(this, "Select a box!");
			return;
		}

		// Take a tour
		TourDialog dlg = new TourDialog(this, _flowChart, selected);
		dlg.setVisible(true);
	}

	/**
	 * Handles the 'Create entity' button click.
	 */
	private void onBtnCreateEntity(ActionEvent e)
	{
		AddEntityDialog dlg = new AddEntityDialog(this);
		dlg.setVisible(true);

		if (!dlg.getAccepted())
			return;

		float rowHeight = _flowChart.getTableRowHeight();
		float captionHeight = _flowChart.getTableCaptionHeight();
		float totalHeight = captionHeight + 4 * rowHeight;
		TableNode t = _flowChart.getFactory().createTableNode(5, 5, 90 + 2 * rowHeight, totalHeight);

		t.setCaption(dlg.getEntityName());
		t.setTextPadding(new Thickness(1, 0, 1, 0));
		t.setTag(dlg.getEntityDescription());

		Group g = _flowChart.getFactory().createGroup(t);
		g.setAutoDeleteItems(true);

		// Create the help button
		Rectangle2D.Float rc = t.getBounds();
		ShapeNode btn = _flowChart.getFactory().createShapeNode(rc.x + 6f,
			(rc.y + rc.height) - rowHeight, rowHeight, rowHeight);
		btn.setTransparent(true);
		btn.setImage(getImageRelative("Files/help.gif"));
		btn.setImageAlign(ImageAlign.Center);
		btn.setLocked(true);
		btn.setTag(1);
		g.attachToCorner(btn, 3);

		btn = _flowChart.getFactory().createShapeNode(rc.x + 6f + rowHeight,
			(rc.y + rc.height) - rowHeight, rowHeight, rowHeight);
		btn.setTransparent(true);
		btn.setImage(getImageRelative("Files/props.gif"));
		btn.setImageAlign(ImageAlign.Center);
		btn.setLocked(true);
		btn.setTag(2);
		g.attachToCorner(btn, 3);

		btn = _flowChart.getFactory().createShapeNode((rc.x + rc.width) - 6f - rowHeight,
			(rc.y + rc.height) - rowHeight, rowHeight, rowHeight);
		btn.setTransparent(true);
		btn.setImage(getImageRelative("Files/erase.gif"));
		btn.setImageAlign(ImageAlign.Center);
		btn.setLocked(true);
		btn.setTag(3);
		g.attachToCorner(btn, 3);

		GradientBrush tbrush = new GradientBrush(
			new Color(173, 216, 230), new Color(173, 216, 230), 90);

		t.setBrush(tbrush);
		t.setCaptionBrush(new SolidBrush(new Color(70, 130, 180)));

		t.setRowCount(3);
		t.setColumnCount(4);

		t.setScrollable(false);
		t.setEnabledHandles(AdjustmentHandles.Move);
		t.setCellFrameStyle(CellFrameStyle.None);
		t.setHandlesStyle(HandlesStyle.HatchHandles3);
		t.getColumns().get(0).setWidth(rowHeight);
		t.getColumns().get(3).setWidth(rowHeight);
		t.getColumns().get(1).setWidth(15);
		t.getColumns().get(1).setColumnStyle(ColumnStyle.AutoWidth);
		t.getColumns().get(2).setWidth(15);
		t.getColumns().get(2).setColumnStyle(ColumnStyle.AutoWidth);
		t.setShape(SimpleShape.RoundedRectangle);
		t.setCaptionBrush(new SolidBrush(Color.WHITE));

		AnchorPoint ptIn = new AnchorPoint(
			(short)50, (short)50, true, false, Color.BLACK, (short)0);
		ptIn.setMarkStyle(MarkStyle.None);
		AnchorPoint ptOut = new AnchorPoint(
			(short)50, (short)50, false, true, Color.BLACK, (short)3);
		ptOut.setMarkStyle(MarkStyle.None);
		AnchorPointList points = new AnchorPointList();

		for (int i = 0; i < 3; i++)
		{
			points.clear();

			if (dlg.getInput(i) != null)
			{
				t.getCell(0, i).setImageAlign(ImageAlign.Center);
				t.getCell(0, i).setImage(
					getImageRelative("Files/anchor1.gif"));
				t.getCell(1, i).setText(dlg.getInput(i));
				points.add((AnchorPoint)ptIn.clone());
			}

			if (dlg.getOutput(i) != null)
			{
				t.getCell(3, i).setImageAlign(ImageAlign.Center);
				t.getCell(3, i).setImage(
					getImageRelative("Files/anchor2.gif"));
				t.getCell(2, i).setText(dlg.getOutput(i));
				t.getCell(2, i).setTextFormat(
					new TextFormat(Align.Far, Align.Center));
				points.add((AnchorPoint)ptOut.clone());
			}

			t.getRows().get(i).setAnchorPattern(new AnchorPattern(points, ""));
		}
	}

	/**
	 * Handles the 'Add table' button click.
	 */
	private void onBtnAddTable(ActionEvent e)
	{
		AddTableDialog dlg = new AddTableDialog(this);

		dlg.setVisible(true);

		if (dlg.getAccepted())
		{
			TableNode t = _flowChart.getFactory().createTableNode(10, 10, 75, 90);
			t.setCaption(dlg.getTableName());
			t.redimTable(2, 0);
			t.getColumns().get(0).setWidth(28);
		}
	}

	/**
	 * Handles the 'Add field' button click.
	 */
	private void onBtnAddField(ActionEvent e)
	{
		DiagramNode node = null;
		for (DiagramNode n : _flowChart.getSelection().getNodes())
		{
			if (n instanceof TableNode)
			{
				node = (TableNode)n;
				break;
			}
		}
		
		if (node == null)
		{
			JOptionPane.showMessageDialog(this, "Select the table to add the field to.");
			return;
		}
		
		//Table t = _flowChart.getSelection().getTables().get(0);
		TableNode t = (TableNode)node;
		
		AddFieldDialog dlg = new AddFieldDialog(this);

		dlg.setFieldName("Field");
		dlg.setFieldType(0);

		dlg.setVisible(true);

		if (dlg.getAccepted())
		{
			int r = t.addRow();
			t.getCell(0, r).setText(dlg.getFieldTypeText());
			t.getCell(1, r).setText(dlg.getFieldName());
		}
	}

	/**
	 * Handles the 'Animate' button click.
	 */
	private void onBtnAnimate(ActionEvent e)
	{
	}

	/**
	 * Handles the 'Layout' button click of the tree layout.
	 */
	private void onBtnTreeLayout(ActionEvent e)
	{
		DiagramNode root = null;
		if (!_flowChart.getSelection().getNodes().isEmpty())
			root = _flowChart.getSelection().getNodes().get(0);
		
		
		double levelSpacing = 10;
		try
		{
			levelSpacing = Double.parseDouble(_txtLevelSpacing.getText());
		}
		catch (NumberFormatException ex)
		{
			levelSpacing = 30;
		}

		double itemSpacing = 10;
		try
		{
			itemSpacing = Double.parseDouble(_txtItemSpacing.getText());
		}
		catch (NumberFormatException ex)
		{
			itemSpacing = 30;
		}

		int layoutType = 0;
		if (_cbType.getModel().getSelectedItem() == "Cascading")
		{
			layoutType = 0;
		}
		else if (_cbType.getModel().getSelectedItem() == "Centered")
		{
			layoutType = 1;
		}
		else if (_cbType.getModel().getSelectedItem() == "Radial")
		{
			layoutType = 2;
		}

		int arrowType = 0;
		if (_cbArrows.getModel().getSelectedItem() == "Straight")
		{
			arrowType = 0;
		}
		else if (_cbArrows.getModel().getSelectedItem() == "Perpendicular-2")
		{
			arrowType = 1;
		}
		else if (_cbArrows.getModel().getSelectedItem() == "Perpendicular-3")
		{
			arrowType = 2;
		}
		else if (_cbArrows.getModel().getSelectedItem() == "Rounded")
		{
			arrowType = 3;
		}

		int direction = 0;
		if (_cbDirection.getModel().getSelectedItem() == "Top-to-bottom")
		{
			direction = 0;
		}
		else if (_cbDirection.getModel().getSelectedItem() == "Bottom-to-top")
		{
			direction = 1;
		}
		else if (_cbDirection.getModel().getSelectedItem() == "Left-to-right")
		{
			direction = 2;
		}
		else if (_cbDirection.getModel().getSelectedItem() == "Right-to-left")
		{
			direction = 3;
		}

		TreeLayout layout = new TreeLayout(root,
			layoutType, false, arrowType, direction,
			(float)levelSpacing, (float)itemSpacing, root != null, 30, 30, false);

		layout.arrange(_flowChart);

		_flowChart.resizeToFitItems(5, false);
	}

	/**
	 * Handles the 'Layout' button click of the radial tree layout.
	 */
	private void onBtnRadialLayout(ActionEvent e)
	{
		int depth = 4;

		_flowChart.clearAll();

		if (File.listRoots().length == 0)
			return;

		try
		{
			// Enumerate c:
			int r = 0;
			File root = File.listRoots()[r];
			while (root.getAbsolutePath().startsWith("A") ||
				root.getAbsolutePath().startsWith("a") ||
				root.getAbsolutePath().startsWith("B") ||
				root.getAbsolutePath().startsWith("b"))
				root = File.listRoots()[++r];
			enumDirectoryInfo(root, null, depth);
		}
		catch (Exception ex)
		{
		}

		TreeLayout layout = new TreeLayout(null, TreeLayoutType.Radial,
			false, TreeLayoutLinkType.Straight,
			com.mindfusion.diagramming.jlayout.TreeLayoutDirection.TopToBottom,
			30, 0, false, 15, 15, true);

		layout.arrange(_flowChart);

		_flowChart.resizeToFitItems(5, false);
		if (_flowChart.getNodes().size() > 0)
			_diagView.bringIntoView(_flowChart.getNodes().get(0));
		_diagView.zoomToFit();
	}

	/**
	 * Handles the 'Layout' button click of the spring layout.
	 */
	private void onBtnSpringLayout(ActionEvent e)
	{
		int iters = 600;
		try
		{
			iters = Integer.parseInt(_txtIterations.getText());
		}
		catch (NumberFormatException ex)
		{
			System.out.println(ex.getMessage());
			iters = 600;
		}

		float nodeDistance = 60f;
		try
		{
			nodeDistance = Float.parseFloat(_txtNodeDistance.getText());
		}
		catch (NumberFormatException ex)
		{
			System.out.println(ex.getMessage());
			nodeDistance = 60f;
		}

		com.mindfusion.diagramming.SpringLayout layout =
			new com.mindfusion.diagramming.SpringLayout(iters, nodeDistance, true, true);
		layout.setRepulsionFactor(600);
		layout.arrange(_flowChart);

		_flowChart.resizeToFitItems(5, false);

		// apply some nice colors
		DiagramNodeList nodes = _flowChart.getNodes();
		Rectangle2D.Float rc;

		if (nodes.size() == 0)
			return;

		rc = nodes.get(0).getBounds();
		for (DiagramNode b : nodes)
			Rectangle2D.Float.union(rc, b.getBounds(), rc);

		float h = rc.height;
		Color color;
		for (DiagramNode b : nodes)
		{
			int g = Math.min(255, (200 + (int)(50 * (b.getBounds().y - rc.y) / h)));
			int r = Math.min(255, ((int)(255f * (b.getBounds().y - rc.y) / h)));
			color = new Color(r, g, Math.max(0, 120 - r));
			b.setBrush(new SolidBrush(color));
		}
	}

	/**
	 * Handles the 'Layout' button click of the layered layout.
	 */
	private void onBtnLayeredLayout(ActionEvent e)
	{
		LayeredLayout layout = new LayeredLayout();

		float layerSpacing = 10;
		try
		{
			layerSpacing = Float.parseFloat(_txtLevelSpacing2.getText());
		}
		catch (NumberFormatException ex)
		{
			layerSpacing = 10;
		}

		float nodeSpacing = 20;
		try
		{
			nodeSpacing = Float.parseFloat(_txtNodeSpacing.getText());
		}
		catch (NumberFormatException ex)
		{
			nodeSpacing = 20;
		}

		int orientation = com.mindfusion.diagramming.Orientation.Vertical;
		if (_rbVertical.getModel().isSelected())
			orientation = com.mindfusion.diagramming.jlayout.Orientation.Vertical;
		else
			orientation = com.mindfusion.diagramming.jlayout.Orientation.Horizontal;

		layout.setLayerDistance(layerSpacing);
		layout.setNodeDistance(nodeSpacing);
		layout.setXGap(5);
		layout.setYGap(5);
		layout.setSplitLayers(true);
		layout.setOrientation(orientation);
		layout.setLinksCompactFactor(_chkCompact.getModel().isSelected() ? 0.65f : 1);
		layout.setTimeLimit(10000);

		layout.arrange(_flowChart);

		_flowChart.resizeToFitItems(5, false);
		_diagView.zoomToFit();

		// Color arrows
		Hashtable map = new Hashtable();
		for (DiagramLink a : _flowChart.getLinks())
		{
			float x = 0;
			for (Point2D.Float point : a.getControlPoints())
				x += point.x;

			x /= a.getControlPoints().size();

			map.put(a, new Float(x));
		}

		float minX = Float.MAX_VALUE;
		float maxX = Float.MIN_VALUE;
		Iterator i = map.values().iterator();
		while (i.hasNext())
		{
			float x = ((Float)i.next()).floatValue();
			minX = Math.min(minX, x);
			maxX = Math.max(maxX, x);
		}

		Enumeration en = map.keys();
		while (en.hasMoreElements())
		{
			DiagramLink a = (DiagramLink)en.nextElement();
			float x = ((Float)map.get(a)).floatValue();

			Color c = new Color(
				70 - (int)((x - minX) * 55 / (maxX - minX)),
				0,
				75 + (int)((x - minX) * 105 / (maxX - minX)));

			Pen pen = new Pen(3f, c);
			a.setPen(pen);

			Pen headPen = new Pen(1f, c);
			a.setHeadPen(headPen);
		}

		_diagView.invalidate();
	}

	/**
	 * Handles the 'Layout' button click of the grid layout.
	 */
	private void onBtnGridLayout(ActionEvent e)
	{
		com.mindfusion.diagramming.GridLayout layout =
			new com.mindfusion.diagramming.GridLayout();

		float size = 25;
		try
		{
			size = Float.parseFloat(_txtGridSize.getText());
		}
		catch (NumberFormatException ex)
		{
			size = 25;
		}

		int iter = 1000;
		try
		{
			iter = Integer.parseInt(_txtIterations.getText());
		}
		catch (NumberFormatException ex)
		{
			iter = 1000;
		}

		layout.setGridSize(size);
		layout.setIterations(iter);
		layout.setXGap(5);
		layout.setYGap(5);

		layout.arrange(_flowChart);

		_flowChart.resizeToFitItems(5, false);
		_diagView.zoomToFit();
	}

	/**
	 * Handles the 'Layout' button click of the clustered spring layout.
	 */
	private void onBtnClusteredLayout(ActionEvent e)
	{
		com.mindfusion.diagramming.SpringLayout layout =
			new com.mindfusion.diagramming.SpringLayout(200, 40f, false, true);

		layout.setEnableClusters(true);
		layout.setRepulsionFactor(1500);
		layout.setIterations(1000);

		layout.arrange(_flowChart);

		_flowChart.resizeToFitItems(5, false);
	}
	
	
	/**
	 * Handles the 'Layout' button click of the circular layout.
	 */
	private void onBtnCircularLayout(ActionEvent e)
	{
		com.mindfusion.diagramming.CircularLayout cl = 
			new com.mindfusion.diagramming.CircularLayout();
		
		float radius = 60;
		double x = 15;
		double y = 15;
		int mgpI = MultipleGraphsPlacement.Horizontal;
		int siftRounds;

		mgpI = _cmBxMGP2.getSelectedIndex() == 1 ? MultipleGraphsPlacement.MinimalArea : mgpI;
		mgpI = _cmBxMGP2.getSelectedIndex() == 2 ? MultipleGraphsPlacement.Vertical : mgpI;
		
		try
		{
			radius = Float.parseFloat(_txFlRadius.getText());
		}
		catch (NumberFormatException ex)
		{
			radius = 60;
		}
		
		try
		{
			siftRounds = Integer.parseInt(_txFlSiftRounds.getText());
		}
		catch (NumberFormatException ex)
		{
			siftRounds = 1;
		}

		try
		{
			x = Double.parseDouble(_txFlMarginX2.getText());
		}
		catch (NumberFormatException ex)
		{
			x = 15;
		}
		
		try
		{
			y = Double.parseDouble(_txFlMarginY2.getText());
		}
		catch (NumberFormatException ex)
		{
			y = 15;
		}
		
		cl.getMargins().setSize(x, y);
		cl.setRadius(radius);
		cl.setSiftingRounds(siftRounds);
		cl.setMultipleGraphsPlacement(mgpI);
		
		cl.arrange(_flowChart);
	}

	/**
	 * Handles the 'Layout' button click of the orthogonal layout.
	 */
	private void onBtnOrthogonalLayout(ActionEvent e)
	{
		com.mindfusion.diagramming.OrthogonalLayout orthogonalLayout = 
			new com.mindfusion.diagramming.OrthogonalLayout();
		
		float padding = 20;
		double x = 15;
		double y = 15;
		int mgpI = MultipleGraphsPlacement.Horizontal;
		boolean refine = true;

		mgpI = _cmBxMGP.getSelectedIndex() == 1 ? MultipleGraphsPlacement.MinimalArea : mgpI;
		mgpI = _cmBxMGP.getSelectedIndex() == 2 ? MultipleGraphsPlacement.Vertical : mgpI;
		
		try
		{
			refine = Boolean.parseBoolean(_cmBxRefine.getSelectedItem().toString());
		}
		catch (NumberFormatException ex)
		{
			refine = true;
		}
		
		try
		{
			padding = Float.parseFloat(_txFlPadding.getText());
		}
		catch (NumberFormatException ex)
		{
			padding = 20;
		}

		try
		{
			x = Double.parseDouble(_txFlMarginX.getText());
		}
		catch (NumberFormatException ex)
		{
			x = 15;
		}
		
		try
		{
			y = Double.parseDouble(_txFlMarginY.getText());
		}
		catch (NumberFormatException ex)
		{
			y = 15;
		}
		
		orthogonalLayout.getMargins().setSize(x, y);
		orthogonalLayout.setRefine(refine);
		orthogonalLayout.setPadding(padding);
		orthogonalLayout.setMultipleGraphsPlacement(mgpI);
		
		orthogonalLayout.arrange(_flowChart);
	}
	
	/**
	 * Handles the 'Outgoing' button click.
	 */
	private void onBtnOutgoing(ActionEvent e)
	{
		if (!checkAndClear())
			return;

		if (_flowChart.getSelection().getNodes().isEmpty())
			return;
		
		Pen pen = new Pen(3f, Color.BLUE);
		DiagramNode bb = _flowChart.getSelection().getNodes().get(0);
		for (DiagramLink aa : bb.getOutgoingLinks())
		{
			aa.setPen(pen);
			aa.setHeadPen(pen);
		}

		_diagView.invalidate();
	}

	/**
	 * Handles the 'Incoming' button click.
	 */
	private void onBtnIncoming(ActionEvent e)
	{
		if (!checkAndClear())
			return;

		if (_flowChart.getSelection().getNodes().isEmpty())
			return;
		
		Pen p = new Pen(3f, Color.BLUE);
		DiagramNode bb = _flowChart.getSelection().getNodes().get(0);
		for (DiagramLink aa : bb.getIncomingLinks())
		{
			aa.setPen(p);
			aa.setHeadPen(p);
		}

		_diagView.invalidate();
	}

	/**
	 * Handles the 'Children' button click.
	 */
	private void onBtnChildren(ActionEvent e)
	{
		if (!checkAndClear())
			return;

		if (_flowChart.getSelection().getNodes().isEmpty())
			return;
		
		Pen p = new Pen(3f, Color.BLUE);
		DiagramNode bb = _flowChart.getSelection().getNodes().get(0);
		for (DiagramLink aa : bb.getOutgoingLinks())
		{
			DiagramNode bbb = (DiagramNode)aa.getDestination();
			bbb.setPen(p);
		}

		_diagView.invalidate();
	}

	/**
	 * Handles the 'Parents' button click.
	 */
	private void onBtnParents(ActionEvent e)
	{
		if (!checkAndClear())
			return;

		if (_flowChart.getSelection().getNodes().isEmpty())
			return;
		
		Pen p = new Pen(3f, Color.BLUE);
		DiagramNode bb = _flowChart.getSelection().getNodes().get(0);
		for (DiagramLink aa : bb.getIncomingLinks())
		{
			DiagramNode bbb = ((DiagramNode)aa.getOrigin());
			bbb.setPen(p);
		}

		_diagView.invalidate();
	}

	/**
	 * Handles the 'Successors' button click.
	 */
	private void onBtnSuccessors(ActionEvent e)
	{
		if (!checkAndClear())
			return;

		if (_flowChart.getSelection().getNodes().isEmpty())
			return;
		
		DiagramNode bb = _flowChart.getSelection().getNodes().get(0);
		DiagramNodeList visited = new DiagramNodeList();
		visitSuccessors(bb, visited);

		_diagView.invalidate();
	}

	/**
	 * Handles the 'Predecessors' button click.
	 */
	private void onBtnPredecessors(ActionEvent e)
	{
		if (!checkAndClear())
			return;

		if (_flowChart.getSelection().getNodes().isEmpty())
			return;
		
		DiagramNode bb = _flowChart.getSelection().getNodes().get(0);
		DiagramNodeList visited = new DiagramNodeList();
		visitPredecessors(bb, visited);

		_diagView.invalidate();
	}

	/**
	 * Handles the 'Find path' button click.
	 */
	private void onBtnFindPath(ActionEvent e)
	{
		if (_flowChart.getSelection().getNodes().size() != 2)
		{
			JOptionPane.showMessageDialog(this, "Select exactly two nodes!");
			return;
		}

		// Reset all arrows colors
		Pen black = new Pen(1f, Color.BLACK);
		Brush brush = new SolidBrush(new Color(255, 235, 205));
		for (DiagramLink arrow : _flowChart.getLinks())
		{
			arrow.setPen(black);
			arrow.setHeadPen(black);
			arrow.setBrush(brush);
		}

		Pen red = new Pen(2f, Color.RED);
		PathFinder finder = new PathFinder();
		Path path = finder.findShortestPath(_flowChart,
			_flowChart.getSelection().getNodes().get(0),
			_flowChart.getSelection().getNodes().get(1));
		if (path != null)
		{
			for (DiagramLink arrow : path.getLinks())
			{
				arrow.setPen(red);
				arrow.setHeadPen(red);
			}
		}
		else
		{
			JOptionPane.showMessageDialog(this, "Path not found!");
		}
	}

	/**
	 * Handles the 'Find all paths' button click.
	 */
	private void onBtnFindAllPaths(ActionEvent e)
	{
		if (_flowChart.getSelection().getNodes().size() != 2)
		{
			JOptionPane.showMessageDialog(this, "Select exactly two nodes!");
			return;
		}

		// Reset all arrows colors
		Pen black = new Pen(1f, Color.BLACK);
		Brush brush = new SolidBrush(new Color(255, 235, 205));
		for (DiagramLink arrow : _flowChart.getLinks())
		{
			arrow.setPen(black);
			arrow.setHeadPen(black);
			arrow.setBrush(brush);
		}

		PathFinder finder = new PathFinder();
		PathList paths = finder.findAllPaths(_flowChart,
			_flowChart.getSelection().getNodes().get(0),
			_flowChart.getSelection().getNodes().get(1));
		if (paths.size() > 0)
		{
			JOptionPane.showMessageDialog(this, "Found a total of " +
				new Integer(paths.size()).toString() + " different paths.");

			// Color the paths
			for (Path path : paths)
			{
				Color color = new Color(_random.nextInt(250),
					_random.nextInt(150), _random.nextInt(150));
				Color fill = new Color(color.getRed() / 2,
					color.getGreen() / 2, color.getBlue() / 2, 175);

				Pen pen = new Pen(2f, color);
				Brush fillBrush = new SolidBrush(fill);

				for (DiagramLink arrow : path.getLinks())
				{
					arrow.setPen(pen);
					arrow.setHeadPen(pen);
					arrow.setBrush(fillBrush);
				}
			}
		}
		else
		{
			JOptionPane.showMessageDialog(this, "No path found!");
		}
	}

	/**
	 * Handles the 'Find all paths' button click.
	 */
	private void onBtnDetectCycle(ActionEvent e)
	{
		// Reset all arrows colors
		Pen black = new Pen(1f, Color.BLACK);
		Brush brush = new SolidBrush(new Color(255, 235, 205));
		for (DiagramLink arrow : _flowChart.getLinks())
		{
			arrow.setPen(black);
			arrow.setHeadPen(black);
			arrow.setBrush(brush);
		}

		Pen red = new Pen(2f, Color.RED);
		PathFinder finder = new PathFinder();
		Path path = finder.findCycle(_flowChart);
		if (path != null)
		{
			for (DiagramLink arrow : path.getLinks())
			{
				arrow.setPen(red);
				arrow.setHeadPen(red);
			}
		}
		else
		{
			JOptionPane.showMessageDialog(this, "No cycle found within the graph!");
		}
	}

	/**
	 * Handles the 'Find cycle' button click.
	 */
	private void onBtnFindCycle(ActionEvent e)
	{
		if (_flowChart.getSelection().getNodes().size() != 1)
		{
			JOptionPane.showMessageDialog(this, "Select exactly one node!");
			return;
		}

		// Reset all arrows colors
		Pen black = new Pen(1f, Color.BLACK);
		Brush brush = new SolidBrush(new Color(255, 235, 205));
		for (DiagramLink arrow : _flowChart.getLinks())
		{
			arrow.setPen(black);
			arrow.setHeadPen(black);
			arrow.setBrush(brush);
		}

		PathFinder finder = new PathFinder();
		Path path = finder.findCycle(_flowChart,
			_flowChart.getSelection().getNodes().get(0));
		if (path != null)
		{
			Pen red = new Pen(2f, Color.RED);
			for (DiagramLink arrow : path.getLinks())
			{
				arrow.setPen(red);
				arrow.setHeadPen(red);
			}
		}
		else
		{
			JOptionPane.showMessageDialog(this, "Cycle not found!");
		}
	}

	/**
	 * Handles the 'Find all cycles' button click.
	 */
	private void onBtnFindAllCycles(ActionEvent e)
	{
		// Reset all arrows colors
		Pen black = new Pen(1f, Color.BLACK);
		Brush brush = new SolidBrush(new Color(255, 235, 205));
		for (DiagramLink arrow : _flowChart.getLinks())
		{
			arrow.setPen(black);
			arrow.setHeadPen(black);
			arrow.setBrush(brush);
		}

		PathFinder finder = new PathFinder();
		PathList cycles = finder.findAllCycles(_flowChart);
		if (cycles.size() > 0)
		{
			JOptionPane.showMessageDialog(this, "Found a total of " +
				new Integer(cycles.size()).toString() + " different cycles.");

			for (Path path : cycles)
			{
				Color color = new Color(_random.nextInt(250),
					_random.nextInt(150), _random.nextInt(150));
				Pen pen = new Pen(2f, color);

				for (DiagramLink arrow : path.getLinks())
				{
					arrow.setPen(pen);
					arrow.setHeadPen(pen);
				}
			}
		}
		else
		{
			JOptionPane.showMessageDialog(this, "No cycle found within the graph!");
		}
	}

	/**
	 * Handles the 'Load XML' button click.
	 */
	private void onBtnLoadXml(ActionEvent e)
	{
		if (_fileDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
			loadXmlFile(_fileDialog.getSelectedFile());
	}

	/**
	 * Handles the 'Show sample' button click.
	 */
	private void onBtnShowSample(ActionEvent e)
	{
		java.net.URL url = getResourceUrl("Files//People.xml");

		loadXmlFile(new File(url.getPath()));
	}

	/**
	 * Handles the 'Add person' button click.
	 */
	private void onBtnAddPerson(ActionEvent e)
	{
		AddPersonDialog dlg = new AddPersonDialog(this);

		dlg.show();

		if (dlg.getAccepted())
		{
			ShapeNode b =
				_flowChart.getFactory().createShapeNode(30, 30, 60, 75);
			ShapeNode bb =
				_flowChart.getFactory().createShapeNode(90, 30, 120, 75);

			Group g = _flowChart.getFactory().createGroup(b);
			g.attachToCorner(bb, 1);

			b.setEnabledHandles(AdjustmentHandles.Move);
			b.setImage(dlg.getPersonImage());
			b.setImageAlign(ImageAlign.Fit);
			b.setCustomDraw(CustomDraw.Additional);
			b.setBrush(new SolidBrush(Color.BLACK));
			b.setAnchorPattern(AnchorPattern.fromId("TopInBottomOut"));

			if (dlg.getGender() == 1)
				b.setTag("male");
			else
				b.setTag("female");

			bb.setLocked(true);
			bb.setImage(
				getImageRelative("Files/texture.gif"));
			bb.setImageAlign(ImageAlign.Tile);
			bb.setText(dlg.getPersonName() + "\n" + dlg.getPersonDates());
			bb.setCustomDraw(CustomDraw.Additional);
			bb.setBrush(new SolidBrush(Color.BLACK));
		}
	}

	/**
	 * Handles the 'Preserve form' button click.
	 */
	private void onBtnPreserveForm(ActionEvent e)
	{
		if (_flowChart.getSelection().getLinks().size() == 0)
		{
			JOptionPane.showMessageDialog(this, "Select the arrow first!");
			return;
		}

		DiagramLink arrow = _flowChart.getSelection().getLinks().get(0);
		arrow.setRetainForm(true);
	}
	
	private void onBtnCreateTreeNode()
	{
		TreeViewNode node = new TreeViewNode(_flowChart);
		node.setBounds(0, 0, 20, 20);
		node.setEnableStyledText(true);
		node.setCaptionHeight(5);
		_flowChart.getNodes().add(node);
	}

	/**
	 * Handles the 'Create' button click in the spanning cells step.
	 */
	private void onBtnCreateSpanning(ActionEvent e)
	{
		// Create a spanned table
		TableNode t = _flowChart.getFactory().createTableNode(60, 60, 144, 120);

		t.redimTable(2, 4);

		t.getColumns().get(0).setColumnStyle(ColumnStyle.AutoWidth);
		t.getColumns().get(1).setWidth(30);
		t.getColumns().get(1).setColumnStyle(ColumnStyle.FixedWidth);

		t.getCell(0, 0).setRowSpan(3);
		t.getCell(0, 3).setColumnSpan(2);

		t.getCell(1, 0).setImage(getImageRelative("Files/006.png"));
		t.getCell(1, 1).setImage(getImageRelative("Files/014.png"));
		t.getCell(1, 2).setImage(getImageRelative("Files/019.png"));
		t.getCell(1, 0).setImageAlign(ImageAlign.Center);
		t.getCell(1, 1).setImageAlign(ImageAlign.Center);
		t.getCell(1, 2).setImageAlign(ImageAlign.Center);

		TextFormat align = new TextFormat(Align.Center, Align.Center);

		t.getCell(0, 3).setText("<double-click to edit>");
		t.getCell(0, 3).setTextFormat(align);
	}

	/**
	 * Handles the image clicked event in the 'Rotation' step.
	 */
	private void onImagePressed(MouseEvent e)
	{
		// Deselect all images
		ImageBox[] images = new ImageBox[]
		{
			_image1,
			_image2,
			_image3,
			_image4,
			_image5,
			_image6,
			_image7,
			_image8,
			_image9,
			_image10,
			_image11,
			_image12,
			_image13,
			_image14
		};

		for (ImageBox image : images)
		{
			image.setBorder(new LineBorder(SystemColor.control));
		}

		((ImageBox)e.getSource()).setBorder(new LineBorder(Color.BLACK));
	}

	/**
	 * Handles the FlowChart.ItemCreated event.
	 */
	private void onItemCreated(NodeEvent e)
	{
		// Check which step is selected
		Object selected = _steps.getSelectionModel().getSelectionPath().getLastPathComponent();
		if (selected == null)
			return;

		if (!(selected instanceof DefaultMutableTreeNode))
			return;

		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)selected;
		Step step = (Step)selectedNode.getUserObject();

		switch (step.getKey())
		{

			case 3:
			{
				if (e.getNode() instanceof ShapeNode)
				{
					((ShapeNode)e.getNode()).setAnchorPattern(
						AnchorPattern.fromId("TopInBottomOut"));
				}
			}
			break;

			case 5:
			{
				if (e.getNode() instanceof TableNode)
				{
					CompositeCmd cmd = _flowChart.getUndoManager().startComposite("id");
					TableNode table = (TableNode)e.getNode();

					table.setOffsetHeaderRows(true);
					table.redimTable(2, 2);

					table.getCell(0, 0).setColumnSpan(2);
					table.getCell(0, 0).setText("<b>Methods</b>");
					table.getRows().get(0).setHeader(true);
					table.getCell(0, 1).setColumnSpan(2);
					table.getCell(0, 1).setText("<b>Properties</b>");
					table.getRows().get(1).setHeader(true);

					table.setCaption("<b>Class</b>");

					table.setAnchorPattern(AnchorPattern.fromId("TopInBottomOut"));

					cmd.execute();
				}
			}
			break;

			case 23:
			{
				if (e.getNode() instanceof ShapeNode)
				{
					((ShapeNode)e.getNode()).setAnchorPattern(
						AnchorPattern.fromId("DecisionInOut"));
				}
			}
			break;
			
			case 18:
			{
				_flowChart.setMeasureUnit(GraphicsUnit.Millimeter);
			}
			
			case 47:
			{
				if (e.getNode() instanceof ControlNode)
				{
					ControlNode controlNode = (ControlNode)e.getNode();
					if (controlNode.getControl() instanceof JButton)
					{
						JButton ctrl = (JButton)controlNode.getControl();
						ctrl.setText("Button");
					}
					else if (controlNode.getControl() instanceof JTextArea)
					{
						JTextArea ctrl = (JTextArea)controlNode.getControl();
						ctrl.setText("Sample text...");
						controlNode.setMouseInputMode(MouseInputMode.HandledByHostedControl);
					}
					else if (controlNode.getControl() instanceof JLabel)
					{
						JLabel ctrl = (JLabel)controlNode.getControl();
						ctrl.setText("Label");
					}
					else if (controlNode.getControl() instanceof JCheckBox)
					{
						JCheckBox ctrl = (JCheckBox)controlNode.getControl();
						ctrl.setText("Check box");
					}
					else if (controlNode.getControl() instanceof JList)
					{
						JList ctrl = (JList)controlNode.getControl();
						ctrl.setListData(new String[]{"President Monti", "King Ming", "Tsar Blair"});
					}
					else if (controlNode.getControl() instanceof JTable)
					{
						JTable ctrl = (JTable)controlNode.getControl();
						ctrl.setModel(new PropertyGridModel());
					}
					
					// Assign the control to the form group
					Group formGroup = _flowChart.findGroup("Form");
					if (formGroup != null)
					{
						formGroup.attachToCorner(e.getNode(), 0);
					}
				}
			}
		}
	}
	
	private void onDeleteTreeViewNode()
	{
		int result = JOptionPane.showConfirmDialog(MainFrame.this, "Are you sure you want to remove " + treeViewItem.getLabel(),
				"Confirm Delete", JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
		
		if (result == JOptionPane.CANCEL_OPTION)
			return;
		
//#if !STANDARD
		CompositeCmd ccmd = _flowChart.getUndoManager().startComposite("Deleting");
		ChangeItemCmd cmd = new ChangeItemCmd(treeViewNode, "Deleting");
//#endif
		//treeNode.DestroyItem(item);
		treeViewNode.removeItem(treeViewItem);
//#if !STANDARD
		cmd.execute();
		ccmd.execute();
	}
	
	private void onAddTreeViewItem()
	{
		AddTreeViewItemDialog form = new AddTreeViewItemDialog(this);
		form.setVisible(true);
		if (form.getAccepted())
		{
			TreeViewItem child = new TreeViewItem();
			child.setHeight(5);
			child.setLabel(form.getItemLabel());
			child.setImage(form.getItemImage());
			child.setLabelFont(treeViewNode.getFont());
			child.setLabelBrush(new SolidBrush(form.getItemColor()));
			//child.setTag(String.valueOf(form.getItemImageIndex()));
			if (treeViewItem != null)
				treeViewItem.getChildren().add(child);
			else
				treeViewNode.getRootItems().add(child);
		}
	}
	
	private TreeViewItem treeViewItem = null;
	private TreeViewNode treeViewNode = null;

	/**
	 * Handles the FlowChart.ItemClicked event.
	 */
	private void onItemClicked(NodeEvent e)
	{
		// Check which step is selected
		Object selected = _steps.getSelectionModel().getSelectionPath().getLastPathComponent();
		if (selected == null)
			return;

		if (!(selected instanceof DefaultMutableTreeNode))
			return;

		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)selected;
		Step step = (Step)selectedNode.getUserObject();

		switch (step.getKey())
		{
			case 44 : //treeviewnodes
			
				if (!(e.getNode() instanceof TreeViewNode))
					return;
				
				TreeViewNode treeNode = (TreeViewNode)e.getNode();
				treeViewNode = treeNode;
				// Hit-test the clicked item
				TreeViewItem item = treeNode.itemFromPoint(e.getMousePosition());
				treeViewItem = item;

				if (e.getMouseButton() == MouseEvent.BUTTON3)
				{
					//ContextMenu menu = new ContextMenu();
					JPopupMenu menu = new JPopupMenu();
					JMenuItem mAdd = new JMenuItem("Add");
					mAdd.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent arg0)
						{
							onAddTreeViewItem();
						}
					});
					menu.add(mAdd);

					if (item != null)
					{
						JMenuItem mDelete = new JMenuItem("Delete...");
						mDelete.addActionListener(new ActionListener(){
							public void actionPerformed(ActionEvent arg0)
							{
								onDeleteTreeViewNode();
							}
						});

						menu.add(mDelete);
					}
					
					Point p =  getMousePosition();
					menu.show(this, p.x, p.y);
				}
				
			break;
			
			case 5: // Class diagram
			{
				if (e.getMouseButton() == MouseEvent.BUTTON3)
				{
					if (e.getNode() instanceof TableNode)
					{
						TableNode table = (TableNode)e.getNode();
						ClassInfoDialog dlg = new ClassInfoDialog(this);

						dlg.setEditedClass(table);

						dlg.setVisible(true);

						if (!dlg.getAccepted())
							return;

						// Distribute icons
						int header = 0;
						for (int i = 0; i < table.getRowCount(); i++)
						{
							if (table.getRows().get(i).getHeader())
							{
								header++;
								continue;
							}

							if (header == 1)
							{
								table.getCell(0, i).setImage(
									getImageRelative("Files/method.png"));
							}
							else
							{
								table.getCell(0, i).setImage(
									getImageRelative("Files/property.png"));
							}
						}
					}
				}
			}
				break;
		
			case 22: // ERM
			{
				for(Group group : _flowChart.getGroups())
				{
					group.setAutoDeleteItems(true);
				}

				if (e.getNode() instanceof ShapeNode)
				{
					ShapeNode box = (ShapeNode)e.getNode();

					if (box.getTag() != null)
					{
						if (box.getTag() instanceof Integer)
						{
							if (box.getMasterGroup() == null)
								return;

							int tag = ((Integer)box.getTag()).intValue();

							if (tag == 1)
							{
								// Display info box
								TableNode t = (TableNode)box.getMasterGroup().getMainItem();

								if (t.getTag() == null)
									return;
								if (!(t.getTag() instanceof String))
									return;
								if (((String)t.getTag()).length() == 0)
									return;

								Graphics2D g = (Graphics2D)_diagView.getGraphics();
								FontMetrics metrics = g.getFontMetrics();
								Rectangle2D.Float bounds = (Rectangle2D.Float)metrics.getStringBounds((String)t.getTag(), g);

								ShapeNode b = (ShapeNode)_flowChart.findNode(new Integer(20));
								if (b == null)
								{
									b = _flowChart.getFactory().createShapeNode(0, 0, 10, 10);
									b.setTag(new Integer(20));
									b.setPen(new Pen(0.6f, new Color(192, 192, 192)));
									b.setBrush(new SolidBrush(new Color(192, 192, 192, 100)));
									b.setTextBrush(new SolidBrush(Color.BLACK));
									b.setHandlesStyle(HandlesStyle.Invisible);
									b.setLocked(true);
									b.setFont(new Font("Times New Roman", Font.PLAIN, 8));
									b.setShadowOffsetX(-10000);
									b.setShadowOffsetY(-10000);
								}

								Rectangle2D.Float rc = new Rectangle2D.Float();
								rc.x = t.getBounds().x + t.getBounds().width - bounds.width;
								rc.y = t.getBounds().y + t.getBounds().height;
								rc.width = bounds.width + 20;
								rc.height = bounds.height + 5;

								b.setBounds(rc);
								b.setVisible(true);
								b.setText((String)t.getTag());

								_flowChart.getSelection().clear();
							}
							else if (tag == 2)
							{
								TableNode t = (TableNode)box.getMasterGroup().getMainItem();

								int ain = 0;
								int aout = 0;
								for (Row r : t.getRows())
								{
									ain += r.getIncomingLinks().size();
									aout += r.getOutgoingLinks().size();
								}

								String msg = t.getCaption() + "\n\nIncoming links: " +
									new Integer(ain).toString() +
									"\nOutgoing links: " + new Integer(aout).toString();
								JOptionPane.showMessageDialog(this, msg);
							}
							else if (tag == 3)
							{
								if (JOptionPane.showConfirmDialog(this,
									"Are you sure you want to delete this entity?",
									"Confirm delete", JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION)
									return;

								TableNode t = (TableNode)box.getMasterGroup().getMainItem();
								while (t.getSubordinateGroup().getAttachedNodes().size() > 0)
									_flowChart.getNodes().remove(t.getSubordinateGroup().getAttachedNodes().get(0));
								_flowChart.getNodes().remove(t);
							}
						}
					}
				}
			}
				break;

			case 36: // Spanning cell
			{
				if (e.getMouseButton() == MouseEvent.BUTTON1)
				{
					if (e.getNode() instanceof TableNode)
					{
						TableNode table = (TableNode)e.getNode();
						int column = 0;
						int row = 0;

						// Calculate clicked row and column
						Point2D.Float position = new Point2D.Float();
						position.setLocation(e.getMousePosition());
						position.x -= table.getBounds().x;
						position.y -= table.getBounds().y;

						float x = position.x;
						for (int i = 0; i < table.getColumnCount(); i++)
						{
							column = i;

							x -= table.getColumns().get(i).getWidth();
							if (x <= 0)
								break;
						}

						float y = position.y;
						for (int j = 0; j < table.getRowCount(); j++)
						{
							row = j;

							y -= table.getRows().get(j).getHeight();
							if (y <= 0)
								break;
						}

						if (column == 0)
						{
							if (row == 0 || row == 1 || row == 2)
							{
								// Pick image
								if (_fileDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
								{
									BufferedImage image = null;
									try
									{
										image = javax.imageio.ImageIO.read(
			        						new java.io.File(_fileDialog.getSelectedFile().getAbsolutePath()));
									}
									catch (IOException ex)
									{
										JOptionPane.showMessageDialog(this, "Failed to load image.");
										return;
									}

									table.getCell(0, 0).setImage(image);
								}
							}
						}
						else if (column == 1)
						{
							switch (row)
							{

								case 0: // person
									JOptionPane.showMessageDialog(this, "Personal data");
									break;

								case 1: // credit card
									JOptionPane.showMessageDialog(this, "Bank account");
									break;

								case 2: // car
									JOptionPane.showMessageDialog(this, "Property");
									break;

							}
						}
					}
				}
			}
				break;

		}
	}
	
	/**
	 * Handles the FlowChart.ItemDblClicked event.
	 */
	private void onLinkDblClicked(LinkEvent e)
	{
		/*
		 * TODO: remove this part
		 */
		DiagramLink arrow = e.getLink();
		String newText = JOptionPane.showInputDialog(
			this, "Enter arrow text", arrow.getText());
		arrow.setText(newText);
		/*
		 **********************************
		 */
	}

	/**
	 * Handles the FlowChart.ItemDblClicked event.
	 */
	private void onItemDblClicked(NodeEvent e)
	{
		/*
		 * TODO: remove this part
		 */
		if (e.getNode() instanceof ShapeNode)
		{
			ShapeNode box = (ShapeNode)e.getNode();
			String newText = JOptionPane.showInputDialog(
				this, "Enter box text", box.getText());
			box.setText(newText);
		}
		/*
		 **********************************
		 */

		// Check which step is selected
		Object selected = _steps.getSelectionModel().getSelectionPath().getLastPathComponent();
		if (selected == null)
			return;

		if (!(selected instanceof DefaultMutableTreeNode))
			return;

		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)selected;
		Step step = (Step)selectedNode.getUserObject();

		switch (step.getKey())
		{

			case 6: // Database relationship diagram
			{
				if (!(e.getNode() instanceof TableNode))
					return;

				TableNode table = (TableNode)e.getNode();

				// Get the relative position
				Point2D position = e.getMousePosition();
				double x = e.getMousePosition().getX() - table.getBounds().x;
				double y = e.getMousePosition().getY() - table.getBounds().y;
				position.setLocation(x, y);
				
				
				// Check if the caption is clicked or a row is clicked
				if (position.getY() < table.getCaptionHeight())
				{
					// The caption is clicked -> Expand / Collapse table
					boolean expanded = false;
					if (table.getTag() == null)
					{
						expanded = true;
					}
					else if (table.getTag() instanceof Float)
					{
						if (((Float)table.getTag()).floatValue() == 0)
							expanded = true;
					}
					else if (!(table.getTag() instanceof Float))
					{
						expanded = true;
					}

					if (expanded)
					{
						table.setTag(new Float(table.getBounds().height));
						Rectangle2D.Float rc = (Rectangle2D.Float)table.getBounds().clone();
						rc.height = table.getCaptionHeight();
						table.setBounds(rc);
					}
					else
					{
						Rectangle2D.Float rc = (Rectangle2D.Float)table.getBounds().clone();
						rc.height = ((Float)table.getTag()).floatValue();
						table.setBounds(rc);
						table.setTag(null);
					}
				}
				else
				{
					// Some row is clicked -> find out which one
					int first = table.getCurrentRow();
					int r = (int)((e.getMousePosition().getY() - table.getCaptionHeight()) /
						table.getRowHeight()) + first;

					if (r < table.getRows().size())
					{
						AddFieldDialog dlg = new AddFieldDialog(this);

						dlg.setTitle("Modify Field");
						dlg.setFieldTypeText(table.getCell(0, r).getText());
						dlg.setFieldName(table.getCell(1, r).getText());
						dlg.setAcceptBtnText("Accept");

						dlg.setVisible(true);

						if (dlg.getAccepted())
						{
							table.getCell(0, r).setText(dlg.getFieldTypeText());
							table.getCell(1, r).setText(dlg.getFieldName());
						}
					}
				}
			}
			break;

		}
	}

	/**
	 * Handles the FlowChart.DocClicked event.
	 */
	private void onDocClicked(DiagramEvent e)
	{
		// Check which step is selected
		Object selected = _steps.getSelectionModel().getSelectionPath().getLastPathComponent();
		if (selected == null)
			return;

		if (!(selected instanceof DefaultMutableTreeNode))
			return;

		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)selected;
		Step step = (Step)selectedNode.getUserObject();

		switch (step.getKey())
		{

			case 8: // Net sample
			{
				if (e.getMouseButton() == MouseEvent.BUTTON3)
				{
					_flowChart.getSelection().clear();
				}
				else if (e.getMouseButton() == MouseEvent.BUTTON1)
				{
					// Create the specified box
					int image = _cbObjects.getSelectedIndex();
					if (image != -1)
					{
						ShapeNode b =
							_flowChart.getFactory().createShapeNode(
									e.getMousePosition().x, e.getMousePosition().y, 1, 1);

						b.setImage(_netImages[image]);
						b.resizeToFitImage();
						b.setTransparent(true);
						b.setEnabledHandles(AdjustmentHandles.Move);

						if (_chkAttachText.isSelected() && _txtAttachText.getText().length() != 0)
						{
							ShapeNode bb = _flowChart.getFactory().createShapeNode(
								e.getMousePosition().x - 30, e.getMousePosition().y + b.getBounds().height,
								b.getBounds().width + 60, 15);

							bb.setLocked(true);
							bb.setTransparent(true);
							bb.setText(_txtAttachText.getText());

							Group g = _flowChart.getFactory().createGroup(b);
							g.attachToCorner(bb, 3);
						}
					}
				}
			}
				break;

			case 22: // ERM
			{
				// Hide the info box (if visible)
				DiagramNode b = _flowChart.findNode(new Integer(20));
				if (b != null)
					b.setVisible(false);
			}
				break;

		}
	}

	/**
	 * Handles the FlowChart.DrawItem event.
	 */
	private void onDrawItem(DrawNodeEvent e)
	{
		if (e.getNode() instanceof ShapeNode)
		{
			ShapeNode box = (ShapeNode)e.getNode();
			Rectangle2D bounds = e.getBounds();
			Graphics2D g = e.getGraphics();
			Rectangle b = new Rectangle((int)bounds.getX(), (int)bounds.getY(),
				(int)bounds.getWidth(), (int)bounds.getHeight());

			// Check which step is selected
			Object selected = _steps.getSelectionModel().getSelectionPath().getLastPathComponent();
			if (selected == null)
				return;

			if (!(selected instanceof DefaultMutableTreeNode))
				return;

			DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)selected;
			Step step = (Step)selectedNode.getUserObject();

			switch (step.getKey())
			{

				case 4: // Genealogy tree
				case 15: // Custom-drawn boxes
				{
					if (!e.getShadow() && box.getCustomDraw() == CustomDraw.Additional)
					{
						for (int f = b.x + 16; f < b.x + b.width - 16; f += 16)
						{
							g.drawImage(_frame, f, b.y, f + 16, b.y + 16,
								16, 0, 32, 16, null);
							g.drawImage(_frame, f, b.y + b.height - 16, f + 16, b.y + b.height,
								16, 32, 32, 48, null);
						}

						for (int f = b.y + 16; f < b.y + b.height - 16; f += 16)
						{
							g.drawImage(_frame, b.x, f, b.x + 16, f + 16,
								0, 16, 16, 32, null);
							g.drawImage(_frame, b.x + b.width - 16, f, b.x + b.width, f + 16,
								32, 16, 48, 32, null);
						}

						g.drawImage(_frame, b.x, b.y, b.x + 16, b.y + 16,
							0, 0, 16, 16, null);
						g.drawImage(_frame, b.x + b.width - 16, b.y, b.x + b.width, b.y + 16,
							32, 0, 48, 16, null);
						g.drawImage(_frame, b.x, b.y + b.height - 16, b.x + 16, b.y + b.height,
							0, 32, 16, 48, null);
						g.drawImage(_frame, b.x + b.width - 16, b.y + b.height - 16, b.x + b.width, b.y + b.height,
							32, 32, 48, 48, null);

						if (box.getTag() != null)
						{
							if ("male".equals(box.getTag().toString()))
							{
								g.drawImage(_male,
									b.x + b.width - 34,
									b.y + b.height - 34, null);
							}
							else
							{
								g.drawImage(_female,
									b.x + b.width - 34,
									b.y + b.height - 34, null);
							}
						}
					}
				}
				break;

			}

			if (step.getKey() == 15) // Custom-drawn
			{
				if (box.getCustomDraw() == CustomDraw.Full)
				{
					if(box.getShape().getId().equals("Ellipse"))
					{
						for (int i = 0; i < 5; i++)
						{
							Rectangle2D.Float rc = new Rectangle2D.Float();
							rc.setFrame(bounds);

							float inflationX = b.width / 2 * (4 - i) / 5;
							float inflationY = b.height / 2 * (4 - i) / 5;
							rc.x += inflationX;
							rc.y += inflationY;
							rc.width -= 2 * inflationX;
							rc.height -= 2 * inflationY;

							if (e.getShadow())
							{
								g.setPaint(Color.BLACK);
								g.setStroke(new BasicStroke(6, BasicStroke.CAP_ROUND,
									BasicStroke.JOIN_MITER, 1.0f));

								g.drawArc((int)rc.x, (int)rc.y, (int)rc.width, (int)rc.height, 0, 360);
							}
							else
							{
								g.setPaint(Color.BLUE);
								g.setStroke(new BasicStroke(6, BasicStroke.CAP_ROUND,
									BasicStroke.JOIN_MITER, 1.0f));

								g.drawArc((int)rc.x, (int)rc.y, (int)rc.width, (int)rc.height, 0, 360);
							}
						}
					}
					else
					{
						if (e.getShadow())
						{
							g.drawImage(_sImage, b.x, b.y, b.width, b.height, null);
						}
						else
						{
							g.drawImage(_tImage, b.x, b.y, b.width, b.height, null);

							g.setPaint(Color.BLACK);
							g.setStroke(new BasicStroke(1, BasicStroke.CAP_ROUND,
								BasicStroke.JOIN_MITER, 1.0f));

							g.drawLine(b.x, b.y, b.x + b.width, b.y);
							g.drawLine(b.x + b.width, b.y, b.x + b.width, b.y + b.height);
							g.drawLine(b.x + b.width, b.y + b.height, b.x, b.y + b.height);
							g.drawLine(b.x, b.y + b.height, b.x, b.y);
						}
					}
				}
				else if (box.getCustomDraw() == CustomDraw.ShadowOnly)
				{
					g.drawImage(_shadow, b.x, b.y, b.width, b.height, null);
				}
			}
		}
	}

	/**
	 * Handles the JTree.ValueCanged event.
	 */
	private void onStepChanged(TreeSelectionEvent e)
	{
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)
			_steps.getLastSelectedPathComponent();

		if (node == null)
			return;

		if (node.getUserObject() instanceof Step)
		{
			Step step = (Step)node.getUserObject();

			// Load the step diagram
			loadStep(step.getFile());

			// Enable grid snapping
			_flowChart.setAlignToGrid(true);

			// Perform step-specific initialization
			switch (step.getKey())
			{

				case 8:
				{
					_flowChart.clearAll();
					
					_flowChart.setSelectAfterCreate(false);
					_flowChart.setBackBrush(new SolidBrush(new Color(240, 248, 255)));
					_diagView.setBehavior(Behavior.DrawLinks);
					_flowChart.setLinkHeadShape(ArrowHeads.Tetragon);
					_flowChart.setLinkPen(new Pen(0.6f, Color.BLACK));
					_flowChart.setLinkBrush(new SolidBrush(new Color(154, 205, 50)));
					_flowChart.setDefaultShape(com.mindfusion.diagramming.Shape.fromId("Rectangle"));

					_flowChart.setBounds(new Rectangle2D.Float(0, 0, 600, 900));
				}
					break;

				case 13: // Drag & drop
				{
					_flowChart.clearAll();

					_flowChart.setMeasureUnit(GraphicsUnit.Point);
					_flowChart.setAdjustmentHandlesSize(8);
					_flowChart.getShapeNodeStyle().setFontFamily("Times New Roman");
					_flowChart.getShapeNodeStyle().setFontSize(12);
					TextFormat tf = new TextFormat(Align.Center, Align.Center);
					tf.setWrapAtCharacter(false);
					tf.setNoWrap(false);
					ShapeNode b =
						_flowChart.getFactory().createShapeNode(10, 10, 300, 80);
					
					b.setTransparent(true);
					b.setBrush(new SolidBrush(Color.WHITE));
					b.setTextBrush(new SolidBrush(Color.BLACK));
					b.setText("Drag an image or listbox items over the document area");
					b.setFont(new Font("Times New Roman", Font.PLAIN, 18));
					b.setLocked(true);
					b.setTextFormat(tf);

					_flowChart.setBounds(new Rectangle2D.Float(0, 0, 1000, 1000));
					_flowChart.setBackBrush(new SolidBrush(Color.WHITE));
					_flowChart.setDefaultShape(com.mindfusion.diagramming.Shape.fromId("Ellipse"));
					_flowChart.setShadowsStyle(ShadowsStyle.None);
				}
					break;

				case 18: // Complex box shapes
				{
					_flowChart.clearAll();
					_flowChart.setMeasureUnit(GraphicsUnit.Point);
					_flowChart.setAdjustmentHandlesSize(8);
					_flowChart.setBounds(
						new Rectangle2D.Float(0, 0, 1000, 1000));
					_flowChart.setShadowsStyle(ShadowsStyle.None);
					_flowChart.setSelectAfterCreate(false);
					_flowChart.setBackBrush(new SolidBrush(new Color(240, 255, 255)));
					
					ShapeNodeStyle shapesStyle = _flowChart.getShapeNodeStyle();
					shapesStyle.setFontFamily("Tahoma");
					shapesStyle.setFontSize(8);
					shapesStyle.setBrush(new GradientBrush(
						new Color(255, 255, 255, 100),
						new Color(0, 128, 192, 255), 45));

					int i = 0;
					int hsize = 60;
					int vsize = 60;
					int perLine = 8;
					Pen black = new Pen(1, Color.BLACK);
					
					Object[] ids = new String[Shape.getShapes().size()];
					int a = 0;
					for (Shape shape : Shape.getShapes())
					{
						ids[a] = shape.getId();
						a++;
					}
					
					List list = Arrays.asList(ids);
					Collections.sort(list);
					for (int j = 0; j < list.size(); j++)
					{
						Shape shape = Shape.fromId(list.get(j).toString());
						if (shape.getIsArrowhead())
							continue;
						ShapeNode bb = _flowChart.getFactory().createShapeNode((i % perLine) * (hsize + 15) + 10,
							(i / perLine) * (vsize + 40) + 10, hsize, vsize);
						bb.setShape(shape);
						bb.setShadowOffsetX(4);
						bb.setShadowOffsetY(4);

						bb.setPen(black);

						Group g = _flowChart.getFactory().createGroup(bb);
						bb = _flowChart.getFactory().createShapeNode((i % perLine) * (hsize + 15) + 2,
							(i / perLine) * (vsize + 40) + vsize + 10, hsize + 16, 18);
						bb.setText(shape.getId());
						bb.setLocked(true);
						bb.setTransparent(true);
						bb.setShape(com.mindfusion.diagramming.Shape.fromId("Rectangle"));

						g.attachToCorner(bb, 3);

						i++;
					}

					_flowChart.resizeToFitItems(4, false);
				}
					break;

				case 21: // Big radial layout
				{
					_flowChart.clearAll();
					_flowChart.setLinkShape(LinkShape.Polyline);
					_flowChart.setLinkSegments((short)1);
					_flowChart.setDynamicLinks(true);
					_flowChart.setShadowsStyle(ShadowsStyle.None);
					_flowChart.setDefaultShape(com.mindfusion.diagramming.Shape.fromId("Ellipse"));
					_flowChart.setBackBrush(new SolidBrush(Color.WHITE));
					_flowChart.setLinkHeadShape(ArrowHeads.None);
					_flowChart.setSelectAfterCreate(false);
				}
					break;
				
				case 6:
				case 22:
					_flowChart.getTableNodeStyle().setFontFamily(_flowChart.getFont().getName());
					_flowChart.getTableNodeStyle().setFontSize(_flowChart.getFont().getSize());
					_flowChart.getTableNodeStyle().setFontStyle(_flowChart.getFont().getStyle());
					break;
					
				case 23:	// auto routing
					_flowChart.setRouteLinks(true);
					break;

				case 37:
				{
					_flowChart.setAlignToGrid(false);
				}
					break;
				case 47:
				{
					_diagView.setBehavior(Behavior.DrawControls);
				}
					break;
				

			}

			// Set the step info
			try
			{
				_info.setPage(getResourceUrl(step.getInfo()));
			}
			catch (java.io.IOException ex)
			{
			}

			// Set the step sample source code
			try
			{
				_sourceCode.setPage(getResourceUrl(step.getSource()));
			}
			catch (java.io.IOException ex)
			{
			}

			// Show the appropriate control panel
			if (step.getControlPaneId() != "")
				showControlPane(step.getControlPaneId());
			else
				showControlPane("Empty");
		}
		else
		{
			// Clear the control contents
			_flowChart.clearAll();
			_flowChart.setBackBrush(new SolidBrush(SystemColor.controlHighlight));
			_diagView.setBehavior(Behavior.LinkShapes);
			_flowChart.setBounds(new Rectangle2D.Float(0, 0, 100, 100));

			// Clear the info
			_info.setText("");

			// Hide any control panes
			showControlPane("Empty");
		}

		// Reset the diagram state
	_diagView.setZoomFactor(100);
	}

	/**
	 * Loads the specified diagram.
	 */
	private void loadStep(String file)
	{
		if (file.length() == 0)
			return;

			try
			{
				if (file.endsWith(".xml"))
					_diagView.loadFromXml("com\\mindfusion\\diagramming\\samples\\demo\\" + file);
				else
					_diagView.loadFrom("com\\mindfusion\\diagramming\\samples\\demo\\" + file);
			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
			catch (XmlException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		
	}


	/**
	 * Performs a check and a style reseting of elements
	 * during graph management.
	 */
	private boolean checkAndClear()
	{
		if(_flowChart.getSelection().getNodes().size() == 0)
		{
			JOptionPane.showMessageDialog(this, "Select a box first!");
			return false;
		}

		Pen pen = new Pen(1f, Color.BLACK);

		for (DiagramLink a : _flowChart.getLinks())
		{
			a.setPen(pen);
			a.setHeadPen(pen);
		}

		for (DiagramNode b : _flowChart.getNodes())
			b.setPen(pen);

		_diagView.invalidate();

		return true;
	}

	/**
	 * Recursive traversal of box' successors.
	 */
	private void visitSuccessors(DiagramNode b, DiagramNodeList visited)
	{
		Pen p = new Pen(3f, Color.BLUE);
		for (DiagramLink a : b.getOutgoingLinks())
		{
			DiagramNode bb = (DiagramNode)a.getDestination();

			if (!visited.contains(bb))
			{
				bb.setPen(p);

				visited.add(bb);

				visitSuccessors(bb, visited);
			}
		}
	}

	/**
	 * Recursive traversal of box' predecessors.
	 */
	private void visitPredecessors(DiagramNode b, DiagramNodeList visited)
	{
		Pen p = new Pen(3f, Color.BLUE);
		for (DiagramLink a : b.getIncomingLinks())
		{
			DiagramNode bb = (DiagramNode)a.getOrigin();

			if (!visited.contains(bb))
			{
				bb.setPen(p);

				visited.add(bb);

				visitPredecessors(bb, visited);
			}
		}
	}

	/**
	 * Loads the contents of the  specified XML file as a diagram.
	 */
	private void loadXmlFile(File file)
	{
		_flowChart.clearAll();
		_flowChart.setMeasureUnit(GraphicsUnit.Point);
		_flowChart.setBackBrush(new SolidBrush(new Color(175, 238, 238)));
		_flowChart.setLinkCascadeOrientation(Orientation.Vertical);
		_flowChart.setLinkShape(LinkShape.Cascading);
		_flowChart.setLinkSegments((short)2);
		_flowChart.setSelectAfterCreate(false);
		_flowChart.getShapeNodeStyle().setFontSize(10);

		// Load the file
		DocumentBuilder builder = null;
		try
		{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setValidating(false);
			factory.setNamespaceAware(false);
			builder = factory.newDocumentBuilder();
		}
		catch (ParserConfigurationException ex)
		{
			JOptionPane.showMessageDialog(this, "Failed to load the XML file.");
			return;
		}

		try
		{
			Document document = builder.parse(file);
			Element root = document.getDocumentElement();

			_yIndent = 5;
			org.w3c.dom.NodeList children = document.getChildNodes();
			for (int i = 0; i < children.getLength(); i++)
				drawXmlNode(5, null, children.item(i));

			Rectangle2D.Float rc = _flowChart.getBounds();
			rc.height = _yIndent + 15;
			_flowChart.setBounds(rc);
		}
        catch (IOException ex)
        {
			JOptionPane.showMessageDialog(this, "Failed to load the XML file.\n" + ex.getMessage());
            return;
        }
        catch (SAXException ex)
        {
			JOptionPane.showMessageDialog(this, "Failed to load the XML file.");
            return;
        }
	}

	/**
	 * Recursively draws the nodes under the specified XML node.
	 */
	private void drawXmlNode(int xIndent, ShapeNode parent, org.w3c.dom.Node node)
	{
		if (node == null)
			return;

		String text = "";
		if (node.getNodeType() == org.w3c.dom.Node.TEXT_NODE)
			text = node.getNodeValue();
		else
			text = node.getNodeName();

		if (text.trim().length() == 0)
			return;

		// Create the box for this node
		ShapeNode box = _flowChart.getFactory().createShapeNode(xIndent, _yIndent, 100, 20);

		box.setText(text);
		box.setAnchorPattern(_leftInBottomOut);

		if (node.getNodeType() == org.w3c.dom.Node.TEXT_NODE)
			box.setBrush(new SolidBrush(new Color(255, 255, 224)));
		else
			box.setBrush(new SolidBrush(new Color(222, 184, 135)));

		_yIndent += 30;

		// Link this node with its parent
		if (parent != null)
			_flowChart.getFactory().createDiagramLink(parent, box);

		// Draw the children
		org.w3c.dom.NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++)
			drawXmlNode(xIndent + 80, box, children.item(i));
	}

	/**
	 * Returns the URL of the resource with the specified file.
	 */
	private java.net.URL getResourceUrl(String resourceFile)
	{
		return MainFrame.class.getResource(resourceFile);
	}

	/**
	 * Recursively enumerates the specified filder's subfolders
	 * and creates a tree in flow chart, corresponding to it.
	 */
	private void enumDirectoryInfo(File folder, ShapeNode parent, int level)
	{
		if (level <= 0)
			return;

		ShapeNode b = _flowChart.getFactory().createShapeNode(0, 0, 10, 10);
		b.setToolTip(folder.getName());

		if (parent == null)
			b.setBrush(new SolidBrush(Color.GREEN));
		else
			b.setBrush(new SolidBrush(new Color(233, 150, 122)));

		if (parent != null)
			_flowChart.getFactory().createDiagramLink(parent, b);

		int maxsubdirs = 100;
		try
		{
			File[] subDirs = folder.listFiles(new FileFilter()
			{
				public boolean accept(File pathname)
				{
					return pathname.isDirectory();
				}
			});
	
			if (subDirs != null)
			{
				for (File file : subDirs)
				{
					maxsubdirs--;
					if (maxsubdirs < 0)
						break;
		
					enumDirectoryInfo(file, b, level - 1);
				}
			}
		}
		catch(Exception exp)
		{
			exp.getMessage(); //skip warning
		}
	}

	public void ItemDropped(MouseEvent e)
	{

		if (!FlowChartTransferHandler.isDropped())
			return;


		Point mouseDevPos = new Point(e.getX(), e.getY());
		Point2D.Float mouseDocPos = _diagView.deviceToDoc(mouseDevPos);

		
		if (FlowChartTransferHandler.getBoxImage() != null)
		{
			ShapeNode box = _flowChart.getFactory().createShapeNode(
				(float)mouseDocPos.getX() - 30, (float)mouseDocPos.getY() - 30, 10, 10);
			box.setImage(FlowChartTransferHandler.getBoxImage());
			box.setImageAlign(ImageAlign.Fit);
			box.resizeToFitImage();
			box.setTransparent(true);
			box.setRotateImage(true);
			box.setEnabledHandles(AdjustmentHandles.All);
		}

		if (FlowChartTransferHandler.getText() != null)
		{
			ShapeNode box = _flowChart.getFactory().createShapeNode(
				(float)mouseDocPos.getX() - 30, (float)mouseDocPos.getY() - 10, 100, 20);
			box.setText(FlowChartTransferHandler.getText());
			box.setRotateText(true);
		}
		
		FlowChartTransferHandler.setDropped();
	}


	/**
	 * Gets the image from the specified file.
	 */
	private Image getImage(String fileName)
	{
		//return getToolkit().createImage(getResourceUrl(fileName));
		try
		{
			return javax.imageio.ImageIO.read(new java.io.File(fileName));
		}
		catch (IOException ex)
		{
			return null;
		}
	}

	/**
	 * Gets the image with the specified relative file name
	 * (e.g. 'Files/image.gif').
	 */
	private Image getImageRelative(String image)
	{
		try
		{
			return getImage(getResourceUrl(image).toURI().getPath());
		}
		catch (Exception e)
		{
			return null;
		}
	}

	public JFrame getOverviewFrame()
	{
		return overviewFrame;
	}
	
	private void changeIndexFlowchartLayout(int index)
	{
		if (index >= 0 && index < _cmBxLoadStep.getItemCount())
		{
			String dirPath = "com\\mindfusion\\diagramming\\samples\\demo\\";
			File file = new File(dirPath + "Files\\Resources\\Flowcharts\\").listFiles()[index];
			try
			{
				_flowChart.loadFrom(file.getAbsolutePath());
			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
		}
	}
	
	private void loadFlowchartLayoutFile(int index)
	{
		if (index >= 0 && index < _cmBxLoadStep.getItemCount())
			_cmBxLoadStep.setSelectedIndex(index);
	}
	
	private void onBtnFlowchartLayout()
	{
		FlowchartLayout layout = new FlowchartLayout();

		if (_rbHorizontalFlowChartLayout.isSelected())
			layout.setOrientation(com.mindfusion.diagramming.jlayout.Orientation.Horizontal);
		else
			layout.setOrientation(com.mindfusion.diagramming.jlayout.Orientation.Vertical);
		layout.setIgnoreRepeatingLinks(_chkIgnoreRepeatingLinks.isSelected());

		layout.arrange(_flowChart);

		_flowChart.resizeToFitItems(5, false);
	}

	private Random _random;

	// Menu
	private JMenuBar _menuBar;
	private JMenu _menuFile;
	private JMenuItem _menuFileClear;
	private JMenuItem _menuFileOpen;
	private JMenuItem _menuFileSave;
	private JMenuItem _menuFileImport;
	private JMenuItem _menuFileExport;
	private JMenuItem _menuFilePrintPreview;
	private JMenuItem _menuFilePrint;
	private JMenuItem _menuFileExit;
	private JMenu _menuEdit;
	private JMenuItem _menuEditUndo;
	private JMenuItem _menuEditRedo;
	private JMenuItem _menuEditCut;
	private JMenuItem _menuEditCopy;
	private JMenuItem _menuEditPaste;
	private JMenu _menuObject;
	private JMenuItem _menuObjectDelete;
	private JMenuItem _menuObjectGroup;
	private JMenuItem _menuObjectUngroup;
	private JMenuItem _menuObjectUnlockAll;
	private JMenu _menuArrow;
	private JMenuItem _menuArrowRouteAll;
	private JMenu _menuHelp;
	private JMenuItem _menuHelpAbout;

	// Controls
	private JTree _steps;
	private JScrollPane _stepsPane;
	private JEditorPane _info;
	private JScrollPane _infoPane;
	private FlowChartTransferHandler _transferHandler;
	private Diagram _flowChart;
	private DiagramView _diagView;
	private JEditorPane _sourceCode;
	private JScrollPane _sourceCodePane;
	private JSplitPane _sourceCodeSplitPane;
	private JScrollPane _flowChartPane;
	private JSplitPane _mainSplitPane;
	private JSplitPane _bottomSplitPane;
	private JTabbedPane _mainTabPane;
	private JPanel _controlPane;

	private JPanel _emptyControlPane;

	private JPanel _dialogueControlPane;
	private JButton _btnTakeATour;

	private JPanel _ermControlPane;
	private JButton _btnCreateEntity;

	private JPanel _dbControlPane;
	private JButton _btnAddTable;
	private JButton _btnAddField;

	private JPanel _vesselsControlPane;
	private JButton _btnAnimate;

	private JPanel _treeLayoutControlPane;
	private JLabel _lblType;
	private JComboBox _cbType;
	private JLabel _lblDirection;
	private JComboBox _cbDirection;
	private JLabel _lblArrows;
	private JComboBox _cbArrows;
	private JLabel _lblLevelSpacing;
	private JTextField _txtLevelSpacing;
	private JLabel _lblItemSpacing;
	private JTextField _txtItemSpacing;
	private JButton _btnTreeLayout;

	private JPanel _radialLayoutControlPane;
	private JButton _btnRadialLayout;

	private JPanel _springLayoutControlPane;
	private JLabel _lblIterations;
	private JTextField _txtIterations;
	private JLabel _lblNodeDistance;
	private JTextField _txtNodeDistance;
	private JButton _btnSpringLayout;

	private JPanel _layeredLayoutControlPane;
	private JRadioButton _rbVertical;
	private JRadioButton _rbHorizontal;
	private ButtonGroup _bgOrientation;
	private JLabel _lblLevelSpacing2;
	private JTextField _txtLevelSpacing2;
	private JLabel _lblNodeSpacing;
	private JTextField _txtNodeSpacing;
	private JCheckBox _chkCompact;
	private JButton _btnLayeredLayout;

	private JPanel _flowchartLayoutControlPane;
	private JRadioButton _rbVerticalFlowChartLayout;
	private JRadioButton _rbHorizontalFlowChartLayout;
	private JCheckBox _chkIgnoreRepeatingLinks;
	private JButton _prevStep;
	private JButton _nextStep;
	private JComboBox _cmBxLoadStep;
	private JButton _btnFlowchartLayout;

	private JPanel _gridLayoutControlPane;
	private JLabel _lblIterations2;
	private JTextField _txtIterations2;
	private JLabel _lblGridSize;
	private JTextField _txtGridSize;
	private JButton _btnGridLayout;
	
	private JPanel _orthogonalLayoutControlPane;
	private JLabel _lbMGP;
	private JComboBox _cmBxMGP;
	private JLabel _lbRefine;
	private JComboBox _cmBxRefine;
	private JLabel _lbPadding;
	private JTextField _txFlPadding;
	private JLabel _lbMargins;
	private JLabel _lbMarginX;
	private JLabel _lbMarginY;
	private JTextField _txFlMarginX;
	private JTextField _txFlMarginY;
	private JButton _btnOrthogonalLayout;

	private JPanel _circularLayoutControlPane;
	private JLabel _lbMGP2;
	private JComboBox _cmBxMGP2;
	private JLabel _lbRadius;
	private JTextField _txFlRadius;
	private JLabel _lbSiftRounds;
	private JTextField _txFlSiftRounds;
	private JLabel _lbMargins2;
	private JLabel _lbMarginX2;
	private JLabel _lbMarginY2;
	private JTextField _txFlMarginX2;
	private JTextField _txFlMarginY2;
	private JButton _btnCircularLayout;
	
	private JPanel _custeredLayoutControlPane;
	private JButton _btnClusteredLayout;

	private JPanel _graphControlPane;
	private JButton _btnOutgoing;
	private JButton _btnIncoming;
	private JButton _btnChildren;
	private JButton _btnParents;
	private JButton _btnSuccessors;
	private JButton _btnPredecessors;

	private JPanel _graphPathControlPane;
	private JButton _btnFindPath;
	private JButton _btnFindAllPaths;
	private JButton _btnDetectCycle;
	private JButton _btnFindCycle;
	private JButton _btnFindAllCycles;

	private JPanel _netSampleControlPane;
	private JLabel _lblObjects;
	private JComboBox _cbObjects;
	private JCheckBox _chkAttachText;
	private JTextField _txtAttachText;

	private JPanel _visualXmlControlPane;
	private JButton _btnLoadXml;
	private JButton _btnShowSample;

	private JPanel _genealogyControlPane;
	private JButton _btnAddPerson;

	private JPanel _rotationControlPane;
	private ImageBox _image1;
	private JLabel _label1;
	private ImageBox _image2;
	private JLabel _label2;
	private ImageBox _image3;
	private JLabel _label3;
	private ImageBox _image4;
	private JLabel _label4;
	private ImageBox _image5;
	private JLabel _label5;
	private ImageBox _image6;
	private JLabel _label6;
	private ImageBox _image7;
	private JLabel _label7;
	private ImageBox _image8;
	private JLabel _label8;
	private ImageBox _image9;
	private JLabel _label9;
	private ImageBox _image10;
	private JLabel _label10;
	private ImageBox _image11;
	private JLabel _label11;
	private ImageBox _image12;
	private JLabel _label12;
	private ImageBox _image13;
	private JLabel _label13;
	private ImageBox _image14;
	private JLabel _label14;

	private JPanel _dragDropControlPane;
	private ImageBox _imgDrag;
	private JList _lstDrag;

	private JPanel _formPreservationControlPane;
	private JButton _btnPreserveForm;

	private JPanel _spanningControlPane;
	private JButton _btnCreateSpanning;
	
	private JPanel _treeNodePane;
	private JButton _btnCreateTreeNode;
	
	private JPanel _formDesigner;
	private JLabel _lblFormDesigner;
	private JComboBox _cmBxControlType;
	private JButton _btnTestForm;

	// Private data
	private int _yIndent;
	private AnchorPattern _leftInBottomOut;
	private AnchorPattern _decisionInOut;

	// File chooser
	private JFileChooser _fileDialog;

	private Image _frame;
	private Image _male;
	private Image _female;

	private BufferedImage _tImage;
	private BufferedImage _sImage;
	private BufferedImage _shadow;
	private Image[] _netImages;

	private Overview overview;
	private JFrame overviewFrame;
}

class PropertyGridModel extends AbstractTableModel
{
	private static final long serialVersionUID = 1L;

	public PropertyGridModel() 
	{
	}

	// These are the names of the columns represented by this TableModel
    static final String[] columnNames = new String[] { "Author", "Title" };

    // These are the types of the columns represented by this TableModel
    static final Class[] columnTypes = new Class[] { String.class, String.class };
    
    static final String[][] values = {{"Victor Hugo", "Les Miserables"}, 
    								{"Jaroslav Hasek", "The Good Soldier Svejk"}, 
    								{"J. R. R. Tolkien", "The Lord of the Rings"}, 
    								{"Edgar Allan Poe", "The Raven"}, 
    								{"Oscar Wilde", "The Picture of Dorian Gray"}};
	
    @Override
	public String getColumnName(int column) {
        return columnNames[column];
    }

    @Override
	public Class getColumnClass(int column) {
    	return columnTypes[column];
    }

	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		return values.length;
	}

	public Object getValueAt(int row, int column)
	{
		return values[row][column];
	}
}