import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Float;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JScrollPane;

import com.mindfusion.diagramming.AnchorPattern;
import com.mindfusion.diagramming.AnchorPoint;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramAdapter;
import com.mindfusion.diagramming.DiagramLink;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.HandlesStyle;
import com.mindfusion.diagramming.LinkEvent;
import com.mindfusion.diagramming.LinkShape;
import com.mindfusion.diagramming.MarkStyle;
import com.mindfusion.diagramming.NodeEvent;
import com.mindfusion.diagramming.NodeValidationEvent;
import com.mindfusion.diagramming.Orientation;
import com.mindfusion.diagramming.Pen;
import com.mindfusion.diagramming.Shape;
import com.mindfusion.diagramming.SnapToAnchor;
import com.mindfusion.diagramming.SolidBrush;
import com.mindfusion.diagramming.lanes.Grid;
import com.mindfusion.diagramming.lanes.Header;
import com.mindfusion.diagramming.lanes.HeaderList;
import com.mindfusion.diagramming.lanes.ICell;


public class LanesFrame extends JFrame
{
	public LanesFrame()
	{
		super("MindFusion.Diagramming sample: Lanes");
		initFrame();
		initDiagram();
	}
	
	private void initDiagram()
	{
		// Initialize flowchart
		diagram.setBackBrush(new SolidBrush(new Color(240,240,250)));
		diagram.setLinkShape(LinkShape.Cascading);
		diagram.setLinkSegments((short)2);
		diagram.setLinkCascadeOrientation(Orientation.Horizontal);
		diagram.setSnapToAnchor(SnapToAnchor.OnCreateOrModify);
		diagram.getDiagramLinkStyle().setBrush(new SolidBrush(Color.black));
		diagram.setLinkHeadShape(Shape.fromId("Alternative"));
		diagram.setLinkHeadShapeSize(1.4f);
		diagram.getShapeNodeStyle().setBrush(new SolidBrush(new Color(40, 0, 0, 0)));
		diagram.setShadowOffsetX(0.6f);
		diagram.setShadowOffsetY(0.6f);
		diagram.setDefaultShape(Shape.fromId("Rectangle"));
		diagram.setAllowSelfLoops(false);
		diagram.getShapeNodeStyle().setBrush(new SolidBrush(new Color(247,247,242)));
		
		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void linkCreated(LinkEvent e) 
			{
				diagramLinkCreated(e);
			}
			public void nodeCreated(NodeEvent e) 
			{
				diagramNodeCreated(e);
			}
			public void nodeModified(NodeEvent e) 
			{
				diagramNodeModified(e);
			}
			public void nodeCreating(NodeValidationEvent e)
			{
				diagramNodeCreating(e);
			}
			public void nodeModifying(NodeValidationEvent e)
			{
				diagramNodeModifying(e);
			}
		});

		pattern = new AnchorPattern( new AnchorPoint[] {
			new AnchorPoint(100, 50, false, true, MarkStyle.None),
			new AnchorPoint(0, 0, true, false, MarkStyle.None)
		}, "myPattern");

		// Initialize grid
		Header header;
		Grid grid = diagram.getLaneGrid();
		HeaderList columns = grid.getColumnHeaders();

		grid.setMinHeaderSize(8);
		grid.setHookHeaders(false);
		grid.setHeadersOnTop(false);
		grid.setColumnHeadersHeights(new float[] { 8, 8 });
		grid.setAllowResizeHeaders(true);

		for (int i = 1; i <= 8; i++)
		{
			header = new Header("Week " + i + ", 2007");
			header.getSubHeaders().add(new Header("S"));
			header.getSubHeaders().add(new Header("M"));
			header.getSubHeaders().add(new Header("T"));
			header.getSubHeaders().add(new Header("W"));
			header.getSubHeaders().add(new Header("T"));
			header.getSubHeaders().add(new Header("F"));
			header.getSubHeaders().add(new Header("S"));
			columns.add(header);
		}

		for (int i = 1; i <= 6; i++)
			grid.getRowHeaders().add(new Header("Task " + i));

		int count = 8 * 7;
		for (int i = 0; i < count; i++)
		{
			Header column = grid.getColumn(i);

			if (i % 7 != 0)
			{
				grid.get(column, null).getStyle().setLeftBorderPen(null);
			}
			else
			{
				grid.get(column, null).getStyle().setLeftBorderPen(new Pen(0, Color.gray));
			}

			if (i != count - 1)
				grid.get(column, null).getStyle().setRightBorderPen(null);

			if (i % 7 == 0 ||
				i % 7 == 6)
			{
				grid.get(column, null).getStyle().setBackgroundBrush(
					new SolidBrush(new Color(238, 232, 170)));
			}
		}

		count = 6;
		for (int i = 0; i < count; i++)
		{
			Header row = grid.getRow(i);

			if (i == 0)
			{
				grid.get(null, row).getStyle().setTopBorderPen(null);
			}
			else
			{
				grid.get(null, row).getStyle().setTopBorderPen(new Pen(0, Color.gray));
			}

			if (i != count - 1)
				grid.get(null, row).getStyle().setBottomBorderPen(null);
		}

		diagram.setEnableLanes(true);

		// Ensure the document is big enough to contain the grid
		float width = Math.max(50, grid.getColumnCount() * 6 + 20);
		float height = Math.max(50, grid.getRowCount() * 6 + 20);

		diagram.setBounds(new Rectangle2D.Float(0, 0, width, height));
		diagram.setAlignToGrid(false);
	}

	private void initFrame()
	{
		diagram = new Diagram();
		diagramView = new DiagramView(diagram);
		
		JScrollPane scrollPane = new JScrollPane(diagramView);
		
		BorderLayout layout = new BorderLayout();
		this.setLayout(layout);
		
		this.add(scrollPane);
		this.setSize(750, 450);
		this.addWindowListener(new WindowAdapter(){
			@Override
			public void windowClosing(WindowEvent arg0)
			{
				System.exit(0);
			}
		});
	}
	
	/// <summary>
	/// Gets the size of a pixel measured in document units.
	/// </summary>
	private float getPixel()
	{
		Point2D.Float pt1 = diagramView.deviceToDoc(0, 0);
		Point2D.Float pt2 = diagramView.deviceToDoc(1, 1);
		
		return pt2.x - pt1.x;
	}

	/// <summary>
	/// Fixes the segments of the specified arrow according
	/// to the position of its related boxes.
	/// </summary>
	private void setLinkShape(DiagramLink a)
	{
		Float originBounds = (Float)a.getOrigin().getBounds().clone();
		Float destBounds = (Float)a.getDestination().getBounds().clone();

		if (originBounds.getMaxX() < destBounds.getX())
		{
			a.setSegmentCount(2);
		}
		else
		{
			a.setSegmentCount(4);

			Point2D.Float pt0 = (Point2D.Float)a.getControlPoints().get(0).clone();
			Point2D.Float pt1;
			Point2D.Float pt2;
			Point2D.Float pt3 = (Point2D.Float)a.getControlPoints().get(3).clone();
			Point2D.Float pt4 = (Point2D.Float)a.getControlPoints().get(4).clone();

			pt1 = (Point2D.Float)pt0.clone();
			pt1.x += 2;

			pt2 = (Point2D.Float)pt1.clone();
			pt2.y += 4;

			pt3.x = pt4.x;
			pt3.y = pt2.y;

			a.getControlPoints().set(1, pt1);
			a.getControlPoints().set(2, pt2);
			a.getControlPoints().set(3, pt3);

			a.updateFromPoints();
		}
	}
	
	private void diagramNodeCreated(NodeEvent e)
	{
		e.getNode().setHandlesStyle(HandlesStyle.Custom);
		e.getNode().setAnchorPattern(pattern);

		// Place the box within the grid
		Float bounds = (Float)e.getNode().getBounds().clone();
		Point2D.Float topLeft = new Point2D.Float(bounds.x, bounds.y);

		Grid grid = diagram.getLaneGrid();

		Float cellBounds = new Float();
		grid.getCellFromPoint(topLeft, cellBounds);

		float pixel = getPixel();

		bounds.y = cellBounds.y + 4 * pixel;
		bounds.height = cellBounds.height - 10 * pixel;
		e.getNode().setBounds(bounds);
	}

	private void diagramNodeCreating(NodeValidationEvent e)
	{
		Float bounds = (Float)e.getNode().getBounds().clone();
		Point2D.Float topLeft = new Point2D.Float(bounds.x, bounds.y);

		Grid grid = diagram.getLaneGrid();

		Float cellBounds = new Float();
		ICell cell = grid.getCellFromPoint(topLeft, cellBounds);

		if (cell == null)
			e.setCancel(true);
	}
	
	private void diagramNodeModifying(NodeValidationEvent e)
	{
		Float bounds = (Float)e.getNode().getBounds().clone();
		Point2D.Float topLeft = new Point2D.Float(bounds.x, bounds.y + bounds.height / 2);

		Grid grid = diagram.getLaneGrid();

		Float cellBounds = new Float();
		ICell cell = grid.getCellFromPoint(topLeft, cellBounds);

		if (cell == null)
			e.setCancel(true);
	}

	private void diagramLinkCreated(LinkEvent e)
	{
		e.getLink().setAllowMoveStart(false);
		e.getLink().setAllowMoveEnd(false);
		setLinkShape(e.getLink());
	}

	private void diagramNodeModified(NodeEvent e)
	{
		// Place the box within the grid
		Float bounds = (Float)e.getNode().getBounds().clone();
		Point2D.Float topLeft = new Point2D.Float(bounds.x, bounds.y + bounds.height / 2);

		Grid grid = diagram.getLaneGrid();

		Float cellBounds = new Float();
		grid.getCellFromPoint(topLeft, cellBounds);

		float pixel = getPixel();

		bounds.y = cellBounds.y + 4 * pixel;
		bounds.height = cellBounds.height - 10 * pixel;
		e.getNode().setBounds(bounds);

		// Fix links
		ArrayList<DiagramLink> links = new ArrayList<DiagramLink>();
		links.addAll(e.getNode().getIncomingLinks());
		links.addAll(e.getNode().getOutgoingLinks());

		for (DiagramLink link : links)
		{
			setLinkShape(link);
		}	
	}

	private DiagramView diagramView;
	private Diagram diagram;
	private AnchorPattern pattern = null;
	
	private static final long serialVersionUID = 1L;
}