import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collections;
import java.util.List;
import java.util.Arrays;

import javax.swing.JFrame;
import javax.swing.JScrollPane;

import com.mindfusion.diagramming.*;

public class MainFrame extends JFrame
{
	public MainFrame()
	{
		super("MindFusion.Diagramming sample: Predefined Shapes");

		diag = new Diagram();
		diagView = new DiagramView(diag);
		fcScroll = new JScrollPane(diagView);
		
		this.setLayout(new GridLayout(1,1));
		this.add(fcScroll);
		
		onLoad();
		
		this.setSize(800,600);
		
		this.addWindowListener
		(
			new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{
					System.exit(0);
				}
			}
		);
	}
	
	@SuppressWarnings("unchecked")
	private void onLoad()
	{
		diag.setSelectAfterCreate(false);
		
		diag.setBackBrush(new SolidBrush(new Color(234,255,255)));
		diag.getDiagramStyle().setShadowBrush(new SolidBrush(new Color(242,242,242)));
	
		ShapeNodeStyle shapesStyle = diag.getShapeNodeStyle(); 
		shapesStyle.setBrush(new GradientBrush(new Color(255, 255, 255, 100),
			new Color(0, 128, 192, 255), 45));
		shapesStyle.setStroke(new SolidBrush(Color.black));
		shapesStyle.setFontFamily("Tahoma");
		shapesStyle.setFontSize(3);
		
		int i = 0;
		int hsize = 17;
		int vsize = 19;
		int perLine = 8;
		
		Object[] ids = new String[Shape.getShapes().size()];
		int a = 0;
		for (Shape shape : Shape.getShapes())
		{
			ids[a] = shape.getId();
			a++;
		}
		
		List list = Arrays.asList(ids);
		
		try
		{
			Collections.sort(list);
		}
		catch (Exception exp)
		{
		}
		
		// enum all predefined shapes
		for (int j = 0; j < list.size(); j++)
		{
			Shape shape = Shape.fromId(list.get(j).toString());
			
			// skip some arrowhead shapes that aren't that useful as node shapes
			if (shape.getOutline().length == 0) 
				continue;
			if (shape.getId() == "RevWithCirc") 
				continue;
			if (shape.getId() == "DoubleArrow") 
				continue;

			// create a box of this shape
			ShapeNode node = diag.getFactory().createShapeNode(
				(i % perLine) * (hsize + 12) + 2,
				(i / perLine) * (vsize + 15) + 4,
				hsize, vsize);
			node.setShape(shape);

			// attach text below the box
			Group labels = diag.getFactory().createGroup(node);
			ShapeNode label = diag.getFactory().createShapeNode(
				node.getCenter().getX() - hsize / 2 - 2,
				(i / perLine) * (vsize + 15) + vsize + 4,
				hsize + 4, 8);
			label.setShape(Shape.fromId("Rectangle"));
			label.setText(shape.getId().startsWith("Bpmn") ?
				"Bpmn\n\r" + shape.getId().substring(4) : shape.getId());
			label.resizeToFitText(FitSize.KeepRatio);
			label.setLocked(true);
			label.setTransparent(true);
			label.moveTo(
				(float)(node.getCenter().getX() - label.getBounds().getWidth() / 2),
				(float)label.getBounds().getY());

			labels.attachToCorner(label, 3);
			
			i = i + 1;			
		}

		diag.resizeToFitItems(4, true);
	}
	
	private JScrollPane fcScroll;
	private Diagram diag;
	private DiagramView diagView;
}