/**
 * Copyright (c) 2013, MindFusion LLC - Bulgaria.
 */

package samples.scripting;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.beans.*;
import java.io.IOException;
import java.util.*;

import javax.imageio.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;

import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;
import com.mindfusion.graphs.*;
import com.mindfusion.scripting.*;
import com.mindfusion.scripting.Compiler;


public class MainFrame extends JFrame
	implements ActionListener, SimpleEvaluationContextListener, PropertyChangeListener, ListSelectionListener
{
	public MainFrame()
	{
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationByPlatform(true);
		setSize(800, 600);
		setTitle("JDiagram Sample: Scripting");
		setExtendedState(JFrame.MAXIMIZED_BOTH);
		try
		{
			ArrayList<Image> images = new ArrayList<Image>();
			images.add(ImageIO.read(App.class.getResourceAsStream("App.png")));
			images.add(ImageIO.read(App.class.getResourceAsStream("App32.png")));
			setIconImages(images);
		}
		catch (Exception ex)
		{
		}

		// Change the application font
		Font defaultFont = new Font("Arial", Font.PLAIN, 12);
		UIManager.put("Button.font", defaultFont);
		UIManager.put("Label.font", defaultFont);
		UIManager.put("TextArea.font", defaultFont);
		UIManager.put("List.font", defaultFont);

		initializeComponent();

		// Create the special variable list node
		variableListNode = diagram.getFactory().createShapeNode(0, 0, 1, 1);
		variableListNode.setVisible(false);
		variableListNode.setTag("variableList");
		variableListNode.setLocked(true);
		updateVariableListPosition();

		evaluationContext = new SimpleEvaluationContext(this);
		evaluationContext.addListener(this);

		operationPattern = new AnchorPattern(
			new AnchorPoint[]
			{
				new AnchorPoint(50, 0, true, false, MarkStyle.Custom),
				new AnchorPoint(50, 100, false, true, MarkStyle.Custom),
			}, "operationAnchor");
		operationPattern.getPoints().get(0).setTag(0);
		operationPattern.getPoints().get(1).setTag(3);

		choicePattern = new AnchorPattern(
			new AnchorPoint[]
			{
				new AnchorPoint(50, 0, true, false, MarkStyle.Custom),
				new AnchorPoint(50, 100, false, true, MarkStyle.Custom),
				new AnchorPoint(100, 50, false, true, MarkStyle.Custom),
			}, "choiceAnchor");
		choicePattern.getPoints().get(0).setTag(0);
		choicePattern.getPoints().get(1).setTag(1);
		choicePattern.getPoints().get(2).setTag(2);

		examplesList.setListData(new String[] {"Factorial", "Gcd", "Fibonacci" });
		examplesList.addListSelectionListener(this);
	}

	private void initializeComponent()
	{
		setLayout(new BorderLayout());

		panel1 = new JPanel(new BorderLayout());
		label1 = new JTextArea();
		panel2 = new JPanel(null);
		panel3 = new JPanel(new FlowLayout(FlowLayout.LEFT, 3, 3));
		label2 = new JLabel("Creation:");
		startButton = new JButton("Start node");
		operationButton = new JButton("Operation");
		choiceButton = new JButton("Choice");
		endButton = new JButton("End node");
		clearButton = new JButton("Clear");
		label3 = new JLabel("Execution:");
		panel4 = new JPanel(new FlowLayout(FlowLayout.LEFT, 3, 3));
		validateButton = new JButton("Validate");
		runButton = new JButton("Run");
		stepRunButton = new JButton("Step by step");
		stopButton = new JButton("Stop");
		panel5 = new JPanel(new FlowLayout(FlowLayout.LEFT, 3, 3));
		label8 = new JLabel("Serialization:");
		saveButton = new JButton("Save");
		loadButton = new JButton("Load");
		panel6 = new JPanel(new VerticalFlowLayout(VerticalFlowLayout.LEFT, VerticalFlowLayout.TOP, 5, 5));
		examplesList = new JList<String>();
		diagramView = new DiagramView();
		diagram = diagramView.getDiagram();

		// panel1
		panel1.add(label1);
		// label1
		label1.setText("This sample demonstrates the use of com.mindfusion.scripting " + 
			"in order to implement a dynamic process calculation.\n" +
			"To view one of the predefined samples, double-click on it in the list below. " +
			"To create your own nodes, use the buttons in the \"Creation\" section. " +
			"To run the process click \"Run\" or \"Step by step\".");
		label1.setLineWrap(true);
		label1.setEditable(false);
		label1.setWrapStyleWord(true);
		label1.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		label1.setOpaque(false);
		// panel2
		panel2.setPreferredSize(new Dimension(776, 116));
		panel2.add(panel3);
		panel2.add(panel4);
		panel2.add(panel5);
		panel2.add(panel6);
		// panel3
		panel3.setLocation(6, 6);
		panel3.setSize(566, 30);
		panel3.setBorder(BorderFactory.createLineBorder(Color.gray));
		panel3.add(label2);
		panel3.add(startButton);
		panel3.add(operationButton);
		panel3.add(choiceButton);
		panel3.add(endButton);
		panel3.add(Box.createHorizontalStrut(20));
		panel3.add(clearButton);
		// label2
		label2.setPreferredSize(new Dimension(85, 20));
		// startButton
		startButton.setPreferredSize(new Dimension(75, 22));
		startButton.setMargin(new Insets(0, 0, 0, 0));
		startButton.setBackground(new Color(0xC0, 0xD4, 0xEE));
		startButton.addActionListener(this);
		// operationButton
		operationButton.setPreferredSize(new Dimension(75, 22));
		operationButton.setMargin(new Insets(0, 0, 0, 0));
		operationButton.setBackground(new Color(0xC0, 0xD4, 0xEE));
		operationButton.addActionListener(this);
		// choiceButton
		choiceButton.setPreferredSize(new Dimension(75, 22));
		choiceButton.setMargin(new Insets(0, 0, 0, 0));
		choiceButton.setBackground(new Color(0xC0, 0xD4, 0xEE));
		choiceButton.addActionListener(this);
		// endButton
		endButton.setPreferredSize(new Dimension(75, 22));
		endButton.setMargin(new Insets(0, 0, 0, 0));
		endButton.setBackground(new Color(0xC0, 0xD4, 0xEE));
		endButton.addActionListener(this);
		// clearButton
		clearButton.setPreferredSize(new Dimension(75, 22));
		clearButton.setMargin(new Insets(0, 0, 0, 0));
		clearButton.setBackground(new Color(0xC0, 0xD4, 0xEE));
		clearButton.addActionListener(this);
		// panel4
		panel4.setLocation(6, 42);
		panel4.setSize(566, 30);
		panel4.setBorder(BorderFactory.createLineBorder(Color.gray));
		panel4.add(label3);
		panel4.add(validateButton);
		panel4.add(runButton);
		panel4.add(stepRunButton);
		panel4.add(stopButton);
		// label3
		label3.setPreferredSize(new Dimension(85, 20));
		// validateButton
		validateButton.setPreferredSize(new Dimension(75, 22));
		validateButton.setMargin(new Insets(0, 0, 0, 0));
		validateButton.setBackground(new Color(0xF0, 0xE6, 0x8C));
		validateButton.addActionListener(this);
		// runButton
		runButton.setPreferredSize(new Dimension(75, 22));
		runButton.setMargin(new Insets(0, 0, 0, 0));
		runButton.setBackground(new Color(0x9A, 0xCD, 0x32));
		runButton.addActionListener(this);
		// stepRunButton
		stepRunButton.setPreferredSize(new Dimension(75, 22));
		stepRunButton.setMargin(new Insets(0, 0, 0, 0));
		stepRunButton.setBackground(new Color(0x9A, 0xCD, 0x32));
		stepRunButton.addActionListener(this);
		// stopButton
		stopButton.setPreferredSize(new Dimension(75, 22));
		stopButton.setEnabled(false);
		stopButton.setMargin(new Insets(0, 0, 0, 0));
		stopButton.setBackground(new Color(0xFF, 0x7F, 0x50));
		stopButton.addActionListener(this);
		// panel5
		panel5.setLocation(6, 78);
		panel5.setSize(566, 30);
		panel5.setBorder(BorderFactory.createLineBorder(Color.gray));
		panel5.add(label8);
		panel5.add(saveButton);
		panel5.add(loadButton);
		// label8
		label8.setPreferredSize(new Dimension(85, 20));
		// saveButton
		saveButton.setPreferredSize(new Dimension(75, 22));
		saveButton.setMargin(new Insets(0, 0, 0, 0));
		saveButton.setBackground(new Color(0xC0, 0xD4, 0xEE));
		saveButton.addActionListener(this);
		// loadButton
		loadButton.setPreferredSize(new Dimension(75, 22));
		loadButton.setMargin(new Insets(0, 0, 0, 0));
		loadButton.setBackground(new Color(0xC0, 0xD4, 0xEE));
		loadButton.addActionListener(this);
		// panel6
		panel6.setLocation(580, 6);
		panel6.setSize(184, 102);
		panel6.add(new JLabel("Examples:"));
		panel6.add(examplesList);
		// examplesList
		examplesList.setPreferredSize(new Dimension(174, 77));
		examplesList.setBorder(BorderFactory.createLineBorder(Color.gray));
		// diagram
		diagram.setBackBrush(new SolidBrush(Color.white));
		diagram.setRouteLinks(true);
		diagram.setShowAnchors(ShowAnchors.Always);
		diagram.setSelectionOnTop(false);
		diagram.setLinkShape(LinkShape.Cascading);
		diagram.setLinkHeadShape(ArrowHeads.Triangle);
		diagram.setLinkHeadShapeSize(3);
		diagram.setShadowOffsetX(0);
		diagram.setShadowOffsetY(0);
		diagram.setLinkCustomDraw(CustomDraw.ShadowOnly);
		diagram.setLinkHandlesStyle(HandlesStyle.Custom);
		diagram.getDiagramStyle().setShadowBrush(new SolidBrush(new Color(0, 0, 0, 30)));
		diagram.getDiagramLinkStyle().setBrush(new SolidBrush(Color.BLACK));
		diagram.addDiagramListener(new DiagramListener());
		// diagramView
		diagramView.setBehavior(Behavior.DrawLinks);
		diagramView.setModificationStart(ModificationStart.AutoHandles);
		diagramView.addPropertyChangeListener(this);
		JScrollPane pane = new JScrollPane(diagramView);
		pane.addComponentListener(new ComponentAdapter()
		{
			@Override()
			public void componentResized(ComponentEvent e)
			{
				super.componentResized(e);
				updateVariableListPosition();
			}
		});

		Container c = getContentPane();
		c.add(panel1, BorderLayout.NORTH);
		c.add(panel2, BorderLayout.SOUTH);
		c.add(pane, BorderLayout.CENTER);
	}

	/**
	 * {@link ActionListener#actionPerformed} implementation.
	 */
	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();
		if (source == startButton)
		{
			diagram.getSelection().clear();

			for (DiagramNode n: diagram.getNodes())
			{
				if (!"start".equals(n.getTag()))
					continue;

				n.setSelected(true);
				diagramView.bringIntoView(n);

				return;
			}

			Point2D.Float zero = diagramView.deviceToDoc(0, 0);
			Point2D.Float five = diagramView.deviceToDoc(5, 0);
			float pixel5 = five.x - zero.x;

			ShapeNode node = diagram.getFactory().createShapeNode(
				20 + diagramView.getScrollX(), 20 + diagramView.getScrollY(), 24, 12);
			node.setShape(Shape.fromId("Rect"));
			node.setText("Start");
			node.setTag("start");
			node.setBrush(new SolidBrush(Colors.LimeGreen));
			Pen pen = new Pen(pixel5, Colors.Black);
			pen.setLineJoin(BasicStroke.JOIN_MITER);
			node.setPen(pen);
			node.setAllowIncomingLinks(false);
			node.setAllowOutgoingLinks(true);
			node.setSelected(true);
			node.setHandlesStyle(HandlesStyle.Invisible);
		}
		else if (source == operationButton)
		{
			diagram.getSelection().clear();

			JTextArea textArea = new JTextArea();
			textArea.setBorder(new LineBorder(new Color(0xE0, 0xE0, 0xE0), 5));
			ControlNode node = new ControlNode(diagramView, textArea);
			node.setTag("operation");
			node.setBounds(new Rectangle2D.Float(20 + diagramView.getScrollX(), 20 + diagramView.getScrollY(), 40, 30));
			node.setAnchorPattern(operationPattern);
			node.setSelected(true);
			node.setHandlesStyle(HandlesStyle.Invisible);
			diagram.getNodes().add(node);
		}
		else if (source == choiceButton)
		{
			diagram.getSelection().clear();

			JTextArea textArea = new JTextArea();
			textArea.setBorder(new LineBorder(new Color(0xE0, 0xE0, 0xE0), 5));
			ControlNode node = new ControlNode(diagramView, textArea);
			node.setTag("choice");
			node.setBounds(new Rectangle2D.Float(20 + diagramView.getScrollX(), 20 + diagramView.getScrollY(), 40, 30));
			node.setAnchorPattern(choicePattern);
			node.setSelected(true);
			node.setHandlesStyle(HandlesStyle.Invisible);
			diagram.getNodes().add(node);
		}
		else if (source == endButton)
		{
			diagram.getSelection().clear();

			for (DiagramNode n: diagram.getNodes())
			{
				if (!"end".equals(n.getTag()))
					continue;

				n.setSelected(true);
				diagramView.bringIntoView(n);

				return;
			}

			Point2D.Float zero = diagramView.deviceToDoc(0, 0);
			Point2D.Float five = diagramView.deviceToDoc(5, 0);
			float pixel5 = five.x - zero.x;

			ShapeNode node = diagram.getFactory().createShapeNode(
				20 + diagramView.getScrollX(), 20 + diagramView.getScrollY(), 24, 12);
			node.setShape(Shape.fromId("Rect"));
			node.setText("End");
			node.setTag("end");
			node.setBrush(new SolidBrush(Colors.LightSalmon));
			Pen pen = new Pen(pixel5, Colors.Black);
			pen.setLineJoin(BasicStroke.JOIN_MITER);
			node.setPen(pen);
			node.setAllowIncomingLinks(true);
			node.setAllowOutgoingLinks(false);
			node.setSelected(true);
			node.setHandlesStyle(HandlesStyle.Invisible);
		}
		else if (source == clearButton)
		{
			for (int i = 0; i < diagram.getNodes().size(); )
			{
				DiagramNode node = diagram.getNodes().get(i);
				if ("variableList".equals(node.getTag()))
				{
					i++;
					continue;
				}

				diagram.getNodes().remove(diagram.getNodes().get(i));
			}
		}
		else if (source == validateButton)
		{
			if (validateProcess())
			{
				JOptionPane.showMessageDialog(this, "Validation successful!",
					"Validation", JOptionPane.INFORMATION_MESSAGE);
			}
		}
		else if (source == runButton)
		{
			variableListNode.setText("");
			variableListNode.resizeToFitText(FitSize.KeepRatio);
			evaluationContext.getLocalVars().clear();

			if (!validateProcess())
				return;

			DiagramNode node = findStartNode();

			while (true)
			{
				try
				{
					node = process(node);
				}
				catch (Exception ex)
				{
					JOptionPane.showMessageDialog(this, ex.getMessage(),
						"Execution Error", JOptionPane.ERROR_MESSAGE);
					return;
				}

				if (node == null)
					break;
			}
		}
		else if (source == stepRunButton)
		{
			if (!tracing)
			{
				variableListNode.setText("");
				variableListNode.resizeToFitText(FitSize.KeepRatio);
				evaluationContext.getLocalVars().clear();

				if (!validateProcess())
					return;

				tracing = true;

				// Update UI
				startButton.setEnabled(false);
				operationButton.setEnabled(false);
				choiceButton.setEnabled(false);
				endButton.setEnabled(false);
				clearButton.setEnabled(false);
				validateButton.setEnabled(false);
				runButton.setEnabled(false);
				saveButton.setEnabled(false);
				loadButton.setEnabled(false);

				stepRunButton.setText("Next");
				stopButton.setEnabled(true);

				variableListNode.setVisible(true);

				currentNode = findStartNode();
				diagram.getSelection().change(currentNode);
				diagramView.bringIntoView(currentNode);
			}
			else
			{
				try
				{
					DiagramNode previousNode = currentNode;
					currentNode = process(previousNode);

					if (currentNode != null)
					{
						diagram.getSelection().change(currentNode);
						diagramView.bringIntoView(currentNode);
					}
					else
					{
						stopRun();
					}
				}
				catch (Exception ex)
				{
					JOptionPane.showMessageDialog(this, ex.getMessage(),
						"Execution Error", JOptionPane.INFORMATION_MESSAGE);
					stopRun();
				}
			}
		}
		else if (source == stopButton)
		{
			stopRun();
		}
		else if (source == saveButton)
		{
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new FileNameExtensionFilter("XML Files", "xml"));
			if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				try
				{
					diagramView.saveToXml(chooser.getSelectedFile().getAbsolutePath());
				}
				catch (XmlException ex)
				{
					JOptionPane.showMessageDialog(this, "Failed to save: " + ex.getMessage(),
						"Save document", JOptionPane.ERROR_MESSAGE);
				}
				catch (IOException ex)
				{
					JOptionPane.showMessageDialog(this, "Failed to save: " + ex.getMessage(),
						"Save document", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		else if (source == loadButton)
		{
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new FileNameExtensionFilter("XML Files", "xml"));
			if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				try
				{
					diagramView.loadFromXml(chooser.getSelectedFile().getAbsolutePath());
					onDiagramLoaded();
				}
				catch (XmlException ex)
				{
					JOptionPane.showMessageDialog(this, "Failed to load: " + ex.getMessage(),
						"Open document", JOptionPane.ERROR_MESSAGE);
				}
				catch (IOException ex)
				{
					JOptionPane.showMessageDialog(this, "Failed to load: " + ex.getMessage(),
						"Open document", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	private void onDiagramLoaded()
	{
		// Obtain a reference to the new variable list node
		for (DiagramNode node: diagram.getNodes())
		{
			if ("variableList".equals(node.getTag()))
			{
				variableListNode = (ShapeNode)node;
				variableListNode.zTop(false);
			}

			if (node instanceof ControlNode)
			{
				((JTextArea)((ControlNode)node).getControl()).setBorder(
					new LineBorder(node.getSelected() ? Color.orange : new Color(0xE0, 0xE0, 0xE0), 5));
			}
		}

		// Link handles style appear to not be persisted
		for (DiagramLink link: diagram.getLinks())
			link.setHandlesStyle(HandlesStyle.Custom);
		diagram.setLinkHandlesStyle(HandlesStyle.Custom);
	}

	/**
	 * {@link SimpleEvaluationContextListener#dispatchFunctionCall} implementation.
	 */
	public void dispatchFunctionCall(EvaluateFunctionEventObject e)
	{
		if ("MessageBox".equals(e.getName()))
		{
			e.setResult(null);
			if (e.getArguments().length > 0 && e.getArguments()[0] != null)
			{
				JOptionPane.showMessageDialog(this, e.getArguments()[0].toString());
			}
		}

		else if ("Prompt".equals(e.getName()))
		{
			String message = "";
			if (e.getArguments().length > 0 && e.getArguments()[0] != null)
				message = e.getArguments()[0].toString();
			String result = JOptionPane.showInputDialog(this, message);
			if (!"".equals(result))
				e.setResult(result);
			else
				e.setResult(null);
		}
	}

	/**
	 * {@link PropertyChangeListener#propertyChange} implementation.
	 */
	public void propertyChange(PropertyChangeEvent evt)
	{
		if (evt.getSource() == diagramView)
		{
			if ("scrollX".equals(evt.getPropertyName()) ||
				"scrollY".equals(evt.getPropertyName()))
			{
				updateVariableListPosition();
			}
		}
	}

	/**
	 * {@link ListSelectionListener#valueChanged} implementation.
	 */
	public void valueChanged(ListSelectionEvent e)
	{
		if (examplesList.getSelectedIndex() != -1)
		{
			String name = examplesList.getSelectedValue();
			String content = "";

			if ("Factorial".equals(name))
				content = "Factorial.xml";
			else if ("Gcd".equals(name))
				content = "Gcd.xml";
			else if ("Fibonacci".equals(name))
				content = "Fib.xml";

			if (!"".equals(content))
			{
				try
				{
					diagramView.loadFromXml(App.class.getResourceAsStream(content));
					onDiagramLoaded();
				}
				catch (XmlException ex)
				{
					JOptionPane.showMessageDialog(this, "Failed to load: " + ex.getMessage(),
						"Open document", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	private final class DiagramListener extends DiagramAdapter
	{
		@Override()
		public void drawAnchorPoint(DrawAnchorPointEvent e)
		{
			super.drawAnchorPoint(e);

			final float size = 1.5f;
			Rectangle2D.Double bounds = new Rectangle2D.Double(e.getLocation().getX() - size,
				e.getLocation().getY() - size, 2 * size, 2 * size);
			AnchorPoint point = e.getAnchorPattern().getPoints().get(e.getAnchorIndex());
			Graphics2D g = e.getGraphics();

			switch ((Integer)point.getTag())
			{

				case 0: // Top-in
				case 3: // Bottom-out
					{
						Point2D.Double[] triangle = new Point2D.Double[]
						{
							new Point2D.Double(bounds.getMinX(), bounds.getMinY()),
							new Point2D.Double(bounds.getMinX()+ bounds.getWidth() / 2, bounds.getMaxY()),
							new Point2D.Double(bounds.getMaxX(), bounds.getMinY()),
							new Point2D.Double(bounds.getMinX(), bounds.getMinY()),
						};

						GeneralPath path = new GeneralPath();
						path.append(new Line2D.Double(triangle[0], triangle[1]), true);
						path.append(new Line2D.Double(triangle[1], triangle[2]), true);
						path.append(new Line2D.Double(triangle[2], triangle[3]), true);
						path.append(new Line2D.Double(triangle[3], triangle[0]), true);

						Brush brush = new SolidBrush(new Color(0, 0, 0, 100));
						brush.applyTo(g, null);
						g.fill(path);
						Pen pen = new Pen(0.4f, Color.black);
						pen.applyTo(g);
						g.draw(path);
					}
					break;

				case 1: // True
					{
						Ellipse2D ellipse = new Ellipse2D.Double(bounds.x, bounds.y, bounds.width, bounds.height);

						Brush brush = new SolidBrush(new Color(0, 128, 0, 150));
						brush.applyTo(g, null);
						g.fill(ellipse);
						Pen pen = new Pen(0.4f, Color.black);
						pen.applyTo(g);
						g.draw(ellipse);

						pen = new Pen(0.4f, Color.white);
						pen.applyTo(g);
						g.draw(new Line2D.Double(bounds.getMinX() + 0.75, bounds.getMinY() + bounds.getHeight() / 2,
							bounds.getMaxX() - 0.75, bounds.getMinY() + bounds.getHeight() / 2));
						g.draw(new Line2D.Double(bounds.getMinX() + bounds.getWidth() / 2, bounds.getMinY() + 0.75,
							bounds.getMinX() + bounds.getWidth() / 2, bounds.getMaxY() - 0.75));
					}
					break;

				case 2: // False
					{
						Ellipse2D ellipse = new Ellipse2D.Double(bounds.x, bounds.y, bounds.width, bounds.height);

						Brush brush = new SolidBrush(new Color(255, 0, 0, 150));
						brush.applyTo(g, null);
						g.fill(ellipse);
						Pen pen = new Pen(0.4f, Color.black);
						pen.applyTo(g);
						g.draw(ellipse);

						pen = new Pen(0.4f, Color.white);
						pen.applyTo(g);
						g.draw(new Line2D.Double(bounds.getMinX() + 0.75, bounds.getMinY() + bounds.getHeight() / 2,
							bounds.getMaxX() - 0.75, bounds.getMinY() + bounds.getHeight() / 2));
					}
					break;

			}
		}

		@Override()
		public void drawLink(DrawLinkEvent e)
		{
			super.drawLink(e);

			if (e.getLink().getSelected())
			{
				PointList points = e.getPoints();
				GeneralPath path = new GeneralPath();
				for (int i = 0; i < points.size() - 1; i++)
					path.append(new Line2D.Float(points.get(i), points.get(i + 1)), true);
				Pen pen = new Pen(1, Colors.Orange);
				pen.applyTo(e.getGraphics());
				e.getGraphics().draw(path);
			}
		}

		@Override()
		public void nodeSelected(NodeEvent e)
		{
			super.nodeSelected(e);

			if (e.getNode() instanceof ShapeNode)
			{
				((ShapeNode)e.getNode()).getPen().setColor(Colors.Orange);
			}
			else if (e.getNode() instanceof ControlNode)
			{
				JTextArea control = (JTextArea)((ControlNode)e.getNode()).getControl();
				control.setBorder(new LineBorder(Colors.Orange, 5));
			}
		}

		@Override()
		public void nodeDeselected(NodeEvent e)
		{
			super.nodeDeselected(e);

			if (e.getNode() instanceof ShapeNode)
			{
				((ShapeNode)e.getNode()).getPen().setColor(Colors.Black);
			}
			else if (e.getNode() instanceof ControlNode)
			{
				JTextArea control = (JTextArea)((ControlNode)e.getNode()).getControl();
				control.setBorder(new LineBorder(new Color(0xE0, 0xE0, 0xE0), 5));
			}
		}

		@Override()
		public void linkCreating(LinkValidationEvent e)
		{
			super.linkCreating(e);

			// Prevent self loops
			if (e.getOrigin() == e.getDestination())
			{
				e.setCancel(true);
				return;
			}

			// Prevent two links going out of the same anchor point
			for (DiagramLink link: e.getLink().getOrigin().getOutgoingLinks())
			{
				if (link.getOriginAnchor() == e.getLink().getOriginAnchor())
				{
					e.setCancel(true);
					return;
				}
			}
		}

		@Override()
		public void nodeCreated(NodeEvent e)
		{
			super.nodeCreated(e);
			variableListNode.zTop(false);
		}

		@Override()
		public void nodeClicked(NodeEvent e)
		{
			super.nodeClicked(e);
			if (e.getNode() instanceof ControlNode)
			{
				ControlNode node = (ControlNode)e.getNode();
				node.getControl().requestFocus();
			}
		}
	}

	private void updateVariableListPosition()
	{
		Rectangle clientViewport = diagramView.getVisibleRect();
		Rectangle2D.Float viewport = diagramView.deviceToDoc(clientViewport);

		variableListNode.setBounds(
			(float)viewport.getMaxX() - variableListNode.getBounds().width - 2,
			(float)viewport.getMaxY()  - variableListNode.getBounds().height - 2,
			variableListNode.getBounds().width, variableListNode.getBounds().height);
	}

	private boolean validateProcess()
	{
		// Check for connectivity
		Graph graph = new Graph();
		HashMap<DiagramNode, Vertex> vertexMap = new HashMap<DiagramNode, Vertex>();
		HashMap<DiagramLink, Edge> edgeMap = new HashMap<DiagramLink, Edge>();
		for (DiagramNode node: diagram.getNodes())
		{
			if (node == variableListNode)
				continue;

			String tag = String.valueOf(node.getTag());
			if (tag.startsWith("function:"))
				continue;

			vertexMap.put(node, graph.addVertex());
		}

		for (DiagramLink link: diagram.getLinks())
		{
			Vertex origin = vertexMap.get(link.getOrigin());
			Vertex destination = vertexMap.get(link.getDestination());

			edgeMap.put(link, graph.addEdge(origin, destination));
		}

		if (graph.getConnectedComponents().length > 1)
		{
			JOptionPane.showMessageDialog(this, "Validation failed! The process contains disconnected components.",
				"Validation", JOptionPane.ERROR_MESSAGE);
			return false;
		}

		// Check for start/end nodes
		boolean hasStart = false;
		boolean hasEnd = false;
		for (DiagramNode node: diagram.getNodes())
		{
			if ("start".equals(node.getTag()))
				hasStart = true;
			else if ("end".equals(node.getTag()))
				hasEnd = true;
		}

		if (!hasStart)
		{
			JOptionPane.showMessageDialog(this, "Validation failed! The process does not have a start node.",
				"Validation", JOptionPane.ERROR_MESSAGE);
			return false;
		}

		if (!hasEnd)
		{
			JOptionPane.showMessageDialog(this, "Validation failed! The process does not have an end node.",
				"Validation", JOptionPane.ERROR_MESSAGE);
			return false;
		}

		// Check for missing links - all out anchors must have links
		for (DiagramNode node: diagram.getNodes())
		{
			if ("choice".equals(node.getTag()))
			{
				if (node.getIncomingLinks().size() == 0)
				{
					diagram.getSelection().change(node);
					JOptionPane.showMessageDialog(this, "Validation failed! A choice has an undefined input.",
						"Validation", JOptionPane.ERROR_MESSAGE);
					return false;
				}
				if (node.getOutgoingLinks().size() != 2)
				{
					diagram.getSelection().change(node);
					JOptionPane.showMessageDialog(this, "Validation failed! A choice has an undefined output.",
						"Validation", JOptionPane.ERROR_MESSAGE);
					return false;
				}
			}
			else if ("operation".equals(node.getTag()))
			{
				if (node.getIncomingLinks().size() == 0)
				{
					diagram.getSelection().change(node);
					JOptionPane.showMessageDialog(this, "Validation failed! An operation has an undefined input.",
						"Validation", JOptionPane.ERROR_MESSAGE);
					return false;
				}
				if (node.getOutgoingLinks().size() != 1)
				{
					diagram.getSelection().change(node);
					JOptionPane.showMessageDialog(this, "Validation failed! An operation has an undefined output.",
						"Validation", JOptionPane.ERROR_MESSAGE);
					return false;
				}
			}
		}

		return true;
	}

	private DiagramNode process(DiagramNode node)
	{
		if ("end".equals(node.getTag()))
			return null;

		if ("start".equals(node.getTag()))
			return node.getOutgoingLinks().get(0).getDestination();

		if (node instanceof ControlNode)
		{
			String code = ((JTextArea)((ControlNode)node).getControl()).getText();
			Object result = execute(code);

			if ("operation".equals(node.getTag()))
				return node.getOutgoingLinks().get(0).getDestination();

			if ("choice".equals(node.getTag()))
			{
				if (result == null)
					throw new Error("Failed to evaluate choice.");

				boolean bresult = false;
				if (result instanceof Boolean)
					bresult = ((Boolean)result).booleanValue();
				else
					bresult = Boolean.parseBoolean(String.valueOf(result));
				if (bresult)
					return findTrueLink(node).getDestination();
				else
					return findFalseLink(node).getDestination();
			}
		}

		throw new Error("Invalid process.");
	}

	private Object execute(String expression)
	{
		return execute(expression, evaluationContext);
	}

	private Object execute(String expression, SimpleEvaluationContext context)
	{
		try
		{
			ByteCode code = Compiler.compile(expression, true);
			Object result = code.evaluate(context);

			// Update variables
			StringBuilder text = new StringBuilder();
			for (String varName: context.getLocalVars().keySet())
				text.append(String.format("%s: %s\n", varName, context.getLocalVars().get(varName)));
			variableListNode.setText(text.toString());
			variableListNode.resizeToFitText(FitSize.KeepRatio);
			updateVariableListPosition();
	
			return result;
		}
		catch (Exception ex)
		{
			return null;
		}
	}

	private void stopRun()
	{
		variableListNode.setVisible(false);

		diagram.getSelection().clear();

		startButton.setEnabled(true);
		operationButton.setEnabled(true);
		choiceButton.setEnabled(true);
		endButton.setEnabled(true);
		clearButton.setEnabled(true);
		validateButton.setEnabled(true);
		runButton.setEnabled(true);
		saveButton.setEnabled(true);
		loadButton.setEnabled(true);

		stepRunButton.setText("Step by step");
		stopButton.setEnabled(false);

		tracing = false;
	}

	private DiagramNode findStartNode()
	{
		for (DiagramNode node: diagram.getNodes())
		{
			if ("start".equals(node.getTag()))
				return node;
		}

		return null;
	}

	private DiagramLink findTrueLink(DiagramNode node)
	{
		for (DiagramLink link: node.getOutgoingLinks())
		{
			if (link.getOriginAnchor() == 1)
				return link;
		}

		return null;
	}

	private DiagramLink findFalseLink(DiagramNode node)
	{
		for (DiagramLink link: node.getOutgoingLinks())
		{
			if (link.getOriginAnchor() == 2)
				return link;
		}

		return null;
	}


	private static final long serialVersionUID = 1L;

	private JPanel panel1;
	private JTextArea label1;
	private JPanel panel2;
	private JPanel panel3;
	private JLabel label2;
	private JButton startButton;
	private JButton operationButton;
	private JButton choiceButton;
	private JButton endButton;
	private JButton clearButton;
	private JPanel panel4;
	private JLabel label3;
	private JButton validateButton;
	private JButton runButton;
	private JButton stepRunButton;
	private JButton stopButton;
	private JPanel panel5;
	private JLabel label8;
	private JButton saveButton;
	private JButton loadButton;
	private JPanel panel6;
	private JList<String> examplesList;
	private DiagramView diagramView;
	private Diagram diagram;

	private SimpleEvaluationContext evaluationContext;
	private AnchorPattern operationPattern;
	private AnchorPattern choicePattern;
	private ShapeNode variableListNode;
	private boolean tracing;
	private DiagramNode currentNode;
}

final class Colors
{
	public static final Color Black = new Color(0, 0, 0);
	public static final Color LimeGreen = new Color(0x32, 0xCD, 0x32);
	public static final Color LightSalmon = new Color(0xFF, 0xA0, 0x7A);
	public static final Color LightSteelBlue = new Color(0xB0, 0xC4, 0xDE);
	public static final Color DarkGoldenrod = new Color(0xB8, 0x86, 0x0B);
	public static final Color Orange = new Color(0xFF, 0xA5, 0x00);
}
