import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.*;
import java.awt.geom.Point2D;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.*;
import javax.swing.SpringLayout;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.mindfusion.diagramming.*;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import java.io.*;

public class SiteMapFrame extends JFrame
{
	public SiteMapFrame()
	{
		super("MindFusion.Diagramming sample: Site Map");
		initFrame();
		initDiagram();
		startNewSite();
	}

	private void initDiagram()
	{
		diagram.setBackBrush(new SolidBrush(new Color(20, 160, 160)));
		diagram.setAllowUnconnectedLinks(true);
		diagramView.setBehavior(Behavior.LinkShapes);
		diagram.setShowGrid(true);
		diagram.setDefaultShape(Shape.fromId("RoundRect"));
		diagram.getShapeNodeStyle().setFontFamily("Arial");
		diagram.getShapeNodeStyle().setFontSize(3);
		diagram.setGridColor(new Color(120, 120, 120));
		diagram.setAllowSelfLoops(false);
		diagram.getShapeNodeStyle().setTextBrush(new SolidBrush(Color.blue));
		
		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void nodeCreated(NodeEvent e)
			{
				if (e.getNode() instanceof ShapeNode)
				{
					ShapeNode node = (ShapeNode)e.getNode();
					onNodeCreated(node);
				}
			}
			public void linkCreated(LinkEvent e)
			{
					DiagramLink link = e.getLink();
					onLinkCreated(link);
			}
			
			public void nodeClicked(NodeEvent e)
			{
				onNodeClicked(e);
			}
			
			public void nodeDeleting(NodeValidationEvent e)
			{
				if (e.getNode() instanceof ShapeNode)
					onNodeDeleted(e);
			}
			
			public void serializeTag(SerializeTagEvent e)
			{
				// TODO Auto-generated method stub
				serialize(e);
			}
			public void deserializeTag(SerializeTagEvent e)
			{
				deserialize(e);
			}
			
		});
	}

	private void startNewSite()
	{
		// create the home page properties
		PageProps props = new PageProps();
		props.title = "Home";
		props.file = "index.html";
		props.inf = -1;
		
		// create a box and associate it with the home page
		rootBox = diagram.getFactory().createShapeNode(10, 10, 24, 14);
		rootBox.setText("Home");
		rootBox.setTag(props);

		// deselect the box; it gets selected because AutoSelect is turned on
		diagram.getSelection().clear();

		fileNum = 1;
	}

	// display 'property page' for a node
	private void setPageProps(ShapeNode node)
	{
		PagePropsFrame frame = new PagePropsFrame(this);

		// get the page attributes
		
		PageProps props = (PageProps)(node.getTag());

		// fill the form fields
		frame.txFldTitle.setText(props.title);
		frame.txFldFileName.setText(props.file);
		frame.txFldImageOne.setText(props.img1Path);
		frame.txFldImageTwo.setText(props.img2Path);

		frame.setVisible(true);
		
		if (frame.getAccepted() == true)
		{
			// get info from form fields
			node.setText(frame.txFldTitle.getText());
			props.title = frame.txFldTitle.getText();
			props.file = frame.txFldFileName.getText();
			props.img1Path = frame.txFldImageOne.getText();
			props.img2Path = frame.txFldImageTwo.getText();

			node.setTag(props);
		}
	}

	private void onNodeCreated(ShapeNode node)
	{
		// create page object for the new box
		PageProps props = new PageProps();
		props.file = "file" + fileNum + ".html";
		props.title = "Page " + fileNum;
		node.setTag(props);

		// display the properties
		setPageProps(node);

		fileNum++;
	}
	
	private void onLinkCreated(DiagramLink link)
	{
		if (link.getDestination() instanceof DummyNode)
		{
			Point2D.Float pt = link.getControlPoints().get(link.getControlPoints().size() - 1);
			ShapeNode newBox = diagram.getFactory().createShapeNode(pt.x - 10, pt.y - 7, 24, 14);
			onNodeCreated(newBox);
			link.setDestination(newBox);
		}
		
	}
	
	ShapeNode currNode = null;
	private void onNodeClicked(NodeEvent e)
	{
		// we handle only right-clicks
		if (e.getMouseButton() != MouseEvent.BUTTON3)
			return;

		if (!(e.getNode() instanceof ShapeNode))
			return;
		
		// save the box reference
		currNode = (ShapeNode)e.getNode();

		Point mousePos = getMousePosition();

		// display context menu for the box
		cmBox.show(this, mousePos.x, mousePos.y);
	}
	
	private void onNodeDeleted(NodeValidationEvent e)
	{
		// get the associated page
		PageProps props = (PageProps)((ShapeNode)e.getNode()).getTag();

		// if that is the homepage ...
		if (props.inf == -1)
		{
			// ... do not delete it
			e.setCancel(true);
			JOptionPane.showMessageDialog(this, "Home page cannot be deleted!");
		}
	}
	
	private void onFileNew()
	{
		diagram.clearAll();
		startNewSite();
	}
	
	private void onFileLoad()
	{
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				diagram.loadFromXml(chooser.getSelectedFile().getAbsolutePath());
			}
			catch (FileNotFoundException e1)
			{
				e1.printStackTrace();
			} 
			catch (IOException e1)
			{
				e1.printStackTrace();
			}
			catch (XmlException e1)
			{
				e1.printStackTrace();
			}
		}
	}
	
	private void onFileSave()
	{
		if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
    	{
			try
			{
				diagram.saveToXml(chooser.getSelectedFile().getAbsolutePath());
			} 
			catch (FileNotFoundException e1)
			{
				e1.printStackTrace();
			}
			catch (IOException e1)
			{
				e1.printStackTrace();
			} 
			catch (XmlException e1)
			{
				e1.printStackTrace();
			}
		}
	}
	
	private void serialize(SerializeTagEvent e)
	{
		Object tag = e.getTag();
		if(tag instanceof PageProps)
		{
			e.setHandled(true);
			PageProps props = (PageProps)tag;
			Document doc = e.getTagElement().getOwnerDocument();

			Element element = doc.createElement("Title");
			org.w3c.dom.Text node = doc.createTextNode(props.title);
			element.appendChild(node);
			e.getTagElement().appendChild(element);

			element = doc.createElement("File");
			node = doc.createTextNode(props.file);
			element.appendChild(node);
			e.getTagElement().appendChild(element);

			element = doc.createElement("ImagePath1");
			node = doc.createTextNode(props.img1Path);
			element.appendChild(node);
			e.getTagElement().appendChild(element);

			element = doc.createElement("ImagePath2");
			node = doc.createTextNode(props.img2Path);
			element.appendChild(node);
			e.getTagElement().appendChild(element);

			element = doc.createElement("Inf");
			node = doc.createTextNode(String.valueOf(props.inf));
			element.appendChild(node);
			e.getTagElement().appendChild(element);
		}
	}

	private void deserialize(SerializeTagEvent e)
	{
		Element tag = e.getTagElement();
		
		if ("Tag".equals(tag.getNodeName()) && tag.getChildNodes().getLength() > 0)
		{
			e.setHandled(true);
			PageProps props = new PageProps();
			
			props.title = readText(tag, "Title", "");
			props.file = readText(tag, "File", "");
			props.img1Path = readText(tag, "ImagePath1", "");
			props.img2Path = readText(tag, "ImagePath2", "");
			props.inf = readInt(tag, "Inf");
			
			e.setTag(props);
		}				
	}

	private Element getChildElement(Element element, String name)
	{
		org.w3c.dom.NodeList children = element.getChildNodes();
	    for (int i = 0; i < children.getLength(); ++i)
	    {
	        org.w3c.dom.Node node = children.item(i);
	        if (node.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE &&
	            node.getNodeName().equals(name))
	            return (Element)children.item(i);
	    }
	
	    return null;
	}

    private String readText(Element element, String name, String defaultValue)
    {
        Element contElement = getChildElement(element, name);
        if (contElement != null && contElement.getFirstChild() != null)
            return contElement.getFirstChild().getNodeValue();

        return defaultValue;
    }

    private int readInt(Element element, String name)
    {
        Element contElement = getChildElement(element, name);
        if (contElement != null && contElement.getFirstChild() != null)
            return Integer.parseInt(contElement.getFirstChild().getNodeValue());

        return 0;
    }
	
	private void onMenuDeleted()
	{
		PageProps props = (PageProps)currNode.getTag();
		if (props.inf == -1)
			JOptionPane.showMessageDialog(this, "Home page cannot be deleted!");
		else
			diagram.getNodes().remove(currNode);
	}
	
	private void onMenuView()
	{
		PageProps props = (PageProps)currNode.getTag();
		
		try
		{
			Runtime.getRuntime().exec( 
					new String[]{"C:\\Program Files\\Internet Explorer\\iexplore.exe", props.file});
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	private void onMenuGenerate()
	{
		if (rootBox.getOutgoingLinks().size() == 0)
		{
			JOptionPane.showMessageDialog(this, "There are not links from the start page!");
			return;
		}

		files.clear();
		generateHtml(rootBox);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension propViewDim = new Dimension(230, 290);
		int x = 10, offsetX = 10;
		int y = 10, offsetY = 10;
		int step = 30;
		
		Set set = files.entrySet();
		Iterator it = set.iterator();
		// display the generated html files
		while(it.hasNext())
		{
			if (offsetX > screenSize.width - propViewDim.width - 10 ||
					offsetY > screenSize.height - propViewDim.width - 10)
			{
				x += 2*step;
				offsetX = x;
				offsetY = y;
			}
			
			Map.Entry e = (Map.Entry) it.next();
			String file = (String)e.getKey();
			
			PageViewFrame frame = new PageViewFrame(offsetX, offsetY);
			frame.setTitle(file);
			frame.textArea.setText((String)files.get(file));
			frame.setVisible(true);
			
			offsetX += step;
			offsetY += step;
		}
	}
	
//	 generate HTML code for a page
	private void generateHtml(ShapeNode root)
	{
		PageProps props = (PageProps)root.getTag();

		// if not generated yet ...
		if (files.containsKey(props.file) == false)
		{
			String html = "";
			html += "<HTML>\r\n";
			html += "<HEAD>\r\n";
			html += "<TITLE>" + props.title	+ "</TITLE>\r\n";
			html += "</HEAD>\r\n\r\n";

			html += "<BODY>\r\n\r\n";

			// add link to the parent page
			if (root.getIncomingLinks().size() > 0)
			{
				PageProps parentProps =
					(PageProps)root.getIncomingLinks().get(0).getOrigin().getTag();
				html += "<A HREF=\"" + parentProps.file + "\">Level up: ";
				html += parentProps.title + "</A><BR>\r\n";
			}

			// add link to all child pages
			for (DiagramLink link : root.getOutgoingLinks())
			{
				PageProps linkProps =
					(PageProps)link.getDestination().getTag();
				html += "<A HREF=\"" + linkProps.file + "\">Level down: ";
				html += linkProps.title + "</A><BR>\r\n";
			}

			html += "\r\n</BODY>\r\n";
			html += "</HTML>\r\n";

			// save the generated HTML
			files.put(props.file, html);

			// call recursively for all children
			for(DiagramLink link : root.getOutgoingLinks())
			{
				generateHtml((ShapeNode)link.getDestination());
			}
		}
	}

	
	class PageProps implements Serializable
	{
		String title = "New page";
		String file = "";
		String img1Path = "";
		String img2Path = "";
		int inf = 0;
	}
	
	private void initFrame()
	{
		String textInfo =
			"This sample demonstrates how you could let users create flowharts faster by " +
			"just drawing links from existing nodes to empty area on the canvas. The LinkCreated event " +
			"handler checks where the link end point is, creates a node there and connects it with the link.";
		
		mBar = new JMenuBar();
		mFile = new JMenu();
		mTools = new JMenu();
		mExit = new JMenuItem();
		mFNew = new JMenuItem();
		mFLoad  = new JMenuItem();
		mFSave = new JMenuItem();
		mTPages = new JMenuItem();
		txArInfo = new JTextArea();
		
		diagram = new Diagram();
		diagramView = new DiagramView(diagram);

		txArScrollPane = new JScrollPane(txArInfo);
		diagramScrP = new JScrollPane(diagramView);
		
		cmBox = new JPopupMenu();
		miProps = new JMenuItem();
		miDelete = new JMenuItem();
		miView = new JMenuItem();
		
		chooser = new JFileChooser();
		
		this.setJMenuBar(mBar);
		//
		// mBar
		//
		mBar.setName("mBar");
		mBar.add(mFile);
		mBar.add(mTools);
		
		//
		// mFile
		//
		mFile.setName("mFile");
		mFile.setText("File");
		mFile.add(mFNew);
		mFile.add(mFLoad);
		mFile.add(mFSave);
		mFile.addSeparator();
		mFile.add(mExit);
		
		//
		// mFNew
		//
		mFNew.setName("mFNew");
		mFNew.setText("New");
		mFNew.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				onFileNew();
			}
		});
		
		//
		// mFLoad
		//
		mFLoad.setName("mFLoad");
		mFLoad.setText("Load");
		mFLoad.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				onFileLoad();
			}
		});
		
		//
		// mFSave
		//
		mFSave.setName("mFSave");
		mFSave.setText("Save");
		mFSave.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				onFileSave();
			}
		});
		
		
		//
		// mTools
		//
		mTools.setName("mTools");
		mTools.setText("Tools");
		mTools.add(mTPages);
		
		//
		// mTPages
		//
		mTPages.setName("mTPages");
		mTPages.setText("Generate pages");
		mTPages.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				onMenuGenerate();
			}
		});
		
		//
		// mExit
		//
		mExit.setName("mExit");
		mExit.setText("Exit");
		mExit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				System.exit(0);
			}
		});

		//
		// cmBox
		//
		cmBox.setName("cmBox");
        cmBox.add(miProps);
        cmBox.add(miDelete);
        cmBox.add(miView);

        //
        // miProps
        //
        miProps.setName("miProps");
        miProps.setText("Properties");
        miProps.addActionListener(new ActionListener()
        {
			public void actionPerformed(ActionEvent arg0) 
			{
				setPageProps(currNode);
			}
        });

        //
        // miDelete
        //
        miDelete.setName("miDelete");
        miDelete.setText("Delete");
        miDelete.addActionListener(new ActionListener()
        {
			public void actionPerformed(ActionEvent arg0) 
			{
				onMenuDeleted();
			}
        });
                
        //
        // miView
        //
        miView.setName("miView");
        miView.setText("View");
        miView.addActionListener(new ActionListener()
        {
			public void actionPerformed(ActionEvent arg0) 
			{
				onMenuView();
			}
        });
        
		//
		// txArInfo
		//
		txArInfo.setName("txArInfo");
		txArInfo.setText(textInfo);
		txArInfo.setEditable(false);
		txArInfo.setBackground(Color.white);
		txArInfo.setWrapStyleWord(true);
		txArInfo.setLineWrap(true);
		txArInfo.setAutoscrolls(true);

		Container cp = getContentPane();
		cp.setLayout(null);
		cp.add(diagramScrP);
		cp.add(txArScrollPane);
		
		SpringLayout layout = new SpringLayout();
		layout.putConstraint(SpringLayout.SOUTH, txArScrollPane, 0, SpringLayout.SOUTH, cp);
		layout.putConstraint(SpringLayout.EAST, txArScrollPane, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.WEST, txArScrollPane, 0, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.NORTH, txArScrollPane, -50, SpringLayout.SOUTH, cp);
		
		layout.putConstraint(SpringLayout.EAST, diagramScrP, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.SOUTH, diagramScrP, 0, SpringLayout.NORTH, txArScrollPane);
		layout.putConstraint(SpringLayout.WEST, diagramScrP, 0, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.NORTH, diagramScrP, 0, SpringLayout.NORTH, cp);
		
		cp.setLayout(layout);
		
		this.addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				System.exit(0);
			}
		});
		
		setSize(650, 550);
	}

	HashMap files = new HashMap();
	
	// the initial node
	private ShapeNode rootBox = null;

	// page counter
	private int fileNum = 1;
	
	private JMenuBar mBar;
	private JMenu mFile;
	private JMenu mTools;
	private JMenuItem mExit;
	private JMenuItem mFNew;
	private JMenuItem mFLoad;
	private JMenuItem mFSave;
	private JMenuItem mTPages;
	
	private JTextArea txArInfo;
	private JScrollPane txArScrollPane;
	
	private Diagram diagram;
	private DiagramView diagramView;
	private JScrollPane diagramScrP;

	private JPopupMenu cmBox;
    
	private JMenuItem miProps;
	private JMenuItem miDelete;
	private JMenuItem miView;

	private JFileChooser chooser;
	
	private static final long serialVersionUID = 1L;
}